/** 
 *******************************************************************************
 * @file    bsp.h
 * @brief   Board Support Package External Reference Declaration. \n
 *          TMPM4KN Evaluation Board.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __BSP_H
#define __BSP_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "cg.h"
#include "port.h"
#include "except.h"
#include "dmac.h"
#include "t32a.h"
#include "uart.h"
#include "uart_ex.h"

#if defined ( __SES_ARM )    /* Segger Compiler */
#define __weak __attribute__((weak)) 
#endif

/**
 *  @addtogroup BSP
 *  @{
 */

/** 
 *  @addtogroup TMPM4KN_EVAL
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4KN_EVAL_Exported_macro TMPM4KN Evaluation Board Exported Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4KN_EVAL_Exported_define TMPM4KN Evaluation Board Exported Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4KN_EVAL_Exported_define
 *  @{
 */

/* no define */


/** 
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TMPM4KN_EVAL_Exported_define
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  PORT LED.
*/
/*----------------------------------*/
typedef enum
{
    BSP_LED_1 = 0,          /*!< PORT LED : D10 */
    BSP_LED_2,              /*!< PORT LED : D9  */
    BSP_LED_3,              /*!< PORT LED : D8 */
    BSP_LED_4,              /*!< PORT LED : D7 */
    /*>>> be set bottom */
    BSP_LED_MAX             /*!< max */
    /*<<< be set bottom */
} BSPLed;


/*----------------------------------*/
/** 
 * @brief  PORT Push SW.
*/
/*----------------------------------*/
typedef enum
{
    BSP_PSW_1 = 0,          /*!< PORT Push SW :S4 */
    BSP_PSW_2,              /*!< PORT Push SW :S5 */
    BSP_PSW_3,              /*!< PORT Push SW :S6 */
    BSP_PSW_4,              /*!< PORT Push SW :S7 */
    /*>>> be set bottom */
    BSP_PSW_MAX             /*!< max */
    /*<<< be set bottom */
} BSPPsw;

/*----------------------------------*/
/** 
 * @brief  PORT Slide SW.
*/
/*----------------------------------*/
typedef enum
{
    BSP_SSW_1 = 0,          /*!< PORT Slide SW :S4 */
    BSP_SSW_2,              /*!< PORT Slide SW :S5 */
    BSP_SSW_3,              /*!< PORT Slide SW :S6 */
    BSP_SSW_4,              /*!< PORT Slide SW :S7 */
    BSP_SSW_5,              /*!< PORT Slide SW :S9 */
    /*>>> be set bottom */
    BSP_SSW_MAX             /*!< max */
    /*<<< be set bottom */
} BSPSsw;

/*----------------------------------*/
/** 
 * @brief  USB UART.
*/
/*----------------------------------*/
typedef enum
{
    BSP_USB_UART_0 = 0,     /*!< USB UART 0 */
    BSP_USB_UART_1 = 1,     /*!< USB UART 1 */
    BSP_USB_UART_2 = 2,     /*!< USB UART 2 */
    BSP_USB_UART_3 = 3,     /*!< USB UART 3 */
    /*>>> be set bottom */
    BSP_USB_UART_MAX        /*!< max */
    /*<<< be set bottom */
} BSPUsbUart;

/*----------------------------------*/
/** 
 * @brief  ADC.
*/
/*----------------------------------*/
typedef enum
{
    BSP_ADC_0 = 0,  /*!< Adc 0 */
    /*>>> be set bottom */
    BSP_ADC_MAX     /*!< max */
    /*<<< be set bottom */
} BSPAdc;

/*----------------------------------*/
/** 
 * @brief  TSPI.
*/
/*----------------------------------*/
typedef enum
{
    BSP_TSPI_0 = 0,       /*!< TSPI 0 */
    BSP_TSPI_1 = 1,       /*!< TSPI 1 */
    /*>>> be set bottom */
    BSP_TSPI_MAX          /*!< max */
    /*<<< be set bottom */
} BSPTspi;

/*----------------------------------*/
/** 
 * @brief  Timer.
*/
/*----------------------------------*/
typedef enum
{
    BSP_TIMER_1MS = 0,      /*!< 1ms timer */
    BSP_TIMER_1S,           /*!< 1s timer  */
    /*>>> be set bottom */
    BSP_TIMER_MAX           /*!< max */
    /*<<< be set bottom */
} BSPTimer;

/*----------------------------------*/
/** 
 * @brief  Output Pulse.
*/
/*----------------------------------*/
typedef enum
{
    BSP_OUT_PULSE_SAMPLE_A = 0, /*!< pulse A */
    BSP_OUT_PULSE_SAMPLE_B,     /*!< pulse B */
    /*>>> be set bottom */
    BSP_OUT_PULSE_SAMPLE_MAX    /*!< max */
    /*<<< be set bottom */
} BSPOutputPulse;

/*----------------------------------*/
/** 
 * @brief  Timer Counter.
*/
/*----------------------------------*/
typedef enum
{
    BSP_COUNTER_SAMPLE_A = 0, /*!< pulse A */
    BSP_COUNTER_SAMPLE_B,     /*!< pulse B */
    /*>>> be set bottom */
    BSP_COUNTER_SAMPLE_MAX    /*!< max */
    /*<<< be set bottom */
} BSPCounter;



/** 
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/

/*----------------------------------*/
/** 
 * @brief  External Interrupt.
*/
/*----------------------------------*/
typedef enum
{
    BSP_EXINT_0 = 0,  /*!< low level */
    BSP_EXINT_1 = 1,  /*!<  high level */
    /*>>> be set bottom */
    BSP_EXINT_MAX     /*!< max */
    /*<<< be set bottom */
} BSPExInt;

/*----------------------------------*/
/** 
 * @brief  DNF Table Info.
*/
/*----------------------------------*/
typedef enum
{
    BSP_DNF_0 = 0,  /*!< DNF 0 */
    /*>>> be set bottom */
    BSP_DNF_MAX     /*!< max */
    /*<<< be set bottom */
} BSPDnf;

/**
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4KN_EVAL_Exported_typedef TMPM4KN Evaluation Board Exported Typedef
 *  @{
 */
/* No Define */
/**
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_typedef */
 
/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TMPM4KN_EVAL_Exported_functions TMPM4KN Evaluation Board Exported Functions
 *  @{
 */

void bsp_initialize(void);
void bsp_finalize(void);
gpio_t *bsp_get_gpio_instance_address(void);
/* led */
void bsp_get_gpio_led(BSPLed led, uint32_t *p_group, uint32_t *p_num);
gpio_gr_t bsp_get_gpio_group_led(BSPLed led);
gpio_num_t bsp_get_gpio_num_led(BSPLed led);
/* psw */
void bsp_get_gpio_psw(BSPPsw psw, uint32_t *p_group, uint32_t *p_num);
gpio_gr_t bsp_get_gpio_group_psw(BSPPsw psw);
gpio_num_t bsp_get_gpio_num_psw(BSPPsw psw);
/* ssw */
void bsp_get_gpio_ssw(BSPSsw ssw, uint32_t *p_group, uint32_t *p_num);
gpio_gr_t bsp_get_gpio_group_ssw(BSPSsw ssw);
gpio_num_t bsp_get_gpio_num_ssw(BSPSsw ssw);
/* usb uart */
uint32_t bsp_get_usb_uart_tx_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_dma_ch(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_tx_baudrate(BSPUsbUart uart);
uint32_t bsp_get_usb_uart_rx_baudrate(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_tx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_rx_nvic(BSPUsbUart uart);
IRQn_Type bsp_get_usb_uart_err_nvic(BSPUsbUart uart);
/* tspi */
uint32_t bsp_get_tspi_tx_ch(BSPTspi tspi);
uint32_t bsp_get_tspi_clk_ch(BSPTspi tspi);
uint32_t bsp_get_tspi_rx_ch(BSPTspi tspi);
IRQn_Type bsp_get_tspi_tx_nvic(BSPTspi tspi);
IRQn_Type bsp_get_tspi_rx_nvic(BSPTspi tspi);
IRQn_Type bsp_get_tspi_tx_err_nvic(BSPTspi tspi);
IRQn_Type bsp_get_tspi_rx_err_nvic(BSPTspi tspi);
uint32_t bsp_get_tspi_tx_dma_ch(BSPTspi tspi);
uint32_t bsp_get_tspi_rx_dma_ch(BSPTspi tspi);
/* timer */
void bsp_get_timer(BSPTimer timer, uint32_t *p_ch, uint32_t *p_type);
uint32_t bsp_get_timer_ch(BSPTimer timer);
uint32_t bsp_get_timer_type(BSPTimer timer);
IRQn_Type bsp_get_timer_nvic(BSPTimer timer);
/* output pulse */
uint32_t bsp_get_output_pulse_ch(BSPOutputPulse pulse);
uint32_t bsp_get_output_pulse_type(BSPOutputPulse pulse);
IRQn_Type bsp_get_output_pulse_nvic(BSPOutputPulse pulse);
/* adc */
uint32_t bsp_get_adc_adin(BSPAdc adc);
uint32_t bsp_get_adc_ch(BSPAdc adc);
uint32_t bsp_get_adc_dma_ch(BSPAdc adc);
/* external interrupt */
void bsp_irq_exint_enable(void);
void bsp_irq_exint_disable(void);
IRQn_Type bsp_get_exint_nvic(void);
/* dnf */
void bsp_irq_dnf_enable(void);
void bsp_irq_dnf_disable(void);
TSB_DNF_TypeDef * bsp_get_dnf_reg(BSPDnf dnf);
uint32_t bsp_get_dnf_int_num(BSPDnf dnf);
IRQn_Type bsp_get_dnf_nvic(BSPDnf dnf);


/*>>> Add to processing by user. */
extern void irq_systick(void);
extern void irq_psw(BSPPsw psw);
extern void irq_tspi_rx(BSPTspi tspi);
extern void irq_tspi_tx(BSPTspi tspi);
extern void irq_tspi_err(BSPTspi tspi);
extern void irq_usb_uart_rx(BSPUsbUart uart);
extern void irq_usb_uart_tx(BSPUsbUart uart);
extern void irq_usb_uart_err(BSPUsbUart uart);
extern void irq_uart4_rx(BSPUsbUart uart);
extern void irq_uart4_tx(BSPUsbUart uart);
extern void irq_uart4_err(BSPUsbUart uart);
extern void irq_timer(BSPTimer timer);
extern void irq_ppg(BSPOutputPulse ppg);
extern void irq_counter(BSPCounter cnt);
extern void irq_dmac_done(uint32_t ch);
extern void irq_dmac_err(void);
extern void irq_int17(void);
extern void irq_i2c(void);
extern void irq_tbe(void);
extern void irq_rbf(void);
extern void irq_adc_sgl(void);
extern void irq_adc_cnt(void);
extern void irq_adc_trig(void);
extern void irq_adc_cp0(void);
extern void irq_adc_cp1(void);

extern void irq_exint(BSPExInt exi);
/*<<< Add to processing by user. */

/** 
 *  @}
 */ /* End of group TMPM4KN_EVAL_Exported_functions */

/**
 *  @}
 */ /* End of group TMPM4KN_EVAL */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __BSP_H */

