/** 
 *******************************************************************************
 * @file    main.c
 * @brief   FLASH_Code Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_led.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "flash.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup FLASH_Code FLASH_Code Sample Appli
 *  @{
 */


/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_macro FLASH_Code Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FLASH_Code_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_define FLASH_Code Private Define
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(ms).
 *  @{
 */
#define CFG_LED_BLINK_CYCLE             ((uint32_t)(1000))

#define CFG_LED_BLINK_FRQ1               ((float)(0.5))
#define CFG_LED_BLINK_DUTY1              ((float)(0.5))
#define CFG_LED_ON_TIME1   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ1 * CFG_LED_BLINK_DUTY1))
#define CFG_LED_OFF_TIME1  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ1 * (1 - CFG_LED_BLINK_DUTY1)))
#define CFG_LED_BLINK_FRQ2               ((float)(0.5))
#define CFG_LED_BLINK_DUTY2              ((float)(0.5))
#define CFG_LED_ON_TIME2   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ2 * CFG_LED_BLINK_DUTY2))
#define CFG_LED_OFF_TIME2  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ2 * (1 - CFG_LED_BLINK_DUTY2)))
#define CFG_LED_BLINK_FRQ3               ((float)(0.5))
#define CFG_LED_BLINK_DUTY3              ((float)(0.5))
#define CFG_LED_ON_TIME3   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ3 * CFG_LED_BLINK_DUTY3))
#define CFG_LED_OFF_TIME3  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ3 * (1 - CFG_LED_BLINK_DUTY3)))
#define CFG_LED_BLINK_FRQ4               ((float)(0.5))
#define CFG_LED_BLINK_DUTY4              ((float)(0.5))
#define CFG_LED_ON_TIME4   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ4 * CFG_LED_BLINK_DUTY4))
#define CFG_LED_OFF_TIME4  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ4 * (1 - CFG_LED_BLINK_DUTY4)))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */

/** 
 *  @}
 */ /* End of group FLASH_Code_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_define FLASH_Code Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

#if defined ( __CC_ARM   )      /* RealView Compiler */
extern uint32_t Load$$FLASH_CODE_RAM_MAIN$$Base;
extern uint32_t Image$$FLASH_CODE_RAM_MAIN$$Base;
extern uint32_t Load$$FLASH_CODE_RAM_MAIN$$Length;
extern uint32_t Image$$FLASH_DEMO_A$$Base;
extern uint32_t Load$$FLASH_DEMO_B$$Base;
extern uint32_t Image$$RAM_DEMO_A$$Base;
extern uint32_t Image$$RAM_DEMO_B$$Base;
extern uint32_t Image$$FLASH_DEMO_A$$Length;
extern uint32_t Image$$FLASH_DEMO_B$$Length;
#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma section = "FLASH_CODE_RAM_MAIN"
#pragma section = "FLASH_CODE_ROM_MAIN"
#pragma section = "FLASH_DEMO_A"
#pragma section = "FLASH_DEMO_B"
#pragma section = "RAM_DEMO_A"
#pragma section = "RAM_DEMO_B"
#elif defined (__SES_ARM)       /* Segger Compiler */
extern unsigned char __FLASH_A_segment_used_start__[];
extern unsigned char __FLASH_A_segment_used_size__[];
extern unsigned char __FLASH_B_segment_used_start__[];
extern unsigned char __FLASH_B_segment_used_size__[];
extern unsigned char __RAM_A_segment_used_start__[];
extern unsigned char __RAM_B_segment_used_start__[];
#endif



#if defined ( __CC_ARM   )      /* RealView Compiler */
#define DEMO_A_FLASH            (uint32_t *)&Image$$FLASH_DEMO_A$$Base
#define DEMO_B_FLASH            (uint32_t *)&Load$$FLASH_DEMO_B$$Base

#define DEMO_A_RAM              (uint32_t *)&Image$$RAM_DEMO_A$$Base
#define DEMO_B_RAM              (uint32_t *)&Image$$RAM_DEMO_B$$Base

#define SIZE_DEMO_A               (uint32_t)&Image$$FLASH_DEMO_A$$Length
#define SIZE_DEMO_B               (uint32_t)&Image$$FLASH_DEMO_B$$Length

#elif defined ( __ICCARM__ )    /* IAR Compiler */
#define DEMO_A_FLASH            ((uint32_t *)__section_begin("FLASH_DEMO_A"))
#define DEMO_B_FLASH            ((uint32_t *)__section_begin("FLASH_DEMO_B"))

#define DEMO_A_RAM              ((uint32_t *)__section_begin("RAM_DEMO_A"))
#define DEMO_B_RAM              ((uint32_t *)__section_begin("RAM_DEMO_B"))

#define SIZE_DEMO_A             ((uint32_t)__section_size("FLASH_DEMO_A"))
#define SIZE_DEMO_B             ((uint32_t)__section_size("FLASH_DEMO_B"))
#elif defined (__SES_ARM)       /* Segger Compiler */
#define DEMO_A_FLASH            (uint32_t *)__FLASH_A_segment_used_start__
#define DEMO_B_FLASH            (uint32_t *)__FLASH_B_segment_used_start__

#define DEMO_A_RAM              (uint32_t *)__RAM_A_segment_used_start__
#define DEMO_B_RAM              (uint32_t *)__RAM_B_segment_used_start__

#define SIZE_DEMO_A             (uint32_t)__FLASH_A_segment_used_size__
#define SIZE_DEMO_B             (uint32_t)__FLASH_B_segment_used_size__
#endif

/** 
 *  @}
 */ /* End of group FLASH_Code_Private_define */

/** 
 *  @defgroup FLASH_Code_Exported_functions FLASH_Code Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  data a
  */
/*--------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
const char data_a[0x1000] __attribute__((section("DATA_A"))) =
#elif defined ( __ICCARM__ )    /* IAR Compiler */
const char data_a[0x1000] __attribute__((section("STR_A"))) =
#elif defined (__SES_ARM)       /* Segger Compiler */
const char data_a[0x1000] =
#endif    
{
    "Execute Program A \n"
};

/** 
 *  @defgroup FLASH_Code_Exported_functions FLASH_Code Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  data b
  */
/*--------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
const char data_b[0x1000] __attribute__((section("DATA_B"))) =
#elif defined ( __ICCARM__ )    /* IAR Compiler */
const char data_b[0x1000] __attribute__((section("STR_B"))) =
#elif defined (__SES_ARM)       /* Segger Compiler */
const char data_b[0x1000] =
#endif    
{
    "Execute Program B \n"
};

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_define FLASH_Code Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_RAMTRANSFER,   /*!< RAM transferring. */
    PROC_ERASING,       /*!< Erasing.          */
    PROC_REWRITING,     /*!< Rewriting.        */
    PROC_FINISHED,      /*!< Finished.         */
    PROC_ERROR,         /*!< Error.            */
    PROC_WRITE_ERROR,   /*!< Write Error.      */
    PROC_ERASE_ERROR    /*!< Erase Error.      */
} Processing;
/*----------------------------------*/
/** 
 * @brief  LED mode.
*/
/*----------------------------------*/
typedef enum
{
      LED_RUN,         /*!< Normal Mode    :RUNNING.   */
      LED_STOP,        /*!< Normal Mode    :END.       */
} LED_STATUS;
/** 
 *  @}
 */ /* End of group FLASH_Code_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_typedef FLASH_Code Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;                       /*!< Application :T32A.     */
    flash_t     flash;                      /*!< Application :flash.     */
    uart_io_t   uart;                       /*!< Application :USB UART. */
    sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    timer_t     timer;                      /*!< Application :Timer (1sec timer). */
    cg_t        cg;                         /*!< Driver      :CG.       */
} instance_t;

/**
 *  @}
 */ /* End of group FLASH_Code_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_variables FLASH_Code Private Variables
 *  @{
 */
volatile uint32_t LedPattern;
static instance_t instance;     /*!< Instance.                */
static uint32_t count[(uint32_t)BSP_LED_MAX] = {0U, 0U, 0U, 0U};
static uint32_t on[(uint32_t)BSP_LED_MAX] = {0U, 0U, 0U, 0U};
static uint32_t led_on_time[(uint32_t)BSP_LED_MAX] = {CFG_LED_ON_TIME1, CFG_LED_ON_TIME2, CFG_LED_ON_TIME3, CFG_LED_ON_TIME4};
static uint32_t led_off_time[(uint32_t)BSP_LED_MAX] = {CFG_LED_OFF_TIME1, CFG_LED_OFF_TIME2, CFG_LED_OFF_TIME3, CFG_LED_OFF_TIME4};
static volatile Processing proc;         /*!< Processing mode.         */
static LED_STATUS led_st = LED_RUN;

/**
 *  @}
 */ /* End of group FLASH_Code_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Private_fuctions FLASH_Code Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static void timer_interval_handler(uint32_t id);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size);
void copy_user_program(void);
TXZ_Result rewrite_user_program(void);
void (* startup) (void);

/*--------------------------------------------------*/
/** 
  * @brief  copy 32-bit data from source to dest
  * @param  dest
  * @param  source
  * @param  size
  * @retval None.
  * @note   -
  */
/*--------------------------------------------------*/
static void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size)
{
    uint32_t *dest_addr, *source_addr, tmpsize;
    uint32_t i, tmps, tmpd, mask;

    dest_addr = dest;
    source_addr = source;

    tmpsize = size >> 2U;
    for (i = 0U; i < tmpsize; i++) {    /* 32bits copy */
        *dest_addr = *source_addr;
        dest_addr++;
        source_addr++;
    }
    if (size & 0x00000003U) {   /* if the last data size is not 0(maybe 1,2 or 3), copy the last data */
        mask = 0xFFFFFF00U;
        i = size & 0x00000003U;
        tmps = *source_addr;
        tmpd = *dest_addr;
        while (i - 1U) {
            mask = mask << 8U;
            i--;
        }
        tmps = tmps & (~mask);
        tmpd = tmpd & (mask);
        *dest_addr = tmps + tmpd;       /* 32bits copy, but only change the bytes need to be changed */
    } else {
        /* Do nothing */
    }
}
/**
 *  @}
 */ /* End of group FLASH_Code_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Exported_functions FLASH_Code Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  copy user program.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void copy_user_program(void)
{
    Copy_Routine(DEMO_A_RAM, DEMO_A_FLASH, SIZE_DEMO_A);        /* copy A to RAM */
    Copy_Routine(DEMO_B_RAM, DEMO_B_FLASH, SIZE_DEMO_B);        /* copy B to RAM */
}

/*--------------------------------------------------*/
/** 
  * @brief  rewrite user program.
  * @param  -
  * @retval Result.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result rewrite_user_program(void)
{
    TXZ_Result retval = TXZ_SUCCESS;
    flash_t *p_flash = &instance.flash;

    p_flash->init.src_address = DEMO_B_RAM;
    p_flash->init.dst_address = DEMO_A_FLASH;
    p_flash->init.size = SIZE_DEMO_B;

    if(fc_write_code_flash(p_flash) == TXZ_ERROR){
        retval = TXZ_ERROR;
    }
    else {
        p_flash->init.src_address = DEMO_A_RAM;
        p_flash->init.dst_address = DEMO_B_FLASH;
        p_flash->init.size = SIZE_DEMO_A;
        if(fc_write_code_flash(p_flash) == TXZ_ERROR){
            retval = TXZ_ERROR;
        }
        else {
            /* do nothing */
        }
    }

    return (retval);

}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.uart.init.p_reg    = MAIN_NULL;
    instance.t32a.p_instance    = MAIN_NULL;
    instance.flash.p_instance   = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.cg.p_instance      = MAIN_NULL;
    {
        uint32_t i;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            instance.sw[i].init.p_gpio  = MAIN_NULL;
        }
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            instance.led[i].init.p_gpio = MAIN_NULL;
        }
    }
    /* Processing mode. */
    proc = PROC_IDLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; (i < (uint32_t)BSP_PSW_MAX) && (proc != PROC_ERROR); i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                case BSP_PSW_1:
                    proc = PROC_RAMTRANSFER;
                    break;
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    int32_t i;
    led_t *p_led = MAIN_NULL;
    /*----------------------*/
    /* SW                   */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_1ms_timer_handler(&instance.sw[i]);
    }

    if(led_st != LED_STOP){
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            if(count[i] < led_on_time[i]) {   /* 1ms * 1000 = 1sec LED on */
                count[i]++;
                on[i] = 1;
            }else if((count[i] >= led_on_time[i]) && (count[i] < (led_on_time[i] + led_off_time[i]))){  /* 1ms * 1000 = 1sec LED off */
                count[i]++;
                on[i] = 0;
            }else{
                count[i] = 0;
            }
            if(on[i] == 1){
                if((i == BSP_LED_1) || (i == BSP_LED_3))
                {
                    p_led = &instance.led[i];
                    if(LedPattern == 0) {
                        led_turn_on(p_led);
                    }
                    else {
                        led_turn_off(p_led);
                    }
                }
                else {
                    p_led = &instance.led[i];
                    if(LedPattern == 0) {
                        led_turn_off(p_led);
                    }
                    else {
                        led_turn_on(p_led);
                    }
                }
            }else{
                p_led = &instance.led[i];
                led_turn_off(p_led);
            }
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
    /*----------------------*/
    /* Flash                */
    /*----------------------*/
    {
        flash_t *p_flash = &instance.flash;

        /*--- Construct  ---*/
        /* Register Allocation */
        p_flash->p_instance = TSB_FC;
        /*--- Initialize ---*/
        p_flash->init.status = FC_SR0_RDYBSY;
        p_flash->init.num_of_pages_code = 1;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;
        
        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state               = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_t *param = &instance.uart;
        param->init.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param->init.p_reg  = TSB_UART0;
            break;
        case 1:
            param->init.p_reg  = TSB_UART1;
            break;
        case 2:
            param->init.p_reg  = TSB_UART2;
            break;
        default:
            param->init.p_reg  = MAIN_NULL;
            break;
        }
        param->init.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param->init.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param->init.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param->init.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param->init.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param->init);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_finalize(&instance.sw[i]);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    if (instance.uart.init.p_reg != MAIN_NULL)
    {
        uart_io_finalize();
    }

    return (result);
}

/**
 *  @}
 */ /* End of group FLASH_Code_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Code_Exported_functions FLASH_Code Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t i;
    char cmp_data[] = "Execute Program A \n";
    cg_t *p_cg = &instance.cg;

    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    p_cg->p_instance = TSB_CG;
    cg_set_warm_up_time(p_cg, CG_WARM_UP_SRC_IHOSC1, WU_TIME_INT);
    cg_ihosc1_enable();            /* internal HOSC enable */
    cg_start_warmup();
    cg_wait_warmup();
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        case PROC_IDLE:
            if(memcmp(cmp_data, data_a, sizeof(cmp_data)) == 0) {
                LedPattern = 0;
            }
            else {
                LedPattern = 1;
            }
            printf("%s",(const char *)&data_a[0]);
            printf("Please press the SW1 \n\n");
            timer_start(&instance.timer);
            proc = PROC_START;
            break;
        case PROC_START:
            /*----------------------*/
            /* SW                   */
            /*----------------------*/
            for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
            {
                 sw_task(&instance.sw[i]);
            }
            break;
        case PROC_RAMTRANSFER:
            printf("RAM transferring \n");
            timer_stop(&instance.timer);
            copy_user_program();
            proc = PROC_ERASING;
            break;
        case PROC_ERASING:
            instance.flash.init.first_page_code = FC_CODE_FLASH_PAGE8;
            if(fc_erase_page_code_flash(&instance.flash) == TXZ_ERROR){
                proc = PROC_ERASE_ERROR;
            }
            else {
                instance.flash.init.first_page_code = FC_CODE_FLASH_PAGE16;
                if(fc_erase_page_code_flash(&instance.flash) == TXZ_ERROR){
                    proc = PROC_ERASE_ERROR;
                }
                else {
                    printf("Erasing. \n");
                    proc = PROC_REWRITING;
                }
            }
            break;
        case PROC_REWRITING:
            if(rewrite_user_program() == TXZ_ERROR){
                proc = PROC_WRITE_ERROR;
            }
            else {
                printf("Rewriting. \n");
                proc = PROC_FINISHED;
            }
            break;
        case PROC_FINISHED:
            printf("Finished. \n");
            proc = PROC_IDLE;
            break;
        case PROC_WRITE_ERROR:
            printf("Writing Error!! \n");
            proc = PROC_IDLE;
            break;
        case PROC_ERASE_ERROR:
            printf("Erasing Error!! \n");
            proc = PROC_IDLE;
            break;
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
    /*--- usb uart 1 ---*/
    case BSP_USB_UART_1:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group FLASH_Code_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH_Code */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
