/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART_TRANS Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_led.h"
#include "bsp_timer.h"
#include "bsp_uart_io.h"
#include "bsp_uart_io_dma.h"
#include "sys_timer.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART_TRANS UART_TRANS Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_macro UART_TRANS Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group UART_TRANS_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_define UART_TRANS Private Define
 *  @{
 */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */

/*----------------------*/
/*  DMA Setting         */
/*----------------------*/
/* Work Area = ((The total number of the used channel) * (4 * DWORD) * 2)  */
#define CFG_DMA_WORK_SIZE     ((uint32_t)(DMA_CH_NUM * 4 * sizeof(uint32_t) * 2))     /*!< DMA work area. */

/** 
 *  @}
 */ /* End of group UART_TRANS_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_define UART_TRANS Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */
#define USER_ID_UART            ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

#define CFG_DMA_ch_dstinc_1byte               0x00000000 ; /*!< dstinc 1 byte */
#define CFG_DMA_ch_dstinc_2byte               0x00000001 ; /*!< dstinc 2 byte */
#define CFG_DMA_ch_dstinc_4byte               0x00000010 ; /*!< dstinc 4 byte */
#define CFG_DMA_ch_dstinc_none                0x00000011 ; /*!< dstinc none */
#define CFG_DMA_ch_dstsize_1byte              0x00000000 ; /*!< dstsize 1 byte */
#define CFG_DMA_ch_dstsize_2byte              0x00000001 ; /*!< dstsize 2 byte */
#define CFG_DMA_ch_dstsize_4byte              0x00000010 ; /*!< dstsize 4 byte */
#define CFG_DMA_ch_srcinc_1byte               0x00000000 ; /*!< srcinc 1 byte */
#define CFG_DMA_ch_srcinc_2byte               0x00000001 ; /*!< srcinc 2 byte */
#define CFG_DMA_ch_srcinc_4byte               0x00000010 ; /*!< srcinc 4 byte */
#define CFG_DMA_ch_srcinc_none                0x00000011 ; /*!< srcinc none */
#define CFG_DMA_ch_srcsize_1byte              0x00000000 ; /*!< srcsize 1 byte */
#define CFG_DMA_ch_srcsize_2byte              0x00000001 ; /*!< srcsize 2 byte */
#define CFG_DMA_ch_srcsize_4byte              0x00000010 ; /*!< srcsize 4 byte */
#define CFG_DMA_ch_rpow_001                   0x00000000 ; /*!< R power 1 */
#define CFG_DMA_ch_rpow_002                   0x00000001 ; /*!< R power 2 */
#define CFG_DMA_ch_rpow_004                   0x00000010 ; /*!< R power 4 */
#define CFG_DMA_ch_rpow_008                   0x00000011 ; /*!< R power 8 */
#define CFG_DMA_ch_rpow_016                   0x00000100 ; /*!< R power 16 */
#define CFG_DMA_ch_rpow_032                   0x00000101 ; /*!< R power 32 */
#define CFG_DMA_ch_rpow_064                   0x00000110 ; /*!< R power 64 */
#define CFG_DMA_ch_rpow_128                   0x00000111 ; /*!< R power 128 */
#define CFG_DMA_ch_rpow_256                   0x00001000 ; /*!< R power 256 */
#define CFG_DMA_ch_rpow_512                   0x00001001 ; /*!< R power 512 */
#define CFG_DMA_ch_rpow_none                  0x00001111 ; /*!< R power none */
#define CFG_DMA_ch_next_usesingle             0x00000000 ; /*!< next use single */
#define CFG_DMA_ch_next_useburst              0x00000001 ; /*!< next use burst  */

#define MCU_NAME        "TMPM4KNFYA\n"
/** 
 *  @}
 */ /* End of group UART_TRANS_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_define UART_TRANS Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_START,             /*!< Start.                       */
    PROC_OUTPUT_MSG,        /*!< Output message.              */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;
/** 
 *  @}
 */ /* End of group UART_TRANS_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_typedef UART_TRANS Private Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   t32a_t      t32a;                       /*!< Driver      :T32A               */
   timer_t     timer;                      /*!< Application :Timer (1ms timer). */
    dma_t         dma;       /*!< Driver      :DMA.               */
    sw_t          sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
    led_t         led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    uart_io_dma_t uart_dma;   /*!< Application :USB UART.          */
    uart_io_t        uart;
} instance_t;

/*----------------------------------*/
/** 
 * @brief  Input data information.
*/
/*----------------------------------*/
/**
 *  @}
 */ /* End of group UART_TRANS_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_variables UART_TRANS Private Variables
 *  @{
 */
static instance_t instance;  /*!< Instance.               */
static volatile Processing proc;      /*!< Processing mode.        */
static uint8_t led_flag;     /*!< led flag. */
static uint8_t mcu_name[] = MCU_NAME;
static uint8_t data[sizeof( MCU_NAME )];
#ifdef DMA
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static __align(0x400) uint8_t primary[CFG_DMA_WORK_SIZE]; /*!< DMA work area. */
#elif defined ( __ICCARM__ )    /*IAR Compiler */
    #pragma location="dma_work"
    #pragma data_alignment=1024
    static uint8_t primary[CFG_DMA_WORK_SIZE]; /*!< DMA work area. */
#elif defined ( __SES_ARM )     /* Segger Compiler */
    static uint8_t __attribute__((__aligned__(0x400)))primary[CFG_DMA_WORK_SIZE];
#endif
#endif
/**
 *  @}
 */ /* End of group UART_TRANS_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Private_fuctions UART_TRANS Private Fuctions
 *  @{
 */
static void variable_initialize(void);
#ifdef DMA
static TXZ_Result driver_dma_initialize(void);
static TXZ_Result driver_dma_finalize(void);
#endif
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static TXZ_Result uart_initialize(void);
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.uart.init.p_reg    = MAIN_NULL;
    {
        uint32_t i;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            instance.sw[i].init.p_gpio  = MAIN_NULL;
        }
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            instance.led[i].init.p_gpio = MAIN_NULL;
        }
    }
#ifdef DMA
    uint32_t i;

    /* Instance. */
    instance.uart_dma.init.p_reg = MAIN_NULL;
    instance.dma.p_instance     = MAIN_NULL;
    /* DMA work area. */
    for (i=0; i<CFG_DMA_WORK_SIZE; i++)
    {
        primary[i] = 0;
    }
#endif
    /* Processing mode. */
    proc = PROC_START;
    /* led flag. */
    led_flag = 1;
    {
        uint8_t i;

        for (i = 0; i < (sizeof( MCU_NAME )); i++)
        {
            data[i] = mcu_name[i];
        }
    }
}

#ifdef DMA
/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_dma_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Construct (Register Allocation) ---*/
    instance.dma.p_instance = TSB_DMAA;

    /*--- Initialize ---*/
    instance.dma.init.baseAdd = (uint32_t)&primary[0];
    if (dma_init(&instance.dma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_dma_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*--- Finalize ---*/
    if (dma_deinit(&instance.dma) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Destruct (Register Release) ---*/
    /* Register Allocation */
    instance.dma.p_instance = MAIN_NULL;

    return (result);
}
#endif

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
#ifdef DMA
    /*----------------------*/
    /* DMA                  */
    /*----------------------*/
    if (driver_dma_initialize() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
#endif

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */
#ifdef DMA
    /*----------------------*/
    /* DMA                  */
    /*----------------------*/
    if (driver_dma_finalize() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
#endif
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id          = (uint32_t)p_led;
            p_led->init.p_gpio      = bsp_get_gpio_instance_address();
            p_led->init.port.group  = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num    = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.state               = LED_STATE_OFF;
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

#ifdef NODMAC
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /*----------------------*/
    if (instance.uart.init.p_reg != MAIN_NULL)
    {
        uart_io_finalize();
    }
#endif
#ifdef DMA
    /*----------------------*/
    /* USB UART             */
    /*----------------------*/
    if (instance.uart_dma.init.p_reg != MAIN_NULL)
    {
        uart_io_dma_finalize(&instance.uart_dma);
        NVIC_DisableIRQ(INTDMAATC_IRQn);
        NVIC_DisableIRQ(INTDMAAERR_IRQn);
    }
#endif
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_finalize(&instance.sw[i]);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }
    /*----------------------*/
    /* 1ms Timer Stop       */
    /*----------------------*/
    timer_stop(&instance.timer);
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    timer_finalize(&instance.timer);

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        uint32_t i;

        /*----------------------*/
        /* Tick                 */
        /*----------------------*/
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /*----------------------*/
        /* SW                   */
        /*----------------------*/
        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            sw_1ms_timer_handler(&instance.sw[i]);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; (i < (uint32_t)BSP_PSW_MAX) && (proc != PROC_OUTPUT_MSG); i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                case BSP_PSW_1:
                    proc = PROC_OUTPUT_MSG;
                    break;
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  UART1 Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result uart_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

#ifdef NODMAC
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Tx use same UART channel. */
    {
        uart_io_t *param = &instance.uart;
        param->init.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param->init.p_reg  = TSB_UART0;
            break;
        case 1:
            param->init.p_reg  = TSB_UART1;
            break;
        case 2:
            param->init.p_reg  = TSB_UART2;
            break;
        default:
            param->init.p_reg  = MAIN_NULL;
            break;
        }
        param->init.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param->init.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param->init.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param->init.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param->init.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param->init);
        }
        else
        {
            result = TXZ_ERROR;
        }

        return (result);
    }
#endif
#ifdef DMA
    /*----------------------*/
    /* Uart DMA             */
    /*----------------------*/
    /* Tx are same channel */
    {
        uart_io_dma_t *param = &instance.uart_dma;
        param->init.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param->init.p_reg  = TSB_UART0;
            break;
        case 1:
            param->init.p_reg  = TSB_UART1;
            break;
        case 2:
            param->init.p_reg  = TSB_UART2;
            break;
        default:
            param->init.p_reg  = MAIN_NULL;
            break;
        }
        param->init.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param->init.dma.p_dma  = &instance.dma;
        param->init.dma.tx     = bsp_get_usb_uart_tx_dma_ch(BSP_USB_UART_0);
        param->init.dma.rx     = bsp_get_usb_uart_rx_dma_ch(BSP_USB_UART_0);

        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.dstInc    = CFG_DMA_ch_dstinc_1byte;
        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.dstSize   = CFG_DMA_ch_dstsize_1byte;
        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.srcInc    = CFG_DMA_ch_srcinc_none;
        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.srcSize   = CFG_DMA_ch_srcsize_1byte;
        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.arbitNum  = CFG_DMA_ch_rpow_001;
        param->init.dma.p_dma->ch[param->init.dma.rx].cfg.burst     = CFG_DMA_ch_next_useburst;
    
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.dstInc    = CFG_DMA_ch_dstinc_none;
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.dstSize   = CFG_DMA_ch_dstsize_1byte;
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.srcInc    = CFG_DMA_ch_srcinc_1byte;
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.srcSize   = CFG_DMA_ch_srcsize_1byte;
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.arbitNum  = CFG_DMA_ch_rpow_001;
        param->init.dma.p_dma->ch[param->init.dma.tx].cfg.burst     = CFG_DMA_ch_next_useburst;

        if (param->init.p_reg != MAIN_NULL)
        {
            param  = uart_io_dma_initialize(&param->init);
        }
        else
        {
            result = TXZ_ERROR;
        }
        return (result);
    }
#endif
}

/**
 *  @}
 */ /* End of group UART_TRANS_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_TRANS_Exported_functions UART_TRANS Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- uart ---*/
    if (uart_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*----------------------*/
    /* 1ms Timer Start      */
    /*----------------------*/
    timer_start(&instance.timer);

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch(proc)
        {
        /*--- Start ---*/
        case PROC_START:
            {
                uint32_t i;
                /*----------------------*/
                /* SW                   */
                /*----------------------*/
                for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
                {
                    sw_task(&instance.sw[i]);
                }
            }
            break;
        /*--- Output "Message" ---*/
        case PROC_OUTPUT_MSG:
            {
                led_t *p_led1 = &instance.led[1];
                led_t *p_led2 = &instance.led[2];
#ifdef DMA
                NVIC_EnableIRQ(INTDMAATC_IRQn);
                NVIC_EnableIRQ(INTDMAAERR_IRQn);
#endif
                /* Output message. */
                if (printf("%s",(const char *)&data[0]) != -1 )
                {
                    proc = PROC_START;
                    if(led_flag == 1){
                        led_turn_on(p_led1);
                        led_flag = 0;
                    }
                    else{
                        led_turn_off(p_led1);
                        led_flag = 1;
                    }
                    led_turn_off(p_led2);
                }
                else
                {
                    proc = PROC_OUTPUT_ERR;
                }
            }
            break;
        /*--- Output "Error Message" ---*/
        case PROC_OUTPUT_ERR:
            {
                led_t *p_led2 = &instance.led[2];
                led_turn_on(p_led2);
                proc = PROC_START;
            }
            break;
        /*--- Error ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- Default ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif

    return (0);
}

#ifdef NODMAC
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
        /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}
#endif

#ifdef DMA
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Call the DMA driver's handler. */
    dma_irq_handler(&instance.dma, ch);
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Call the DMA driver's handler. */
    dma_error_irq_handler(&instance.dma);
}
#endif

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group UART_TRANS_Exported_functions */

/**
 *  @}
 */ /* End of group UART_TRANS */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
