/** 
 *******************************************************************************
 * @file    cg_m4k2_e.h
 * @brief   This file provides all the functions prototypes for CG driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __CG_M4K2_H
#define __CG_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup CG CG
 *  @brief    CG Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/

#define AVAILABLE               (1)                         /*!< CG RLM function equipped */
#define UNAVAILABLE             (0)                         /*!< CG RLM function not equipped */

#define FSYSM_EQUIPPED                  AVAILABLE
#define IHOSC2_EQUIPPED                 AVAILABLE
#define STOP2_EQUIPPED                  UNAVAILABLE
#define SCOUT_EQUIPPED                  UNAVAILABLE
#define LOW_WARMUP_EQUIPPED             UNAVAILABLE
#define FSYSMENA_EQUIPPED               AVAILABLE
#define FSYSMENB_EQUIPPED               AVAILABLE
#define FSYSMENC_EQUIPPED               UNAVAILABLE
#define FSYSENA_EQUIPPED                AVAILABLE
#define FSYSENB_EQUIPPED                UNAVAILABLE
#define EXTEND2_EQUIPPED                UNAVAILABLE
#define RLM_LOSCCR_EQUIPPED             UNAVAILABLE
#define RLM_LOSCCR_IHOSC2_EQUIPPED      UNAVAILABLE
#define RLM_SHTDNOP_PROTECT_EQUIPPED    UNAVAILABLE
#define FC_FCRACCR_EQUIPPED             UNAVAILABLE


/* Define clocks */
#define IOSC_10M                (10000000UL)

#define EOSC_6M                 (6000000UL)
#define EOSC_8M                 (8000000UL)
#define EOSC_10M                (10000000UL)
#define EOSC_12M                (12000000UL)
#define EOSC_16M                (16000000UL)
#define EOSC_20M                (20000000UL)
#define EOSC_24M                (24000000UL)

#define EXTALH                  EOSC_10M                    /* External high-speed oscillator freq */
#define IXTALH                  IOSC_10M                    /* Internal high-speed oscillator freq */

#define CG_6M_MUL_26_656_FPLL       (0x1C153500UL)      /* fPLL = 6MHz * 26.656   */
#define CG_8M_MUL_20_FPLL           (0x24502800UL)      /* fPLL = 8MHz * 20       */
#define CG_10M_MUL_16_FPLL          (0x2E902000UL)      /* fPLL = 10MHz * 16       */
#define CG_12M_MUL_13_312_FPLL      (0x36DA1A00UL)      /* fPLL = 12MHz * 13.312   */

#define EOSC_6M_DIV4_PLLON      (79970000UL)    /* 6.00MHz  * 53.3125 / 4 */
#define EOSC_8M_DIV4_PLLON      (80000000UL)    /* 8.00MHz  * 40.0000 / 4 */
#define EOSC_10M_DIV4_PLLON     (80000000UL)    /* 10.00MHz * 32.0000 / 4 */
#define EOSC_12M_DIV4_PLLON     (79880000UL)    /* 12.00MHz * 26.6250 / 4 */
#define EOSC_6M_DIV8_PLLON      (39980000UL)    /* 6.00MHz  * 53.3125 / 8 */
#define EOSC_8M_DIV8_PLLON      (40000000UL)    /* 8.00MHz  * 40.0000 / 8 */
#define EOSC_10M_DIV8_PLLON     (40000000UL)    /* 10.00MHz * 32.0000 / 8 */
#define EOSC_12M_DIV8_PLLON     (39940000UL)    /* 12.00MHz * 26.6250 / 8 */
#define EOSC_6M_DIV2_PLLON      (159900000UL)   /* 6.00MHz  * 53.3125 / 2 */
#define EOSC_8M_DIV2_PLLON      (160000000UL)   /* 8.00MHz  * 40.0000 / 2 */
#define EOSC_10M_DIV2_PLLON     (160000000UL)   /* 10.00MHz * 32.0000 / 2 */
#define EOSC_12M_DIV2_PLLON     (159800000UL)   /* 12.00MHz * 26.6250 / 2 */

#define IOSC_10M_DIV4_PLLON     (80000000UL)    /* 10.00MHz * 32.0000 / 4 */
#define IOSC_10M_DIV8_PLLON     (40000000UL)    /* 10.00MHz * 32.0000 / 8 */
#define IOSC_10M_DIV2_PLLON     (160000000UL)   /* 10.00MHz * 32.0000 / 2 */

#if (FC_FCRACCR_EQUIPPED == AVAILABLE)
#define FC_KCR_KEYCODE          (0xA74A9D23UL)  /*!< The specific code that writes the FCKCR register. */

#define FIXED_80MHz             (80000000UL)    /* 80.00MHz */

#define FC_RACCR_THRESHOLD      FIXED_80MHz
#define FC_RACCR_RAMLC_1CLK     (0x00000000UL)  /*!< 1clock(less than FC_RACCR_THRESHOLD MHZ) */
#define FC_RACCR_RAMLC_2CLK     (0x00000010UL)  /*!< 2clock(more than FC_RACCR_THRESHOLD MHZ).*/
#endif

/* Configure Warm-up time */
#define HZ_1M                   (1000000UL)
#define WU_TIME_EXT             (5000UL)                    /* warm-up time for EXT is 5ms   */
#define WU_TIME_INT             (1634UL)                    /* warm-up time for INT is 163.4us x 10 */
#define WU_TIME_INT_MIN         (0x03C << 20U)              /* minimum warm-up time for INT */
#define CGWUPHCR_WUEF_R_DONE    ((uint32_t)0x00000000)      /*!< WUEF    :[R]    :Done    */

/* Warm-up setting value */
/* EOSC OSCCR<WUPT11:0> = (warm-up time(us) * EXTALH - 16) / 16 */
#define WUPHCR_WUPT_EXT         ((uint32_t)(((((uint64_t)WU_TIME_EXT * EXTALH / HZ_1M) - 16UL) /16UL) << 20U))
/* IOSC OSCCR<WUPT11:0> = ((warm-up time(us) - 63.3us) * IXTALH - 41) / 16 */
#define WUPHCR_WUPT_INT         ((uint32_t)(((((uint64_t)(WU_TIME_INT - 633UL) * (IXTALH / 10) / HZ_1M) - 41UL) / 16UL) << 20U))


#if defined(TMPM4KNA)
    #define CG_FC_PERIPH_PORTA      (0x00000001UL << 0)     /* Port A    */
    #define CG_FC_PERIPH_PORTB      (0x00000001UL << 1)     /* Port B    */
    #define CG_FC_PERIPH_PORTC      (0x00000001UL << 2)     /* Port C    */
    #define CG_FC_PERIPH_PORTD      (0x00000001UL << 3)     /* Port D    */
    #define CG_FC_PERIPH_PORTE      (0x00000001UL << 4)     /* Port E    */
    #define CG_FC_PERIPH_PORTF      (0x00000001UL << 5)     /* Port F    */
    #define CG_FC_PERIPH_PORTG      (0x00000001UL << 6)     /* Port G    */
    #define CG_FC_PERIPH_PORTH      (0x00000001UL << 7)     /* Port H    */
    #define CG_FC_PERIPH_PORTJ      (0x00000001UL << 8)     /* Port J    */
    #define CG_FC_PERIPH_PORTK      (0x00000001UL << 9)     /* Port K    */
    #define CG_FC_PERIPH_PORTL      (0x00000001UL << 10)    /* Port L    */
    #define CG_FC_PERIPH_PORTM      (0x00000001UL << 11)    /* Port M    */
    #define CG_FC_PERIPH_PORTN      (0x00000001UL << 12)    /* Port N    */
    #define CG_FC_PERIPH_PORTU      (0x00000001UL << 16)    /* Port U    */
    #define CG_FC_PERIPH_PORTV      (0x00000001UL << 17)    /* Port V    */
    #define CG_FC_PERIPH_TSPI0      (0x00000001UL << 19)    /* TSPI 0    */
    #define CG_FC_PERIPH_TSPI1      (0x00000001UL << 20)    /* TSPI 1    */
    #define CG_FC_PERIPH_UART0      (0x00000001UL << 21)    /* UART 0    */
    #define CG_FC_PERIPH_UART1      (0x00000001UL << 22)    /* UART 1    */
    #define CG_FC_PERIPH_UART2      (0x00000001UL << 23)    /* UART 2    */
    #define CG_FC_PERIPH_UART3      (0x00000001UL << 24)    /* UART 3    */
    #define CG_FC_PERIPH_I2C0       (0x00000001UL << 25)    /* I2C 0     */
    #define CG_FC_PERIPH_I2C1       (0x00000001UL << 26)    /* I2C 1     */
    #define CG_FC_PERIPH_T32A0      (0x00000001UL << 28)    /* T32A 0    */
    #define CG_FC_PERIPH_T32A1      (0x00000001UL << 29)    /* T32A 1    */
    #define CG_FC_PERIPH_T32A2      (0x00000001UL << 30)    /* T32A 2    */
    #define CG_FC_PERIPH_T32A3      (0x00000001UL << 31)    /* T32A 3    */
    #define CG_FSYSMENA_ALL_ENABLE  (0xF7FB1FFFUL)          /*!< CG FSYSMENA ALL ENABLE */
#endif
#if defined(TMPM4KMA)
    #define CG_FC_PERIPH_PORTA      (0x00000001UL << 0)     /* Port A    */
    #define CG_FC_PERIPH_PORTB      (0x00000001UL << 1)     /* Port B    */
    #define CG_FC_PERIPH_PORTC      (0x00000001UL << 2)     /* Port C    */
    #define CG_FC_PERIPH_PORTE      (0x00000001UL << 4)     /* Port E    */
    #define CG_FC_PERIPH_PORTF      (0x00000001UL << 5)     /* Port F    */
    #define CG_FC_PERIPH_PORTG      (0x00000001UL << 6)     /* Port G    */
    #define CG_FC_PERIPH_PORTH      (0x00000001UL << 7)     /* Port H    */
    #define CG_FC_PERIPH_PORTJ      (0x00000001UL << 8)     /* Port J    */
    #define CG_FC_PERIPH_PORTK      (0x00000001UL << 9)     /* Port K    */
    #define CG_FC_PERIPH_PORTL      (0x00000001UL << 10)    /* Port L    */
    #define CG_FC_PERIPH_PORTN      (0x00000001UL << 12)    /* Port N    */
    #define CG_FC_PERIPH_PORTU      (0x00000001UL << 16)    /* Port U    */
    #define CG_FC_PERIPH_TSPI0      (0x00000001UL << 19)    /* TSPI 0    */
    #define CG_FC_PERIPH_TSPI1      (0x00000001UL << 20)    /* TSPI 1    */
    #define CG_FC_PERIPH_UART0      (0x00000001UL << 21)    /* UART 0    */
    #define CG_FC_PERIPH_UART1      (0x00000001UL << 22)    /* UART 1    */
    #define CG_FC_PERIPH_UART2      (0x00000001UL << 23)    /* UART 2    */
    #define CG_FC_PERIPH_UART3      (0x00000001UL << 24)    /* UART 3    */
    #define CG_FC_PERIPH_I2C0       (0x00000001UL << 25)    /* I2C 0     */
    #define CG_FC_PERIPH_I2C1       (0x00000001UL << 26)    /* I2C 1     */
    #define CG_FC_PERIPH_T32A0      (0x00000001UL << 28)    /* T32A 0    */
    #define CG_FC_PERIPH_T32A1      (0x00000001UL << 29)    /* T32A 1    */
    #define CG_FC_PERIPH_T32A2      (0x00000001UL << 30)    /* T32A 2    */
    #define CG_FC_PERIPH_T32A3      (0x00000001UL << 31)    /* T32A 3    */
    #define CG_FSYSMENA_ALL_ENABLE  (0xF7F917F7UL)          /*!< CG FSYSMENA ALL ENABLE */
#endif
#if defined(TMPM4KLA)
    #define CG_FC_PERIPH_PORTA      (0x00000001UL << 0)     /* Port A    */
    #define CG_FC_PERIPH_PORTB      (0x00000001UL << 1)     /* Port B    */
    #define CG_FC_PERIPH_PORTC      (0x00000001UL << 2)     /* Port C    */
    #define CG_FC_PERIPH_PORTE      (0x00000001UL << 4)     /* Port E    */
    #define CG_FC_PERIPH_PORTF      (0x00000001UL << 5)     /* Port F    */
    #define CG_FC_PERIPH_PORTG      (0x00000001UL << 6)     /* Port G    */
    #define CG_FC_PERIPH_PORTH      (0x00000001UL << 7)     /* Port H    */
    #define CG_FC_PERIPH_PORTJ      (0x00000001UL << 8)     /* Port J    */
    #define CG_FC_PERIPH_PORTK      (0x00000001UL << 9)     /* Port K    */
    #define CG_FC_PERIPH_PORTL      (0x00000001UL << 10)    /* Port L    */
    #define CG_FC_PERIPH_PORTU      (0x00000001UL << 16)    /* Port U    */
    #define CG_FC_PERIPH_TSPI0      (0x00000001UL << 19)    /* TSPI 0    */
    #define CG_FC_PERIPH_TSPI1      (0x00000001UL << 20)    /* TSPI 1    */
    #define CG_FC_PERIPH_UART0      (0x00000001UL << 21)    /* UART 0    */
    #define CG_FC_PERIPH_UART1      (0x00000001UL << 22)    /* UART 1    */
    #define CG_FC_PERIPH_UART2      (0x00000001UL << 23)    /* UART 2    */
    #define CG_FC_PERIPH_I2C0       (0x00000001UL << 25)    /* I2C 0     */
    #define CG_FC_PERIPH_I2C1       (0x00000001UL << 26)    /* I2C 1     */
    #define CG_FC_PERIPH_T32A0      (0x00000001UL << 28)    /* T32A 0    */
    #define CG_FC_PERIPH_T32A1      (0x00000001UL << 29)    /* T32A 1    */
    #define CG_FC_PERIPH_T32A2      (0x00000001UL << 30)    /* T32A 2    */
    #define CG_FC_PERIPH_T32A3      (0x00000001UL << 31)    /* T32A 3    */
    #define CG_FSYSMENA_ALL_ENABLE  (0xF6F907F7UL)          /*!< CG FSYSMENA ALL ENABLE */
#endif
#if defined(TMPM4KHA)
    #define CG_FC_PERIPH_PORTA      (0x00000001UL << 0)     /* Port A    */
    #define CG_FC_PERIPH_PORTB      (0x00000001UL << 1)     /* Port B    */
    #define CG_FC_PERIPH_PORTC      (0x00000001UL << 2)     /* Port C    */
    #define CG_FC_PERIPH_PORTE      (0x00000001UL << 4)     /* Port E    */
    #define CG_FC_PERIPH_PORTF      (0x00000001UL << 5)     /* Port F    */
    #define CG_FC_PERIPH_PORTG      (0x00000001UL << 6)     /* Port G    */
    #define CG_FC_PERIPH_PORTH      (0x00000001UL << 7)     /* Port H    */
    #define CG_FC_PERIPH_PORTK      (0x00000001UL << 9)     /* Port K    */
    #define CG_FC_PERIPH_PORTL      (0x00000001UL << 10)    /* Port L    */
    #define CG_FC_PERIPH_PORTU      (0x00000001UL << 16)    /* Port U    */
    #define CG_FC_PERIPH_TSPI0      (0x00000001UL << 19)    /* TSPI 0    */
    #define CG_FC_PERIPH_UART0      (0x00000001UL << 21)    /* UART 0    */
    #define CG_FC_PERIPH_UART1      (0x00000001UL << 22)    /* UART 1    */
    #define CG_FC_PERIPH_UART2      (0x00000001UL << 23)    /* UART 2    */
    #define CG_FC_PERIPH_I2C0       (0x00000001UL << 25)    /* I2C 0     */
    #define CG_FC_PERIPH_T32A0      (0x00000001UL << 28)    /* T32A 0    */
    #define CG_FC_PERIPH_T32A1      (0x00000001UL << 29)    /* T32A 1    */
    #define CG_FC_PERIPH_T32A2      (0x00000001UL << 30)    /* T32A 2    */
    #define CG_FC_PERIPH_T32A3      (0x00000001UL << 31)    /* T32A 3    */
    #define CG_FSYSMENA_ALL_ENABLE  (0xF2E906F7UL)          /*!< CG FSYSMENA ALL ENABLE */
#endif
#define CG_FSYSMENA_ALL_DISABLE     (0x00000000UL)          /*!< CG FSYSMENA ALL DISABLE */

#if defined(TMPM4KNA)
    #define CG_FC_PERIPH_T32A4      (0x00000001UL << 0)     /* T32A 4    */
    #define CG_FC_PERIPH_T32A5      (0x00000001UL << 1)     /* T32A 5    */
    #define CG_FC_PERIPH_ADCA       (0x00000001UL << 2)     /* ADC A     */
    #define CG_FC_PERIPH_ADCB       (0x00000001UL << 3)     /* ADC B     */
    #define CG_FC_PERIPH_ADCC       (0x00000001UL << 4)     /* ADC C     */
    #define CG_FC_PERIPH_OPAMP      (0x00000001UL << 5)     /* OPAMP     */
    #define CG_FC_PERIPH_A_ENC320   (0x00000001UL << 6)     /* A-ENC32 0 */
    #define CG_FC_PERIPH_A_ENC321   (0x00000001UL << 7)     /* A-ENC32 1 */
    #define CG_FC_PERIPH_A_ENC322   (0x00000001UL << 8)     /* A-ENC32 2 */
    #define CG_FC_PERIPH_A_PMD0     (0x00000001UL << 9)     /* A-PMD 0   */
    #define CG_FC_PERIPH_A_PMD1     (0x00000001UL << 10)    /* A-PMD 1   */
    #define CG_FC_PERIPH_A_PMD2     (0x00000001UL << 11)    /* A-PMD 2   */
    #define CG_FC_PERIPH_A_VE0      (0x00000001UL << 12)    /* A-VE 0    */
    #define CG_FC_PERIPH_RAMP1      (0x00000001UL << 13)    /* RAMP 1    */
    #define CG_FC_PERIPH_OFD        (0x00000001UL << 14)    /* OFD       */
    #define CG_FC_PERIPH_TRM        (0x00000001UL << 15)    /* TRM       */
    #define CG_FC_PERIPH_TRGSEL     (0x00000001UL << 16)    /* TRGSEL    */
    #define CG_FC_PERIPH_DMACA      (0x00000001UL << 17)    /* DMAC A    */
    #define CG_FC_PERIPH_EI2C0      (0x00000001UL << 25)    /* EI2C 0    */
    #define CG_FC_PERIPH_EI2C1      (0x00000001UL << 26)    /* EI2C 1    */
    #define CG_FC_PERIPH_NBDIF      (0x00000001UL << 30)    /* NBDIF     */
    #define CG_FC_PERIPH_SIWDT0     (0x00000001UL << 31)    /* SIWDT 0   */
    #define CG_FSYSMENB_ALL_ENABLE  (0xE603FFFFUL)          /*!< CG FSYSMENB ALL ENABLE */
#endif
#if defined(TMPM4KMA)
    #define CG_FC_PERIPH_T32A4      (0x00000001UL << 0)     /* T32A 4    */
    #define CG_FC_PERIPH_T32A5      (0x00000001UL << 1)     /* T32A 5    */
    #define CG_FC_PERIPH_ADCA       (0x00000001UL << 2)     /* ADC A     */
    #define CG_FC_PERIPH_ADCB       (0x00000001UL << 3)     /* ADC B     */
    #define CG_FC_PERIPH_ADCC       (0x00000001UL << 4)     /* ADC C     */
    #define CG_FC_PERIPH_OPAMP      (0x00000001UL << 5)     /* OPAMP     */
    #define CG_FC_PERIPH_A_ENC320   (0x00000001UL << 6)     /* A-ENC32 0 */
    #define CG_FC_PERIPH_A_ENC321   (0x00000001UL << 7)     /* A-ENC32 1 */
    #define CG_FC_PERIPH_A_ENC322   (0x00000001UL << 8)     /* A-ENC32 2 */
    #define CG_FC_PERIPH_A_PMD0     (0x00000001UL << 9)     /* A-PMD 0   */
    #define CG_FC_PERIPH_A_PMD1     (0x00000001UL << 10)    /* A-PMD 1   */
    #define CG_FC_PERIPH_A_PMD2     (0x00000001UL << 11)    /* A-PMD 2   */
    #define CG_FC_PERIPH_A_VE0      (0x00000001UL << 12)    /* A-VE 0    */
    #define CG_FC_PERIPH_RAMP1      (0x00000001UL << 13)    /* RAMP 1    */
    #define CG_FC_PERIPH_OFD        (0x00000001UL << 14)    /* OFD       */
    #define CG_FC_PERIPH_TRM        (0x00000001UL << 15)    /* TRM       */
    #define CG_FC_PERIPH_TRGSEL     (0x00000001UL << 16)    /* TRGSEL    */
    #define CG_FC_PERIPH_DMACA      (0x00000001UL << 17)    /* DMAC A    */
    #define CG_FC_PERIPH_EI2C0      (0x00000001UL << 25)    /* EI2C 0    */
    #define CG_FC_PERIPH_EI2C1      (0x00000001UL << 26)    /* EI2C 1    */
    #define CG_FC_PERIPH_SIWDT0     (0x00000001UL << 31)    /* SIWDT 0   */
    #define CG_FSYSMENB_ALL_ENABLE  (0xA603FFFFUL)          /*!< CG FSYSMENB ALL ENABLE */
#endif
#if defined(TMPM4KLA)
    #define CG_FC_PERIPH_T32A4      (0x00000001UL << 0)     /* T32A 4    */
    #define CG_FC_PERIPH_T32A5      (0x00000001UL << 1)     /* T32A 5    */
    #define CG_FC_PERIPH_ADCA       (0x00000001UL << 2)     /* ADC A     */
    #define CG_FC_PERIPH_ADCB       (0x00000001UL << 3)     /* ADC B     */
    #define CG_FC_PERIPH_ADCC       (0x00000001UL << 4)     /* ADC C     */
    #define CG_FC_PERIPH_OPAMP      (0x00000001UL << 5)     /* OPAMP     */
    #define CG_FC_PERIPH_A_ENC322   (0x00000001UL << 8)     /* A-ENC32 2 */
    #define CG_FC_PERIPH_A_PMD0     (0x00000001UL << 9)     /* A-PMD 0   */
    #define CG_FC_PERIPH_A_PMD1     (0x00000001UL << 10)    /* A-PMD 1   */
    #define CG_FC_PERIPH_A_PMD2     (0x00000001UL << 11)    /* A-PMD 2   */
    #define CG_FC_PERIPH_A_VE0      (0x00000001UL << 12)    /* A-VE 0    */
    #define CG_FC_PERIPH_RAMP1      (0x00000001UL << 13)    /* RAMP 1    */
    #define CG_FC_PERIPH_OFD        (0x00000001UL << 14)    /* OFD       */
    #define CG_FC_PERIPH_TRM        (0x00000001UL << 15)    /* TRM       */
    #define CG_FC_PERIPH_TRGSEL     (0x00000001UL << 16)    /* TRGSEL    */
    #define CG_FC_PERIPH_DMACA      (0x00000001UL << 17)    /* DMAC A    */
    #define CG_FC_PERIPH_EI2C0      (0x00000001UL << 25)    /* EI2C 0    */
    #define CG_FC_PERIPH_EI2C1      (0x00000001UL << 26)    /* EI2C 1    */
    #define CG_FC_PERIPH_SIWDT0     (0x00000001UL << 31)    /* SIWDT 0   */
    #define CG_FSYSMENB_ALL_ENABLE  (0xA603FF3FUL)          /*!< CG FSYSMENB ALL ENABLE */
#endif
#if defined(TMPM4KHA)
    #define CG_FC_PERIPH_T32A4      (0x00000001UL << 0)     /* T32A 4    */
    #define CG_FC_PERIPH_T32A5      (0x00000001UL << 1)     /* T32A 5    */
    #define CG_FC_PERIPH_ADCA       (0x00000001UL << 2)     /* ADC A     */
    #define CG_FC_PERIPH_ADCB       (0x00000001UL << 3)     /* ADC B     */
    #define CG_FC_PERIPH_ADCC       (0x00000001UL << 4)     /* ADC C     */
    #define CG_FC_PERIPH_OPAMP      (0x00000001UL << 5)     /* OPAMP     */
    #define CG_FC_PERIPH_A_PMD0     (0x00000001UL << 9)     /* A-PMD 0   */
    #define CG_FC_PERIPH_A_VE0      (0x00000001UL << 12)    /* A-VE 0    */
    #define CG_FC_PERIPH_RAMP1      (0x00000001UL << 13)    /* RAMP 1    */
    #define CG_FC_PERIPH_OFD        (0x00000001UL << 14)    /* OFD       */
    #define CG_FC_PERIPH_TRM        (0x00000001UL << 15)    /* TRM       */
    #define CG_FC_PERIPH_TRGSEL     (0x00000001UL << 16)    /* TRGSEL    */
    #define CG_FC_PERIPH_DMACA      (0x00000001UL << 17)    /* DMAC A    */
    #define CG_FC_PERIPH_EI2C0      (0x00000001UL << 25)    /* EI2C 0    */
    #define CG_FC_PERIPH_SIWDT0     (0x00000001UL << 31)    /* SIWDT 0   */
    #define CG_FSYSMENB_ALL_ENABLE  (0xA203F23FUL)          /*!< CG FSYSMENB ALL ENABLE */
#endif
#define CG_FSYSMENB_ALL_DISABLE     (0x20000000UL)          /*!< CG FSYSMENB ALL DISABLE */

#define CG_FC_PERIPH_CRC            (0x00000001UL << 0)     /* CRC       */
#define CG_FC_PERIPH_RAMP0          (0x00000001UL << 1)     /* RAMParity */
#define CG_FSYSENA_ALL_ENABLE       (0x00000003UL)          /*!< CG FSYSENA ALL ENABLE */
#define CG_FSYSENA_ALL_DISABLE      (0x00000000UL)          /*!< CG FSYSENA ALL DISABLE */

#if !defined(TMPM4KHA)
    #define CG_FC_PERIPH_OFD1       (0x00000001UL << 23)    /* OFD1      */
    #define CG_FC_PERIPH_DNFA       (0x00000001UL << 26)    /* DNFA      */
    #define CG_FC_PERIPH_DNFB       (0x00000001UL << 27)    /* DNFB      */
    #define CG_FC_PERIPH_DNFC       (0x00000001UL << 28)    /* DNFC      */
    #define CG_FCEN_ALL_ENABLE      (0x1C800000UL)          /*!< CG FCEN ALL ENABLE */
#else
    #define CG_FC_PERIPH_OFD1       (0x00000001UL << 23)    /* OFD1      */
    #define CG_FC_PERIPH_DNFA       (0x00000001UL << 26)    /* DNFA      */
    #define CG_FC_PERIPH_DNFB       (0x00000001UL << 27)    /* DNFB      */
    #define CG_FCEN_ALL_ENABLE      (0x0C800000UL)          /*!< CG FCEN ALL ENABLE */
#endif
#define CG_FCEN_ALL_DISABLE         (0x00000000UL)          /*!< CG FCEN ALL DISABLE */

#define CG_FC_PERIPH_TRCKEN         (0x00000001UL << 0)     /* TRCKEN   */
#define CG_FC_PERIPH_ADCKEN0        (0x00000001UL << 16)    /* ADCKEN0  */
#define CG_FC_PERIPH_ADCKEN1        (0x00000001UL << 17)    /* ADCKEN1  */
#define CG_FC_PERIPH_ADCKEN2        (0x00000001UL << 18)    /* ADCKEN2  */
#define CG_SPCLKEN_ALL_ENABLE       (0x00070001UL)          /*!< CG SPCLKEN ALL ENABLE */
#define CG_SPCLKEN_ALL_DISABLE      (0x00000000UL)          /*!< CG SPCLKEN ALL DISABLE */

/**
 *  @}
 */ /* End of group CG_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Table Definition                                                            */
/*------------------------------------------------------------------------------*/
#define SYSTEM_CORE_CLOCK_TABLE_MAX       (5U)
static const uint32_t system_core_clock_table[SYSTEM_CORE_CLOCK_TABLE_MAX][3] =
                    {{EOSC_6M,  CG_6M_MUL_26_656_FPLL,  EOSC_6M_DIV2_PLLON},
                     {EOSC_8M,  CG_8M_MUL_20_FPLL,      EOSC_8M_DIV2_PLLON},
                     {EOSC_10M, CG_10M_MUL_16_FPLL,     EOSC_10M_DIV2_PLLON},
                     {EOSC_12M, CG_12M_MUL_13_312_FPLL, EOSC_12M_DIV2_PLLON},
                     {IOSC_10M, CG_10M_MUL_16_FPLL,     EOSC_10M_DIV2_PLLON}};
  
/**
 *  @}
 */ /* End of group CG */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __CG_M4K2_H */


