/** 
 *******************************************************************************
 * @file    dnf_m4k2.h
 * @brief   This file provides all the functions prototypes for A-DNF.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __DNF_M4K2_H
#define __DNF_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup DNF DNF
 *  @brief    DNF Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_define
 *  @{
 */

/** ---------------------------------------
 *  @defgroup NFCKCR NFCKCR
 *  @brief    DNF Noise Filter Register.
 *  @{
 */

/** 
 *  @defgroup NFCKCR_NFCKS NFCKS
 *  @brief    [Bit2-0]: Noise Filter Clock Select.
 *  @{
 */
#define REG_DNF_NFCKCR_NFCKS_STOP           ((uint32_t)0x00000000)        /*!< Clock Control circuit stop.                                       */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_2        ((uint32_t)0x00000001)        /*!< fc/2   fc=20MHz: 0.7us 40MHz: 0.35us 80MHz: 0.175us 100MHz:0.14us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_4        ((uint32_t)0x00000002)        /*!< fc/4   fc=20MHz: 1.4us 40MHz: 0.7us  80MHz: 0.35us  100MHz:0.28us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_8        ((uint32_t)0x00000003)        /*!< fc/8   fc=20MHz: 2.8us 40MHz: 1.4us  80MHz: 0.7us   100MHz:0.56us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_16       ((uint32_t)0x00000004)        /*!< fc/16  fc=20MHz: 5.6us 40MHz: 2.8us  80MHz: 1.4us   100MHz:1.12us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_32       ((uint32_t)0x00000005)        /*!< fc/32  fc=20MHz:11.2us 40MHz: 5.6us  80MHz: 2.8us   100MHz:2.24us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_64       ((uint32_t)0x00000006)        /*!< fc/64  fc=20MHz:22.4us 40MHz:11.2us  80MHz: 5.6us   100MHz:4.48us */
#define REG_DNF_NFCKCR_NFCKS_CLOCK_128      ((uint32_t)0x00000007)        /*!< fc/128 fc=20MHz:44.8us 40MHz:22.4us  80MHz:11.2us   100MHz:8.96us */
/**
 *  @}
 */ /* End of group NFCKCR_NFCKS */

/**
 *  @}
 */ /* End of group NFCKCR */



/** ---------------------------------------
 *  @defgroup NFENCR NFENCR
 *  @brief    DNF Noise Filter Enable Register.
 *  @{
 */
/** 
 *  @defgroup NFENCR_NFEN15 NFEN15
 *  @brief    [Bit15]: INT15 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN15_MASK          ((uint32_t)0x00008000)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN15_DISABLE       ((uint32_t)0x00000000)        /*!< INT15 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN15_ENABLE        ((uint32_t)0x00008000)        /*!< INT15 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN15 */

/** 
 *  @defgroup NFENCR_NFEN14 NFEN14
 *  @brief    [Bit14]: INT14 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN14_MASK          ((uint32_t)0x00004000)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN14_DISABLE       ((uint32_t)0x00000000)        /*!< INT14 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN14_ENABLE        ((uint32_t)0x00004000)        /*!< INT14 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN14 */

/** 
 *  @defgroup NFENCR_NFEN13 NFEN13
 *  @brief    [Bit13]: INT13 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN13_MASK          ((uint32_t)0x00002000)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN13_DISABLE       ((uint32_t)0x00000000)        /*!< INT13 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN13_ENABLE        ((uint32_t)0x00002000)        /*!< INT13 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN13 */

/** 
 *  @defgroup NFENCR_NFEN12 NFEN12
 *  @brief    [Bit12]: INT12 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN12_MASK          ((uint32_t)0x00001000)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN12_DISABLE       ((uint32_t)0x00000000)        /*!< INT12 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN12_ENABLE        ((uint32_t)0x00001000)        /*!< INT12 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN12 */

/** 
 *  @defgroup NFENCR_NFEN11 NFEN11
 *  @brief    [Bit11]: INT11 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN11_MASK          ((uint32_t)0x00000800)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN11_DISABLE       ((uint32_t)0x00000000)        /*!< INT11 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN11_ENABLE        ((uint32_t)0x00000800)        /*!< INT11 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN11 */

/** 
 *  @defgroup NFENCR_NFEN10 NFEN10
 *  @brief    [Bit10]: INT10 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN10_MASK          ((uint32_t)0x00000400)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN10_DISABLE       ((uint32_t)0x00000000)        /*!< INT10 Noise Filter Disable.  */
#define REG_DNF_NFENCR_NFEN10_ENABLE        ((uint32_t)0x00000400)        /*!< INT10 Noise Filter Enable.   */
/**
 *  @}
 */ /* End of group NFENCR_NFEN10 */

/** 
 *  @defgroup NFENCR_NFEN9 NFEN9
 *  @brief    [Bit9]: INT9 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN9_MASK           ((uint32_t)0x00000200)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN9_DISABLE        ((uint32_t)0x00000000)        /*!< INT9 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN9_ENABLE         ((uint32_t)0x00000200)        /*!< INT9 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN9 */

/** 
 *  @defgroup NFENCR_NFEN8 NFEN8
 *  @brief    [Bit8]: INT8 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN8_MASK           ((uint32_t)0x00000100)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN8_DISABLE        ((uint32_t)0x00000000)        /*!< INT8 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN8_ENABLE         ((uint32_t)0x00000100)        /*!< INT8 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN8 */

/** 
 *  @defgroup NFENCR_NFEN7 NFEN7
 *  @brief    [Bit7]: INT7 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN7_MASK           ((uint32_t)0x00000080)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN7_DISABLE        ((uint32_t)0x00000000)        /*!< INT7 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN7_ENABLE         ((uint32_t)0x00000080)        /*!< INT7 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN7 */

/** 
 *  @defgroup NFENCR_NFEN6 NFEN6
 *  @brief    [Bit6]: INT6 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN6_MASK           ((uint32_t)0x00000040)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN6_DISABLE        ((uint32_t)0x00000000)        /*!< INT6 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN6_ENABLE         ((uint32_t)0x00000040)        /*!< INT6 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN6 */

/** 
 *  @defgroup NFENCR_NFEN5 NFEN5
 *  @brief    [Bit5]: INT5 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN5_MASK           ((uint32_t)0x00000020)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN5_DISABLE        ((uint32_t)0x00000000)        /*!< INT5 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN5_ENABLE         ((uint32_t)0x00000020)        /*!< INT5 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN5 */

/** 
 *  @defgroup NFENCR_NFEN4 NFEN4
 *  @brief    [Bit4]: INT4 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN4_MASK           ((uint32_t)0x00000010)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN4_DISABLE        ((uint32_t)0x00000000)        /*!< INT4 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN4_ENABLE         ((uint32_t)0x00000010)        /*!< INT4 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN4 */

/** 
 *  @defgroup NFENCR_NFEN3 NFEN3
 *  @brief    [Bit3]: INT3 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN3_MASK           ((uint32_t)0x00000008)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN3_DISABLE        ((uint32_t)0x00000000)        /*!< INT3 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN3_ENABLE         ((uint32_t)0x00000008)        /*!< INT3 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN3 */

/** 
 *  @defgroup NFENCR_NFEN2 NFEN2
 *  @brief    [Bit2]: INT2 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN2_MASK           ((uint32_t)0x00000004)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN2_DISABLE        ((uint32_t)0x00000000)        /*!< INT2 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN2_ENABLE         ((uint32_t)0x00000004)        /*!< INT2 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN2 */

/** 
 *  @defgroup NFENCR_NFEN1 NFEN1
 *  @brief    [Bit1]: INT1 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN1_MASK           ((uint32_t)0x00000002)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN1_DISABLE        ((uint32_t)0x00000000)        /*!< INT1 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN1_ENABLE         ((uint32_t)0x00000002)        /*!< INT1 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN1 */

/** 
 *  @defgroup NFENCR_NFEN0 NFEN0
 *  @brief    [Bit0]: INT0 Noise Filter Enable.
 *  @{
 */
#define REG_DNF_NFENCR_NFEN0_MASK           ((uint32_t)0x00000001)        /*!< for Bit Mask.  */
#define REG_DNF_NFENCR_NFEN0_DISABLE        ((uint32_t)0x00000000)        /*!< INT0 Noise Filter Disable.   */
#define REG_DNF_NFENCR_NFEN0_ENABLE         ((uint32_t)0x00000001)        /*!< INT0 Noise Filter Enable.    */
/**
 *  @}
 */ /* End of group NFENCR_NFEN0 */

/**
 *  @}
 */ /* End of group NFENCR */

/** 
 *  @}
 */ /* End of group DNF_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group DNF_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Table Definition                                                            */
/*------------------------------------------------------------------------------*/
typedef struct encr_nfen
{
    TSB_DNF_TypeDef *addr;
    uint32_t bit;
} encr_nfen_t;

#if defined(TMPM4KNA)
static encr_nfen_t valid_encr_nfen_setting_table[] =
{
    { TSB_DNFA, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN3_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN4_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN5_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN6_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN7_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN8_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN9_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN10_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN11_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN12_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN13_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN14_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN15_MASK },

    { TSB_DNFB, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN3_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN4_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN5_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN6_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN7_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN8_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN9_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN10_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN11_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN12_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN13_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN14_MASK },

    { TSB_DNFC, REG_DNF_NFENCR_NFEN3_MASK },
};
#elif defined(TMPM4KMA)
static encr_nfen_t valid_encr_nfen_setting_table[] =
{
    { TSB_DNFA, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN3_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN5_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN7_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN8_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN9_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN10_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN11_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN13_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN14_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN15_MASK },

    { TSB_DNFB, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN3_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN4_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN5_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN6_MASK },
    /* none */
    { TSB_DNFB, REG_DNF_NFENCR_NFEN8_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN9_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN10_MASK },
    /* none */
    /* none */
    /* none */
    /* none */

    { TSB_DNFC, REG_DNF_NFENCR_NFEN3_MASK },
};
#elif defined(TMPM4KLA)
static encr_nfen_t valid_encr_nfen_setting_table[] =
{
    { TSB_DNFA, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN3_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN5_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN7_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN8_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN9_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN10_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN11_MASK },
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN13_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN14_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN15_MASK },

    { TSB_DNFB, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN3_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN4_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN5_MASK },
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */

    { TSB_DNFC, REG_DNF_NFENCR_NFEN3_MASK },
};
#elif defined(TMPM4KHA)
static encr_nfen_t valid_encr_nfen_setting_table[] =
{
    { TSB_DNFA, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN1_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN2_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN3_MASK },
    /* none */
    /* none */
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN7_MASK },
    { TSB_DNFA, REG_DNF_NFENCR_NFEN8_MASK },
    /* none */
    /* none */
    { TSB_DNFA, REG_DNF_NFENCR_NFEN11_MASK },
    /* none */
    /* none */
    /* none */
    /* none */

    { TSB_DNFB, REG_DNF_NFENCR_NFEN0_MASK },
    { TSB_DNFB, REG_DNF_NFENCR_NFEN1_MASK },
    /* none */
    /* none */
    /* none */
    { TSB_DNFB, REG_DNF_NFENCR_NFEN5_MASK },
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */
    /* none */

    /* none */
};
#endif

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_functions
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Exported_functions */

/**
 *  @}
 */ /* End of group DNF */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __DNF_M4K2_H */
