/** 
 *******************************************************************************
 * @file    ei2c_m4k2.h
 * @brief   This file provides all the functions prototypes for EI2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __EI2C_M4K2_H
#define __EI2C_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup EI2C
 *  @brief      EI2C Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */


#define AVAILABLE           (1)     /*!< EI2C function equipped */
#define UNAVAILABLE         (0)     /*!< EI2C function not equipped */

#define EI2CS_EQUIPPED      UNAVAILABLE /*!< Wakeup on EI2C is not equipped */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */

#ifdef __DEBUG__
/** 
 *  @name  EI2C_NULL Pointer
 *  @brief NULL Pointer.
 *  @{
 */
#define EI2C_NULL                ((void *)0)
/**
 *  @}
 */ /* End of name EI2C_NULL Pointer */
#endif

/** 
 *  @name  EI2CxARST Macro Definition.
 *  @brief EI2CxARST Register Macro Definition.
 *  @{
 */
#define EI2CxARST_SWRES_10       ((uint32_t)0x00000002)    /*!< SWRES=10                */
#define EI2CxARST_SWRES_01       ((uint32_t)0x00000001)    /*!< SWRES=01                */
/**
 *  @}
 */ /* End of name EI2CxARST Macro Definition */

/** 
 *  @name  EI2CxAEN Macro Definition.
 *  @brief EI2CxAEN Register Macro Definition.
 *  @{
 */
#define EI2CxAEN_I2CM_DISABLE    ((uint32_t)0x00000000)     /*!< EI2C Disable */
#define EI2CxAEN_I2CM_ENABLE     ((uint32_t)0x00000001)     /*!< EI2C Enable  */
/**
 *  @}
 */ /* End of name EI2CxAEN Macro Definition */

/** 
 *  @name  EI2CxCR0_DNF Macro Definition.
 *  @brief EI2CxCR0_DNF Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_DNF_1PRSCK      ((uint32_t)0x00000000)     /*!< 1PRSCK                */
#define EI2CxCR0_DNF_2PRSCK      ((uint32_t)0x00001000)     /*!< 2PRSCK                */
#define EI2CxCR0_DNF_3PRSCK      ((uint32_t)0x00002000)     /*!< 3PRSCK                */
#define EI2CxCR0_DNF_4PRSCK      ((uint32_t)0x00003000)     /*!< 4PRSCK                */
#define EI2CxCR0_DNF_5PRSCK      ((uint32_t)0x00004000)     /*!< 5PRSCK                */
#define EI2CxCR0_DNF_6PRSCK      ((uint32_t)0x00005000)     /*!< 6PRSCK                */
#define EI2CxCR0_DNF_NA7         ((uint32_t)0x00006000)     /*!< NA7(the same with 00) */
#define EI2CxCR0_DNF_NA8         ((uint32_t)0x00007000)     /*!< NA8(the same with 00) */
#define EI2CxCR0_DNF_MASK        ((uint32_t)0x00007000)     /*!< CR0 14-12 bits mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_DNF Macro Definition */
 
/** 
 *  @name  EI2CxCR0_NFSEL Macro Definition.
 *  @brief EI2CxCR0_NFSEL Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_NFSEL_DIG       ((uint32_t)0x00000000)     /*!< Digtal noise filter   */
#define EI2CxCR0_NFSEL_ANA       ((uint32_t)0x00000800)     /*!< Analog noise filter   */
#define EI2CxCR0_NFSEL_MASK      ((uint32_t)0x00000800)     /*!< CR0 11 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_NFSEL Macro Definition */
 
/** 
 *  @name  EI2CxCR0_TOPER Macro Definition.
 *  @brief EI2CxCR0_TOPER Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_TOPER_1PRSCK    ((uint32_t)0x00000000)     /*!< timeout 2*13 PRSCK   */
#define EI2CxCR0_TOPER_2PRSCK    ((uint32_t)0x00000200)     /*!< timeout 2*14 PRSCK   */
#define EI2CxCR0_TOPER_3PRSCK    ((uint32_t)0x00000400)     /*!< timeout 2*15 PRSCK   */
#define EI2CxCR0_TOPER_4PRSCK    ((uint32_t)0x00000600)     /*!< timeout 2*16 PRSCK   */
#define EI2CxCR0_TOPER_MASK      ((uint32_t)0x00000600)     /*!< CR0 10-9 bits mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_TOPER Macro Definition */
 
 /** 
 *  @name  EI2CxCR0_TOE Macro Definition.
 *  @brief EI2CxCR0_TOE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_TOE_ENABLE     ((uint32_t)0x00000100)     /*!< timeout Enable        */
#define EI2CxCR0_TOE_DISABLE    ((uint32_t)0x00000000)     /*!< timeout Disable       */
#define EI2CxCR0_TOE_MASK       ((uint32_t)0x00000100)     /*!< CR0 8 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_TOE Macro Definition */
 
/** 
 *  @name  EI2CxCR0_ESPE Macro Definition.
 *  @brief EI2CxCR0_ESPE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_ESPE_ENABLE    ((uint32_t)0x00000010)     /*!< error stop condition Enable  */
#define EI2CxCR0_ESPE_DISABLE   ((uint32_t)0x00000000)     /*!< error stop condition Disable */
#define EI2CxCR0_ESPE_MASK      ((uint32_t)0x00000010)     /*!< CR0 4 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_ESPE Macro Definition */

/** 
 *  @name  EI2CxCR0_ESTE Macro Definition.
 *  @brief EI2CxCR0_ESTE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_ESTE_ENABLE    ((uint32_t)0x00000008)     /*!< error start condition Enable  */
#define EI2CxCR0_ESTE_DISABLE   ((uint32_t)0x00000000)     /*!< error start condition Disable */
#define EI2CxCR0_ESTE_MASK      ((uint32_t)0x00000008)     /*!< CR0 3 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_ESTE Macro Definition */
 
/** 
 *  @name  EI2CxCR0_NACKE Macro Definition.
 *  @brief EI2CxCR0_NACKE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_NACKE_ENABLE    ((uint32_t)0x00000004)     /*!< NACK Check Enable           */
#define EI2CxCR0_NACKE_DISABLE   ((uint32_t)0x00000000)     /*!< NACK Check Disable          */
#define EI2CxCR0_NACKE_MASK      ((uint32_t)0x00000004)     /*!< CR0 2 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_NACKE Macro Definition */
 
/** 
 *  @name  EI2CxCR0_GCE Macro Definition.
 *  @brief EI2CxCR0_GCE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_GCE_ENABLE      ((uint32_t)0x00000002)     /*!< General call Enable         */
#define EI2CxCR0_GCE_DISABLE     ((uint32_t)0x00000000)     /*!< General call Disable        */
#define EI2CxCR0_GCE_MASK        ((uint32_t)0x00000002)     /*!< CR0 1 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_GCE Macro Definition */
 
/** 
 *  @name  EI2CxCR0_ALE Macro Definition.
 *  @brief EI2CxCR0_ALE Register Macro Definition.
 *  @{
 */
#define EI2CxCR0_ALE_ENABLE      ((uint32_t)0x00000001)     /*!< Arbitration lost Enable     */
#define EI2CxCR0_ALE_DISABLE     ((uint32_t)0x00000000)     /*!< Arbitration lost Disable    */
#define EI2CxCR0_ALE_MASK        ((uint32_t)0x00000001)     /*!< CR0 0 bit mask */
/**
 *  @}
 */ /* End of name EI2CxCR0_ALE Macro Definition */

/** 
 *  @name  EI2CxCR1 Macro Definition.
 *  @brief EI2CxCR1 Register Macro Definition.
 *  @{
 */
#define EI2CxCR1_OMC             ((uint32_t)0x00000400)     /*!< OMC=1       */
#define EI2CxCR1_ACKWAIT         ((uint32_t)0x00000010)     /*!< ACKWAIT=1   */
#define EI2CxCR1_ACKSEL_NACK     ((uint32_t)0x00000008)     /*!< ACKSEL=NACK */
#define EI2CxCR1_SP              ((uint32_t)0x00000004)     /*!< SP=1        */
#define EI2CxCR1_RS              ((uint32_t)0x00000002)     /*!< RS=1        */
#define EI2CxCR1_ST              ((uint32_t)0x00000001)     /*!< ST=1        */
#define EI2CxCR1_CLEAR           ((uint32_t)0x00000000)     /*!< Clear       */

/**
 *  @}
 */ /* End of name EI2CxCR1 Macro Definition */

/** 
 *  @name  EI2CxDBRT Macro Definition.
 *  @brief EI2CxDBRT Register Macro Definition.
 *  @{
 */
#define EI2CxDBRT_DBT_MASK         ((uint32_t)0x000000FF)      /* !< DB 7-0 bits mask.     */
/**
 *  @}
 */ /* End of name EI2CxDBRT Macro Definition */
 
/** 
 *  @name  EI2CxDBRR Macro Definition.
 *  @brief EI2CxDBRR Register Macro Definition.
 *  @{
 */
#define EI2CxDBRR_DBR_MASK         ((uint32_t)0x000000FF)      /* !< DB 7-0 bits mask.     */
/**
 *  @}
 */ /* End of name EI2CxDBRR Macro Definition */

/** 
 *  @name  EI2CxSR0 Macro Definition.
 *  @brief EI2CxSR0 Register Macro Definition.
 *  @{
 */
#define EI2CxSR0_INT              ((uint32_t)0x00000000)     /*!< Initial Settings.       */
#define EI2CxSR0_BC               ((uint32_t)0x000000F0)     /*!< BC                      */
#define EI2CxSR0_MST              ((uint32_t)0x00000008)     /*!< MST                     */
#define EI2CxSR0_TRX              ((uint32_t)0x00000004)     /*!< TRX                     */
#define EI2CxSR0_BB               ((uint32_t)0x00000002)     /*!< BB                      */
#define EI2CxSR0_ACKF             ((uint32_t)0x00000001)     /*!< ACKF                    */
/**
 *  @}
 */ /* End of name EI2CxSR0 Macro Definition */


/** 
 *  @name  EI2CxSR1 Macro Definition.
 *  @brief EI2CxSR1 Register Macro Definition.
 *  @{
 */
#define EI2CxSR1_TOERR            ((uint32_t)0x00002000)     /*!< Timeout error.               */
#define EI2CxSR1_ESP              ((uint32_t)0x00001000)     /*!< Error stop condition         */
#define EI2CxSR1_EST              ((uint32_t)0x00000800)     /*!< Error start condition        */
#define EI2CxSR1_AAS2             ((uint32_t)0x00000400)     /*!< The same NO2 address flag    */
#define EI2CxSR1_AAS1             ((uint32_t)0x00000200)     /*!< The same NO1 address flag    */
#define EI2CxSR1_GC               ((uint32_t)0x00000100)     /*!< General call check flag      */
#define EI2CxSR1_AL               ((uint32_t)0x00000080)     /*!< Arbitration lost flag        */
#define EI2CxSR1_NACK             ((uint32_t)0x00000040)     /*!< NACK check flag              */
#define EI2CxSR1_RBF              ((uint32_t)0x00000020)     /*!< Receive buffer full          */
#define EI2CxSR1_TBE              ((uint32_t)0x00000010)     /*!< Transmit buffer empty        */
#define EI2CxSR1_TEND             ((uint32_t)0x00000008)     /*!< Transmit over flag           */
#define EI2CxSR1_SPCF             ((uint32_t)0x00000004)     /*!< Stop Condition check flag    */
#define EI2CxSR1_RSCF             ((uint32_t)0x00000002)     /*!< ReStart Condition check flag */
#define EI2CxSR1_STCF             ((uint32_t)0x00000001)     /*!< Start Condition check flag   */
#define EI2CxSR1_CLEAR            ((uint32_t)0x00003FFF)     /*!< Clear INT status             */

/**
 *  @}
 */ /* End of name EI2CxSR1 Macro Definition */

/** 
 *  @name  EI2CxPRS Macro Definition.
 *  @brief EI2CxPRS Register Macro Definition.
 *  @{
 */
#define EI2CxPRS_PRCK            ((uint32_t)0x0000003F)     /*!< PRCK                    */
#define EI2CxPRS_MAX             ((uint32_t)64)             /*!< PRCK MAX                */
/**
 *  @}
 */ /* End of name EI2CxPRS Macro Definition */

/** 
 *  @name  EI2CxSCL Macro Definition.
 *  @brief EI2CxSCL Register Macro Definition.
 *  @{
 */
#define EI2CxSCL_SCL_MASK        ((uint32_t)0x000000FF)     /*!< SCLH SCLL MASK          */
#define EI2CxSCL_MAX             ((uint32_t)0xFF)           /*!< SCLH SCLL  Max size     */
/**
 *  @}
 */ /* End of name EI2CxSCL Macro Definition */
 
/** 
 *  @name  EI2CxIE Macro Definition.
 *  @brief EI2CxIE Register Macro Definition.
 *  @{
 */
#define EI2CxIE_DMARX         ((uint32_t)0x00008000)     /*!< Status INT(DMA receive req)       */
#define EI2CxIE_DMATX         ((uint32_t)0x00004000)     /*!< Status INT(DMA transmit req)      */
#define EI2CxIE_INTTOE        ((uint32_t)0x00002000)     /*!< Status INT(timeout)               */
#define EI2CxIE_INTESPE       ((uint32_t)0x00001000)     /*!< Status INT(error stop condition)  */
#define EI2CxIE_INTESTE       ((uint32_t)0x00000800)     /*!< Status INT(error start condition) */
#define EI2CxIE_INTASE        ((uint32_t)0x00000200)     /*!< Status INT(same address)          */
#define EI2CxIE_INTGCE        ((uint32_t)0x00000100)     /*!< Status INT(General call)          */
#define EI2CxIE_INTALE        ((uint32_t)0x00000080)     /*!< Status INT(Arbitration lost)      */
#define EI2CxIE_INTNACKE      ((uint32_t)0x00000040)     /*!< Status INT(NACK)                  */
#define EI2CxIE_INTSPE        ((uint32_t)0x00000004)     /*!< Status INT(stop condition)        */
#define EI2CxIE_INTRSE        ((uint32_t)0x00000002)     /*!< Status INT(restart condition)     */
#define EI2CxIE_INTSTE        ((uint32_t)0x00000001)     /*!< Status INT(start condition)       */
#define EI2CxIE_DISABLE       ((uint32_t)0x00000000)     /*!< Disable INT                       */

/**
 *  @}
 */ /* End of name EI2CxIE Macro Definition */

/** 
 *  @name  EI2CxAR1 Macro Definition.
 *  @brief EI2CxAR1 Register Macro Definition.
 *  @{
 */
#define EI2CxAR1_INT            ((uint32_t)0x00000000)      /*!< Initial Settings.     */
#define EI2CxAR1_SAFS1_7BIT     ((uint32_t)0x00000000)      /*!< 7Bit Address Format.  */
#define EI2CxAR1_SAFS1_10BIT    ((uint32_t)0x00008000)      /*!< 10Bit Address Format. */
#define EI2CxAR1_SAFS1_MASK     ((uint32_t)0x00008000)      /*!< Get Address Format    */
#define EI2CxAR1_SA1_7BITMASK   ((uint32_t)0x000000FE)      /*!< 7Bit Address MASK.    */
#define EI2CxAR1_SA1_10BITMASK  ((uint32_t)0x000007FE)      /*!< 10Bit Address MASK.   */
#define EI2CxAR1_SA1E_ENABLE    ((uint32_t)0x00000001)      /*!< No1 Slave Add Enable. */
#define EI2CxAR1_SA1E_DISABLE   ((uint32_t)0x00000000)      /*!< No1 Slave Add Disable.*/
#define EI2CxAR1_SA1E_MASK      ((uint32_t)0x00000001)      /*!< Get Slave Add Status  */

/**
 *  @}
 */ /* End of name EI2CxAR1 Macro Definition */

/** 
 *  @name  EI2CxAR2 Macro Definition.
 *  @brief EI2CxAR2 Register Macro Definition.
 *  @{
 */
#define EI2CxAR2_INT            ((uint32_t)0x00000000)      /*!< Initial Settings.     */
#define EI2CxAR2_SAFS2_7BIT     ((uint32_t)0x00000000)      /*!< 7Bit Address Format.  */
#define EI2CxAR2_SAFS2_10BIT    ((uint32_t)0x00008000)      /*!< 10Bit Address Format. */
#define EI2CxAR2_SAFS2_MASK     ((uint32_t)0x00008000)      /*!< Get Address Format    */
#define EI2CxAR2_SA2_7BITMASK   ((uint32_t)0x000000FE)      /*!< 7Bit Address MASK.    */
#define EI2CxAR2_SA2_10BITMASK  ((uint32_t)0x000007FE)      /*!< 10Bit Address MASK.   */
#define EI2CxAR2_SA2E_ENABLE    ((uint32_t)0x00000001)      /*!< No2 Slave Add Enable. */
#define EI2CxAR2_SA2E_DISABLE   ((uint32_t)0x00000000)      /*!< No2 Slave Add Disable.*/
#define EI2CxAR2_SA2E_MASK      ((uint32_t)0x00000001)      /*!< Get Slave Add Status  */

/**
 *  @}
 */ /* End of name EI2CxAR2 Macro Definition */
 
/** 
 *  @name  EI2CxPM Macro Definition.
 *  @brief EI2CxPM Register Macro Definition.
 *  @{
 */
#define EI2CxPM_SDA_SCL           ((uint32_t)0x00000003)      /* SDA and SCL level. */
/**
 *  @}
 */ /* End of name EI2CxPM Macro Definition */

#if (EI2CS_EQUIPPED == AVAILABLE)

/** 
 *  @name  I2CxWUPCR1_INTEND Macro Definition.
 *  @brief I2CxWUPCR1_INTEND Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_INTEND_RELEASE    ((uint8_t)0x01)      /* Interrupt Release.         */
#define I2CxWUPCR1_INTEND_HOLD       ((uint8_t)0x00)      /* Interrupt setting keep it. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_INTEND Macro Definition */

/** 
 *  @name  I2CxWUPCR1_GC Macro Definition.
 *  @brief I2CxWUPCR1_GC Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_GC_DETECT     ((uint8_t)0x02)      /* General Call detect.     */
#define I2CxWUPCR1_GC_NOTDETECT  ((uint8_t)0x00)      /* General Call not detect. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_GC Macro Definition */

/** 
 *  @name  I2CxWUPCR1_RW Macro Definition.
 *  @brief I2CxWUPCR1_RW Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_RW_TRANS      ((uint8_t)0x08)      /* Slave Trans.   */
#define I2CxWUPCR1_RW_RECEIVE    ((uint8_t)0x00)      /* Salve Receive. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_RW Macro Definition */

/** 
 *  @name  I2CxWUPCR1_I2RES Macro Definition.
 *  @brief I2CxWUPCR1_I2RES Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_I2RES_RESET   ((uint8_t)0x10)         /* I2C BUS Reset.             */
#define I2CxWUPCR1_I2RES_RELEASE ((uint8_t)0x00)         /* I2C BUS Reset Release.     */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_I2RES Macro Definition */


/** 
 *  @name  I2CxWUPCR1_ACK Macro Definition.
 *  @brief I2CxWUPC1R_ACK Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_ACK_NACK       ((uint8_t)0x20)         /* ACL No Output. Output "1" NACK Output */
#define I2CxWUPCR1_ACK_ACK        ((uint8_t)0x00)         /* ACK Output. Output "0"                */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_ACK Macro Definition */

/** 
 *  @name  I2CxWUPCR1_SGCDI Macro Definition.
 *  @brief I2CxWUPCR1_SGCDI Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_SGCDI_OFF          ((uint8_t)0x40)        /* General Call detect OFF. */
#define I2CxWUPCR1_SGCDI_ON           ((uint8_t)0x00)        /* General Call detect ON.  */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_SGCDI Macro Definition */

/** 
 *  @name  I2CxWUPCR1_BUSY Macro Definition.
 *  @brief I2CxWUPCR1_BUSY Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_BUSY_START          ((uint8_t)0x80)        /* General Call detect OFF. */
#define I2CxWUPCR1_BUSY_STOP           ((uint8_t)0x00)        /* General Call detect ON.  */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_BUSY Macro Definition */

/** 
 *  @name  I2CSWUPCR4 Macro Definition.
 *  @brief I2CSWUPCR4 Register Macro Definition.
 *  @{
 */
#define I2CSWUPCR4_WUPSAFS1_7BIT  ((uint8_t)0x00)      /* Primary slave adr 7Bit. */
#define I2CSWUPCR4_WUPSAFS1_10BIT ((uint8_t)0x80)      /* Primary slave adr 10Bit. */
/**
 *  @}
 */ /* End of name I2CSWUPCR4 Macro Definition */

/** 
 *  @name  I2CSWUPCR5 Macro Definition.
 *  @brief I2CSWUPCR5 Register Macro Definition.
 *  @{
 */
#define I2CSWUPCR5_WUPSAFS2_7BIT  ((uint8_t)0x00)      /* Primary slave adr 7Bit. */
#define I2CSWUPCR5_WUPSAFS2_10BIT ((uint8_t)0x80)      /* Primary slave adr 10Bit. */
/**
 *  @}
 */ /* End of name I2CSWUPCR5 Macro Definition */
#endif

/** 
 *  @}
 */ /* End of group EI2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group EI2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Table Definition                                                            */
/*------------------------------------------------------------------------------*/
typedef struct acr0_nfsel
{
    TSB_EI2C_TypeDef *addr;
    uint32_t bit;
} acr0_nfsel_t;

#if defined(TMPM4KNA)
static acr0_nfsel_t valid_acr0_nfsel_setting_table[] =
{
    { TSB_EI2C0, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C0, EI2CxCR0_NFSEL_ANA },

    { TSB_EI2C1, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C1, EI2CxCR0_NFSEL_ANA },
};
#elif defined(TMPM4KMA)
static acr0_nfsel_t valid_acr0_nfsel_setting_table[] =
{
    { TSB_EI2C0, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C0, EI2CxCR0_NFSEL_ANA },

    { TSB_EI2C1, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C1, EI2CxCR0_NFSEL_ANA },
};
#elif defined(TMPM4KLA)
static acr0_nfsel_t valid_acr0_nfsel_setting_table[] =
{
    { TSB_EI2C0, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C0, EI2CxCR0_NFSEL_ANA },

    { TSB_EI2C1, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C1, EI2CxCR0_NFSEL_ANA },
};
#elif defined(TMPM4KHA)
static acr0_nfsel_t valid_acr0_nfsel_setting_table[] =
{
    { TSB_EI2C0, EI2CxCR0_NFSEL_DIG },
    { TSB_EI2C0, EI2CxCR0_NFSEL_ANA },

    /* none */
    /* none */
};
#endif

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_functions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group EI2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_functions
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_functions */

/**
 *  @}
 */ /* End of group EI2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __EI2C_M4K2_H */


