/** 
 *******************************************************************************
 * @file    i2c_m4k2.h
 * @brief   This file provides all the functions prototypes for I2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __I2C_M4K2_H
#define __I2C_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup I2C
 *  @brief      I2C Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

#define AVAILABLE           (1)     /*!< I2C function equipped */
#define UNAVAILABLE         (0)     /*!< I2C function not equipped */

#define I2CS_EQUIPPED       UNAVAILABLE /*!< Wakeup on I2C is not equipped */

/** 
 *  @}
 */ /* End of group I2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

#ifdef __DEBUG__
/** 
 *  @name  I2C_NULL Pointer
 *  @brief NULL Pointer.
 *  @{
 */
#define I2C_NULL                ((void *)0)
/**
 *  @}
 */ /* End of name I2C_NULL Pointer */
#endif

/** 
 *  @defgroup Parameter_Result Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define I2C_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define I2C_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group Parameter_Result */

/** 
 *  @defgroup I2CC_SCKMax SCKMax
 *  @brief    Max SCK.
 *  @{
 */

#define I2C_SCK_MAX                 ((uint32_t)8)            /*!< Max SCK. */

/**
 *  @}
 */ /* End of group I2CC_SCKMax */

/** 
 *  @defgroup I2C_PRSCKMax PRSCK Max
 *  @brief    Max PRSCK.
 *  @{
 */

#define I2C_PRSCK_MAX                 ((uint32_t)32)            /*!< Max PRSCK. */

/**
 *  @}
 */ /* End of group I2C_PRSCKMax */

/** 
 *  @name  I2CxST Macro Definition.
 *  @brief I2CxST Register Macro Definition.
 *  @{
 */
#define I2CxST_NACK             ((uint32_t)0x00000008)    /*!< NACK Interrupt Status.   */
#define I2CxST_I2CBF            ((uint32_t)0x00000004)    /*!< I2CBF Interrupt Status.  */
#define I2CxST_I2CAL            ((uint32_t)0x00000002)    /*!< I2CAL Interrupt Status.  */
#define I2CxST_I2C              ((uint32_t)0x00000001)    /*!< I2C Interrupt Status.    */
#define I2CxST_CLEAR            ((uint32_t)0x0000000F)    /*!< All Bits Clear.          */
/**
 *  @}
 */ /* End of name I2CxST Macro Definition */

/** 
 *  @name  I2CxCR1 Macro Definition.
 *  @brief I2CxCR1 Register Macro Definition.
 *  @{
 */
#define I2CxCR1_ACK             ((uint32_t)0x00000010)     /*!< ACK         */
#define I2CxCR1_NOACK           ((uint32_t)0x00000008)     /*!< NOACK       */
#define I2CxCR1_BC              ((uint32_t)0x000000E0)     /*!< BC          */

/**
 *  @}
 */ /* End of name I2CxCR1 Macro Definition */

/** 
 *  @name  I2CxDBR Macro Definition.
 *  @brief I2CxDBR Register Macro Definition.
 *  @{
 */
#define I2CxDBR_DB_MASK         ((uint32_t)0x000000FF)      /* !< DB 7-0 bits mask.     */
/**
 *  @}
 */ /* End of name I2CxDBR Macro Definition */


/** 
 *  @name  I2CxCR2 Macro Definition.
 *  @brief I2CxCR2 Register Macro Definition.
 *  @{
 */
#define I2CxCR2_PIN_CLEAR       ((uint32_t)0x00000010)     /*!< PIN=1                                  */
#define I2CxCR2_I2CM_DISABLE    ((uint32_t)0x00000000)     /*!< I2CM=0                                 */
#define I2CxCR2_I2CM_ENABLE     ((uint32_t)0x00000008)     /*!< I2CM=1                                 */
#define I2CxCR2_SWRES_10        ((uint32_t)0x00000002)     /*!< SWRES=10                               */
#define I2CxCR2_SWRES_01        ((uint32_t)0x00000001)     /*!< SWRES=01                               */
#define I2CxCR2_START_CONDITION ((uint32_t)0x000000F8)     /*!< MST=1,TRX=1,BB=1,PIN=1,I2CM=1          */
#define I2CxCR2_STOP_CONDITION  ((uint32_t)0x000000D8)     /*!< MST=1,TRX=1,BB=0,PIN=1,I2CM=1          */
#define I2CxCR2_INIT            ((uint32_t)0x00000008)     /*!< MST=0,TRX=0,BB=0,PIN=0,I2CM=1,SWRES=00 */

/**
 *  @}
 */ /* End of name I2CxCR2 Macro Definition */

/** 
 *  @name  I2CxSR Macro Definition.
 *  @brief I2CxSR Register Macro Definition.
 *  @{
 */
#define I2CxSR_MST              ((uint32_t)0x00000080)     /*!< MST                     */
#define I2CxSR_TRX              ((uint32_t)0x00000040)     /*!< TRX                     */
#define I2CxSR_BB               ((uint32_t)0x00000020)     /*!< BB                      */
#define I2CxSR_PIN              ((uint32_t)0x00000010)     /*!< PIN                     */
#define I2CxSR_AL               ((uint32_t)0x00000008)     /*!< AL                      */
#define I2CxSR_AAS              ((uint32_t)0x00000004)     /*!< AAS                     */
#define I2CxSR_AD0              ((uint32_t)0x00000002)     /*!< AD0                     */
#define I2CxSR_LRB              ((uint32_t)0x00000001)     /*!< LRB                     */
/**
 *  @}
 */ /* End of name I2CxSR Macro Definition */

/** 
 *  @name  I2CxPRS Macro Definition.
 *  @brief I2CxPRS Register Macro Definition.
 *  @{
 */
#define I2CxPRS_PRCK            ((uint32_t)0x0000001F)     /*!< PRCK                    */
/**
 *  @}
 */ /* End of name I2CxPRS Macro Definition */

/** 
 *  @name  I2CxIE Macro Definition.
 *  @brief I2CxIE Register Macro Definition.
 *  @{
 */
#define I2CxIE_SELPINCD_ENABLE         ((uint32_t)0x00000040)     /*!< SELPINCD enable                */
#define I2CxIE_DMARI2CTX_ENABLE        ((uint32_t)0x00000020)     /*!< DMARI2CTX enable               */
#define I2CxIE_DMARI2CRX_ENABLE        ((uint32_t)0x00000010)     /*!< DMARI2CRX enable               */
#define I2CxIE_INTNACK_ENABLE          ((uint32_t)0x00000008)     /*!< INTNACK enable                 */
#define I2CxIE_INTI2CBF_ENABLE         ((uint32_t)0x00000004)     /*!< INTI2CBF enable                */
#define I2CxIE_INTI2CAL_ENABLE         ((uint32_t)0x00000002)     /*!< INTI2CAL enable                */
#define I2CxIE_INTI2C_ENABLE           ((uint32_t)0x00000001)     /*!< INTI2C enable                  */
#define I2CxIE_SELPINCD_DISABLE        ((uint32_t)0x00000000)     /*!< SELPINCD disable               */
#define I2CxIE_DMARI2CTX_DISABLE       ((uint32_t)0x00000000)     /*!< DMARI2CTX disable              */
#define I2CxIE_DMARI2CRX_DISABLE       ((uint32_t)0x00000000)     /*!< DMARI2CRX disable              */
#define I2CxIE_INTNACK_DISABLE         ((uint32_t)0x00000000)     /*!< INTNACK disable                */
#define I2CxIE_INTI2CBF_DISABLE        ((uint32_t)0x00000000)     /*!< INTI2CBF disable               */
#define I2CxIE_INTI2CAL_DISABLE        ((uint32_t)0x00000000)     /*!< INTI2CAL disable               */
#define I2CxIE_INTI2C_DISABLE          ((uint32_t)0x00000000)     /*!< INTI2C disable                 */

#define I2CxIE_CLEAR                   ((uint32_t)0x00000000)     /*!< All Clear Setting       */
/**
 *  @}
 */ /* End of name I2CxIE Macro Definition */


/** 
 *  @name  I2CxOP Macro Definition.
 *  @brief I2CxOP Register Macro Definition.
 *  @{
 */
#define I2CxOP_DISAL            ((uint32_t)0x00000080)     /*!< DISAL                   */
#define I2CxOP_SA2ST            ((uint32_t)0x00000040)     /*!< SA2ST                   */
#define I2CxOP_SAST             ((uint32_t)0x00000020)     /*!< SAST                    */
#define I2CxOP_RSTA             ((uint32_t)0x00000008)     /*!< RSTA                    */
#define I2CxOP_GCDI             ((uint32_t)0x00000004)     /*!< GDDI                    */
#define I2CxOP_SREN             ((uint32_t)0x00000002)     /*!< SREN                    */
#define I2CxOP_MFACK            ((uint32_t)0x00000001)     /*!< MFACK                   */
#ifndef I2C_MULTI_MASTER
  #define I2CxOP_INIT             ((uint32_t)0x00000084)     /*!< Initial Settings.       */
#else
  #define I2CxOP_INIT             ((uint32_t)0x00000004)     /*!< Initial Settings.       */
#endif
#define I2CxOP_SLAVE_INIT       ((uint32_t)0x00000084)     /*!< Slave Initial Settings. */
/**
 *  @}
 */ /* End of name I2CxOP Macro Definition */
     
/** 
 *  @name  I2CxAR Macro Definition.
 *  @brief I2CxAR Register Macro Definition.
 *  @{
 */
#define I2CxAR_ALS              ((uint32_t)0x00000001)      /*!< ALS.                   */
#define I2CxAR_INIT             ((uint32_t)0x00000000)      /*!< Initial Settings.      */
#define I2CxAR2_INIT            ((uint32_t)0x00000000)      /*!< Initial Settings.      */

/**
 *  @}
 */ /* End of name I2CxAR Macro Definition */


/** 
 *  @name  I2CxPM Macro Definition.
 *  @brief I2CxPM Register Macro Definition.
 *  @{
 */
#define I2CxPM_SDA_SCL           ((uint32_t)0x00000003)      /* SDA and SCL level. */
/**
 *  @}
 */ /* End of name I2CxPM Macro Definition */
 
/** 
 *  @name  I2CxWUPCR1_INTEND Macro Definition.
 *  @brief I2CxWUPCR1_INTEND Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_INTEND_RELEASE    ((uint8_t)0x01)      /* Interrupt Release.         */
#define I2CxWUPCR1_INTEND_HOLD       ((uint8_t)0x00)      /* Interrupt setting keep it. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_INTEND Macro Definition */

/** 
 *  @name  I2CxWUPCR1_GC Macro Definition.
 *  @brief I2CxWUPCR1_GC Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_GC_DETECT     ((uint8_t)0x02)      /* General Call detect.     */
#define I2CxWUPCR1_GC_NOTDETECT  ((uint8_t)0x00)      /* General Call not detect. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_GC Macro Definition */

/** 
 *  @name  I2CxWUPCR1_RW Macro Definition.
 *  @brief I2CxWUPCR1_RW Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_RW_TRANS      ((uint8_t)0x08)      /* Slave Trans.   */
#define I2CxWUPCR1_RW_RECEIVE    ((uint8_t)0x00)      /* Salve Receive. */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_RW Macro Definition */

/** 
 *  @name  I2CxWUPCR1_I2RES Macro Definition.
 *  @brief I2CxWUPCR1_I2RES Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_I2RES_RESET   ((uint8_t)0x10)         /* I2C BUS Reset.             */
#define I2CxWUPCR1_I2RES_RELEASE ((uint8_t)0x00)         /* I2C BUS Reset Release.     */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_I2RES Macro Definition */


/** 
 *  @name  I2CxWUPCR1_ACK Macro Definition.
 *  @brief I2CxWUPC1R_ACK Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_ACK_NACK       ((uint8_t)0x20)         /* ACL No Output. Output "1" NACK Output */
#define I2CxWUPCR1_ACK_ACK        ((uint8_t)0x00)         /* ACK Output. Output "0"                */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_ACK Macro Definition */

/** 
 *  @name  I2CxWUPCR1_SGCDI Macro Definition.
 *  @brief I2CxWUPCR1_SGCDI Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_SGCDI_OFF          ((uint8_t)0x40)        /* General Call detect OFF. */
#define I2CxWUPCR1_SGCDI_ON           ((uint8_t)0x00)        /* General Call detect ON.  */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_SGCDI Macro Definition */

/** 
 *  @name  I2CxWUPCR1_BUSY Macro Definition.
 *  @brief I2CxWUPCR1_BUSY Register Macro Definition.
 *  @{
 */
#define I2CxWUPCR1_BUSY_START          ((uint8_t)0x80)        /* General Call detect OFF. */
#define I2CxWUPCR1_BUSY_STOP           ((uint8_t)0x00)        /* General Call detect ON.  */
/**
 *  @}
 */ /* End of name I2CxWUPCR1_BUSY Macro Definition */
/** 
 *  @}
 */ /* End of group I2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group I2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_functions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group I2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_functions
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_functions */

/**
 *  @}
 */ /* End of group I2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __I2C_M4K2_H */


