/** 
 *******************************************************************************
 * @file    port_m4k2.h
 * @brief   This file provides all the functions prototypes for PORT driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __PORT_M4K2_H
#define __PORT_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup PORT PORT
 *  @brief    PORT Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Exported_define PORT Exported Define
 *  @{
 */

#define PORT_BASE (0x400E0000UL) /*!< Port Register Base Adress */

/**
 *  @}
 */ /* End of group PORT_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** @defgroup PORT_Exported_Typedef PORT Exported Typedef
  * @{
  */

 
/** 
 *  @enum gpio_pa0_func_t
 *  @brief    PortA0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PA0_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PA0_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PA0_TSPI0CSIN  = GPIO_FR_1, /*!< PAFR1: TSPI0CSIN      */
  GPIO_PA0_T32A00INB0 = GPIO_FR_4, /*!< PAFR4: T32A00INB0     */
}gpio_pa0_func_t;
#endif

/** 
 *  @enum gpio_pa1_func_t
 *  @brief    PortA1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PA1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PA1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PA1_INT15      = 0,         /*!<     0: INT15          */
  GPIO_PA1_TSPI0CS1   = GPIO_FR_1, /*!< PAFR1: TSPI0CS1       */
  GPIO_PA1_T32A00INB1 = GPIO_FR_4, /*!< PAFR4: T32A00INB1     */
}gpio_pa1_func_t;
#endif

/** 
 *  @enum gpio_pa2_func_t
 *  @brief    PortA2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA2_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PA2_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PA2_INT00      = 0,         /*!<     0: INT00          */
  GPIO_PA2_TSPI0RXD   = GPIO_FR_1, /*!< PAFR1: TSPI0RXD       */
  GPIO_PA2_T32A00INA0 = GPIO_FR_4, /*!< PAFR4: T32A00INA0     */
  GPIO_PA2_T32A00INC0 = GPIO_FR_5, /*!< PAFR5: T32A00INC0     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PA2_PMD2DBG    = GPIO_FR_6, /*!< PAFR6: PMD2DBG        */
#endif
  GPIO_PA2_TRGIN0     = GPIO_FR_7, /*!< PAFR7: TRGIN0         */
}gpio_pa2_func_t;
 
 /** 
 *  @enum gpio_pa3_func_t
 *  @brief    PortA3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA3_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PA3_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PA3_INT01B     = 0,         /*!<     0: INT01b         */
  GPIO_PA3_TSPI0TXD   = GPIO_FR_1, /*!< PAFR1: TSPI0TXD       */
  GPIO_PA3_T32A00OUTA = GPIO_FR_4, /*!< PAFR4: T32A00OUTA     */
  GPIO_PA3_T32A00OUTC = GPIO_FR_5, /*!< PAFR5: T32A00OUTC     */
  GPIO_PA3_TRGIN1     = GPIO_FR_7, /*!< PAFR7: TRGIN1         */
}gpio_pa3_func_t;

 /** 
 *  @enum gpio_pa4_func_t
 *  @brief    PortA4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PA4_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PA4_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PA4_INT01A     = 0,         /*!<     0: INT01A         */
  GPIO_PA4_TSPI0SCK   = GPIO_FR_1, /*!< PAFR1: TSPI0SCK       */
  GPIO_PA4_T32A00OUTB = GPIO_FR_4, /*!< PAFR4: T32A00OUTB     */
  GPIO_PA4_TRGIN2     = GPIO_FR_7, /*!< PAFR7: TRGIN2         */
}gpio_pa4_func_t;


/** 
 *  @enum gpio_pb0_func_t
 *  @brief    PortB0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB0_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB0_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB0_UO0        = GPIO_FR_4, /*!< PBFR4: UO0            */
}gpio_pb0_func_t;
 
/** 
 *  @enum gpio_pb1_func_t
 *  @brief    PortB1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB1_XO0        = GPIO_FR_4, /*!< PBFR4: XO0            */
}gpio_pb1_func_t;
 
/** 
 *  @enum gpio_pb2_func_t
 *  @brief    PortB2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB2_INPUT      = 0,         /*!<      0: Input Port     */
  GPIO_PB2_OUTPUT     = 0,         /*!<      0: Output Port    */
  GPIO_PB2_VO0        = GPIO_FR_4, /*!<  PBFR4: VO0            */
}gpio_pb2_func_t;
 
 /** 
 *  @enum gpio_pb3_func_t
 *  @brief    PortB3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB3_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB3_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB3_YO0        = GPIO_FR_4, /*!< PBFR4: YO0            */
}gpio_pb3_func_t;

 /** 
 *  @enum gpio_pb4_func_t
 *  @brief    PortB4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB4_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB4_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB4_WO0        = GPIO_FR_4, /*!< PBFR4: WO0       */
}gpio_pb4_func_t;
 
/** 
 *  @enum gpio_pb5_func_t
 *  @brief    PortB5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PB5_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB5_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB5_ZO0        = GPIO_FR_4, /*!< PBFR4: ZO0            */
}gpio_pb5_func_t;
 
/** 
 *  @enum gpio_pb6_func_t
 *  @brief    PortB6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PB6_INPUT     = 0,         /*!<     0: Input Port     */
  GPIO_PB6_OUTPUT    = 0,         /*!<     0: Output Port    */
  GPIO_PB6_EMG0      = GPIO_FR_4, /*!< PBFR4: EMG0           */
}gpio_pb6_func_t;
#endif
/** 
 *  @enum gpio_pb7_func_t
 *  @brief    PortB7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PB7_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PB7_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PB7_OVV0       = GPIO_FR_4, /*!< PBFR4: OVV0           */
  GPIO_PB7_PMD0DBG    = GPIO_FR_5, /*!< PBFR5: PMD0DBG        */
}gpio_pb7_func_t;
#endif


/** 
 *  @enum gpio_pc0_func_t
 *  @brief    PortC0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PC0_INPUT      = 0,         /*!<     0: Input Port      */
  GPIO_PC0_OUTPUT     = 0,         /*!<     0: Output Port     */
  GPIO_PC0_UT0TXDA    = GPIO_FR_1, /*!< PCFR1: UT0TXDA         */
  GPIO_PC0_UT0RXD     = GPIO_FR_2, /*!< PCFR2: UT0RXD          */
  GPIO_PC0_EI2C0SDA   = GPIO_FR_3, /*!< PCFR3: EI2C0SDA        */
  GPIO_PC0_I2C0SDA    = GPIO_FR_4, /*!< PCFR4: I2C0SDA         */
  GPIO_PC0_T32A02INA0 = GPIO_FR_5, /*!< PCFR5: T32A02INA0      */
  GPIO_PC0_T32A02INC0 = GPIO_FR_6, /*!< PCFR6: T32A02INC0      */
}gpio_pc0_func_t;
 
/** 
 *  @enum gpio_pc1_func_t
 *  @brief    PortC1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PC1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PC1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PC1_INT02A     = 0,         /*!<     0: INT02A         */
  GPIO_PC1_UT0RXD     = GPIO_FR_1, /*!< PCFR1: UT0RXD         */
  GPIO_PC1_UT0TXDA    = GPIO_FR_2, /*!< PCFR2: UT0TXDA        */
  GPIO_PC1_EI2C0SCL   = GPIO_FR_3, /*!< PCFR3: EI2C0SCL       */
  GPIO_PC1_I2C0SCL    = GPIO_FR_4, /*!< PCFR4: I2C0SCL        */
  GPIO_PC1_T32A02OUTA = GPIO_FR_5, /*!< PCFR5: T32A02OUTA     */
  GPIO_PC1_T32A02OUTC = GPIO_FR_6, /*!< PCFR6: T32A02OUTC     */
}gpio_pc1_func_t;
 
/** 
 *  @enum gpio_pc2_func_t
 *  @brief    PortC2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PC2_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PC2_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PC2_INT10      = 0,         /*!<     0: INT10          */
  GPIO_PC2_TSPI0CS0   = GPIO_FR_3, /*!< PCFR3: TSPI0CS0       */
  GPIO_PC2_T32A03OUTA = GPIO_FR_5, /*!< PCFR5: T32A03OUTA     */
  GPIO_PC2_T32A03OUTC = GPIO_FR_6, /*!< PCFR6: T32A03OUTC     */
  GPIO_PC2_PMD0DBG    = GPIO_FR_7, /*!< PCFR7: PMD0DBG        */
}gpio_pc2_func_t;
#endif

 /** 
 *  @enum gpio_pc3_func_t
 *  @brief    PortC3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PC3_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PC3_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PC3_INT03A     = 0,         /*!<     0: INT03a         */
  GPIO_PC3_TSPI0RXD   = GPIO_FR_3, /*!< PCFR3: TSPI0RXD       */
  GPIO_PC3_T32A03OUTB = GPIO_FR_5, /*!< PCFR5: T32A03OUTB     */
  GPIO_PC3_PMD1DBG    = GPIO_FR_7, /*!< PCFR7: PMD1DBG        */
}gpio_pc3_func_t;
#endif

 /** 
 *  @enum gpio_pc4_func_t
 *  @brief    PortC4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PC4_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PC4_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PC4_UT1TXDA    = GPIO_FR_1, /*!< PCFR1: UT1TXDA        */
  GPIO_PC4_UT1RXD     = GPIO_FR_2, /*!< PCFR2: UT1RXD         */
  GPIO_PC4_TSPI0TXD   = GPIO_FR_3, /*!< PCFR3: TSPI0TXD       */
}gpio_pc4_func_t;
#endif

/** 
 *  @enum gpio_pc5_func_t
 *  @brief    PortC5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PC5_INPUT       = 0,         /*!<     0: Input Port     */
  GPIO_PC5_OUTPUT      = 0,         /*!<     0: Output Port    */
  GPIO_PC5_UT1RXD      = GPIO_FR_1, /*!< PCFR1: UT1RXD         */
  GPIO_PC5_UT1TXDA     = GPIO_FR_2, /*!< PCFR2: UT1TXDA        */
  GPIO_PC5_TSPI0SCK    = GPIO_FR_3, /*!< PCFR3: TSPI0SCK       */
}gpio_pc5_func_t;
#endif

/** 
 *  @enum gpio_pc6_func_t
 *  @brief    PortC6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PC6_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PC6_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PC6_INT02B     = 0,          /*!<     0: INT02b         */
  GPIO_PC6_TSPI0CS1   = GPIO_FR_3,  /*!< PCFR3: TSPI0CS1       */
  GPIO_PC6_T32A02INA1 = GPIO_FR_5,  /*!< PCFR5: T32A02INA1     */
  GPIO_PC6_T32A02INC1 = GPIO_FR_6,  /*!< PCFR6: T32A02INC1     */
}gpio_pc6_func_t;
#endif

/** 
 *  @enum gpio_pc7_func_t
 *  @brief    PortC7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PC7_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PC7_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PC7_TSPI0CSIN  = GPIO_FR_3,  /*!< PCFR3: TSPI0CSIN      */
  GPIO_PC7_T32A02INB0 = GPIO_FR_5,  /*!< PCFR5: T32A02INB0     */
}gpio_pc7_func_t;
#endif

/** 
 *  @enum gpio_pd0_func_t
 *  @brief    PortD0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD0_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD0_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD0_INT17B     = 0,          /*!<     0: INT17b         */
  GPIO_PD0_T32A02INB1 = GPIO_FR_4,  /*!< PDFR4: T32A02INB1     */
}gpio_pd0_func_t;
#endif

/** 
 *  @enum gpio_pd1_func_t
 *  @brief    PortD1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD1_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD1_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD1_INT17A     = 0,          /*!<     0: INT17a         */
  GPIO_PD1_T32A02OUTB = GPIO_FR_4,  /*!< PDFR4: T32A02OUTB     */
}gpio_pd1_func_t;
#endif

/** 
 *  @enum gpio_pd2_func_t
 *  @brief    PortD2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD2_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD2_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD2_INT03B     = 0,          /*!<     0: INT03b         */
  GPIO_PD2_UT0CTS_N   = GPIO_FR_1,  /*!< PDFR1: UT0CTS_N       */
  GPIO_PD2_T32A03INA0 = GPIO_FR_4,  /*!< PDFR4: T32A03INA0     */
  GPIO_PD2_T32A03INC0 = GPIO_FR_5,  /*!< PDFR5: T32A03INC0     */
}gpio_pd2_func_t;
#endif

 /** 
 *  @enum gpio_pd3_func_t
 *  @brief    PortD3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD3_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD3_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD3_UT0RTS_N   = GPIO_FR_1,  /*!< PDFR1: UT0RTS_N       */
  GPIO_PD3_I2C1SDA    = GPIO_FR_2,  /*!< PDFR2: I2C1SDA        */
  GPIO_PD3_EI2C1SDA   = GPIO_FR_3,  /*!< PDFR3: EI2C1SDA       */
  GPIO_PD3_T32A03INA1 = GPIO_FR_4,  /*!< PDFR4: T32A03INA1     */
  GPIO_PD3_T32A03INC1 = GPIO_FR_5,  /*!< PDFR5: T32A03INC1     */
  GPIO_PD3_ENC2A      = GPIO_FR_6,  /*!< PDFR6: ENC2A          */
}gpio_pd3_func_t;
#endif

/** 
 *  @enum gpio_pd4_func_t
 *  @brief    PortD4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD4_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD4_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD4_INT18B     = 0,          /*!<     0: INT18b         */
  GPIO_PD4_I2C1SCL    = GPIO_FR_2,  /*!< PDFR2: I2C1SCL        */
  GPIO_PD4_EI2C1SCL   = GPIO_FR_3,  /*!< PDFR3: EI2C1SCL       */
  GPIO_PD4_T32A03INB0 = GPIO_FR_4,  /*!< PDFR4: T32A03INB0     */
  GPIO_PD4_ENC2B      = GPIO_FR_6,  /*!< PDFR6: ENC2B          */
}gpio_pd4_func_t;
#endif

/** 
 *  @enum gpio_pd5_func_t
 *  @brief    PortD5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PD5_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PD5_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PD5_INT18A     = 0,          /*!<     0: INT18a         */
  GPIO_PD5_T32A03INB1 = GPIO_FR_4,  /*!< PDFR4: T32A03INB1     */
  GPIO_PD5_ENC2Z      = GPIO_FR_6,  /*!< PDFR6: ENC2Z          */
}gpio_pd5_func_t;
#endif

/** 
 *  @enum gpio_pe0_func_t
 *  @brief    PortE0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE0_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE0_OUTPUT     = 0,          /*!<     0: Output Port    */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PE0_UO1        = GPIO_FR_6,  /*!< PEFR6: UO1            */
#endif
}gpio_pe0_func_t;
 
/** 
 *  @enum gpio_pe1_func_t
 *  @brief    PortE1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE1_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE1_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE1_INT04B     = 0,          /*!<     0: INT04b         */
  GPIO_PE1_T32A03INA0 = GPIO_FR_4,  /*!< PEFR4: T32A03INA0     */
  GPIO_PE1_T32A03INC0 = GPIO_FR_5,  /*!< PEFR5: T32A03INC0     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PE1_XO1        = GPIO_FR_6,  /*!< PEFR6: XO1            */
#endif
}gpio_pe1_func_t;
 
/** 
 *  @enum gpio_pe2_func_t
 *  @brief    PortE2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE2_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE2_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE2_T32A03OUTA = GPIO_FR_4,  /*!< PEFR4: T32A03OUTA     */
  GPIO_PE2_T32A03OUTC = GPIO_FR_5,  /*!< PEFR5: T32A03OUTC     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PE2_VO1        = GPIO_FR_6,  /*!< PEFR6: VO1            */
#endif
}gpio_pe2_func_t;
 
 /** 
 *  @enum gpio_pe3_func_t
 *  @brief    PortE3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PE3_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE3_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE3_INT04A     = 0,          /*!<     0: INT04a         */
  GPIO_PE3_T32A03INA1 = GPIO_FR_4,  /*!< PEFR4: T32A03INA1     */
  GPIO_PE3_T32A03INC1 = GPIO_FR_5,  /*!< PEFR5: T32A03INC1     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PE3_YO1        = GPIO_FR_6,  /*!< PEFR6: YO1            */
#endif
}gpio_pe3_func_t;

 /** 
 *  @enum gpio_pe4_func_t
 *  @brief    PortE4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PE4_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE4_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE4_INT11A     = 0,          /*!<     0: INT11a         */
  GPIO_PE4_T32A03INB0 = GPIO_FR_4,  /*!< PEFR4: T32A03INB0     */
  GPIO_PE4_WO1        = GPIO_FR_6,  /*!< PEFR6: WO1            */
}gpio_pe4_func_t;
#endif

/** 
 *  @enum gpio_pe5_func_t
 *  @brief    PortE5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PE5_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE5_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE5_INT05A     = 0,          /*!<     0: INT05a         */
  GPIO_PE5_INT11B     = 0,          /*!<     0: INT11b         */
  GPIO_PE5_T32A03INB1 = GPIO_FR_4,  /*!< PEFR4: T32A03INB1     */
  GPIO_PE5_ZO1        = GPIO_FR_6,  /*!< PEFR6: ZO1            */
}gpio_pe5_func_t;
#endif

/** 
 *  @enum gpio_pe6_func_t
 *  @brief    PortE6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PE6_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE6_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE6_INT05B     = 0,          /*!<     0: INT05b         */
  GPIO_PE6_T32A03OUTB = GPIO_FR_4,  /*!< PEFR4: T32A03OUTB     */
  GPIO_PE6_EMG1       = GPIO_FR_6,  /*!< PEFR6: EMG1           */
}gpio_pe6_func_t;
#endif

/** 
 *  @enum gpio_pe7_func_t
 *  @brief    PortE7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PE7_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PE7_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PE7_OVV1       = GPIO_FR_6,  /*!< PEFR6: OVV1           */
  GPIO_PE7_PMD1DBG    = GPIO_FR_7,  /*!< PEFR7: PMD1DBG        */
}gpio_pe7_func_t;
#endif

/** 
 *  @enum gpio_pf0_func_t
 *  @brief    PortF0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PF0_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF0_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF0_UT2TXDA    = GPIO_FR_1,  /*!< PEFR1: UT2TXDA        */
  GPIO_PF0_UT2RXD     = GPIO_FR_2,  /*!< PEFR2: UT2RXD         */
  GPIO_PF0_T32A05INA0 = GPIO_FR_4,  /*!< PEFR4: T32A05INA0     */
  GPIO_PF0_T32A05INC0 = GPIO_FR_5,  /*!< PEFR5: T32A05INC0     */
}gpio_pf0_func_t;
 
/** 
 *  @enum gpio_pf1_func_t
 *  @brief    PortF1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PF1_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF1_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF1_INT06A     = 0,          /*!<     0: INT06a         */
  GPIO_PF1_UT2RXD     = GPIO_FR_1,  /*!< PEFR1: UT2RXD         */
  GPIO_PF1_UT2TXDA    = GPIO_FR_2,  /*!< PEFR2: UT2TXDA        */
  GPIO_PF1_T32A05OUTA = GPIO_FR_4,  /*!< PEFR4: T32A05OUTA     */
  GPIO_PF1_T32A05OUTC = GPIO_FR_5,  /*!< PEFR5: T32A05OUTC     */
}gpio_pf1_func_t;
 
/** 
 *  @enum gpio_pf2_func_t
 *  @brief    PortF2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PF2_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF2_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF2_INT06B     = 0,          /*!<     0: INT06b         */
  GPIO_PF2_T32A05INA1 = GPIO_FR_4,  /*!< PEFR4: T32A05INA1     */
  GPIO_PF2_T32A05INC1 = GPIO_FR_5,  /*!< PEFR5: T32A05INC1     */
}gpio_pf2_func_t;
#endif

 /** 
 *  @enum gpio_pf3_func_t
 *  @brief    PortF3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PF3_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF3_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF3_UT3TXDA    = GPIO_FR_1,  /*!< PEFR1: UT3TXDA        */
  GPIO_PF3_UT3RXD     = GPIO_FR_2,  /*!< PEFR2: UT3RXD         */
  GPIO_PF3_T32A01INA0 = GPIO_FR_4,  /*!< PEFR4: T32A01INA0     */
  GPIO_PF3_T32A01INC0 = GPIO_FR_5,  /*!< PEFR5: T32A01INC0     */
  GPIO_PF3_ENC1A      = GPIO_FR_6,  /*!< PEFR6: ENC1A          */
}gpio_pf3_func_t;
#endif

 /** 
 *  @enum gpio_pf4_func_t
 *  @brief    PortF4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PF4_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF4_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF4_INT14A     = 0,          /*!<     0: INT14a         */
  GPIO_PF4_UT3RXD     = GPIO_FR_1,  /*!< PEFR1: UT3RXD         */
  GPIO_PF4_UT3TXDA    = GPIO_FR_2,  /*!< PEFR2: UT3TXDA        */
#if defined(TMPM4KNA)
  GPIO_PF4_NBDSYNC    = GPIO_FR_3,  /*!< PEFR3: NBDSYNC        */
#endif
  GPIO_PF4_T32A01OUTA = GPIO_FR_4,  /*!< PEFR4: T32A01OUTA     */
  GPIO_PF4_T32A01OUTC = GPIO_FR_5,  /*!< PEFR5: T32A01OUTC     */
  GPIO_PF4_ENC1B      = GPIO_FR_6,  /*!< PEFR6: ENC1B          */
}gpio_pf4_func_t;
#endif

 /** 
 *  @enum gpio_pf5_func_t
 *  @brief    PortF5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PF5_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF5_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF5_INT14B     = 0,          /*!<     0: INT14b         */
  GPIO_PF5_NBDCLK     = GPIO_FR_3,  /*!< PEFR3: NBDCLK         */
  GPIO_PF5_T32A01INA1 = GPIO_FR_4,  /*!< PEFR4: T32A01INA1     */
  GPIO_PF5_T32A01INC1 = GPIO_FR_5,  /*!< PEFR5: T32A01INC1     */
  GPIO_PF5_ENC1Z      = GPIO_FR_6,  /*!< PEFR6: ENC1Z          */
  GPIO_PF5_TRACECLK   = GPIO_FR_7,  /*!< PEFR7: TRACECLK       */
}gpio_pf5_func_t;
#endif

 /** 
 *  @enum gpio_pf6_func_t
 *  @brief    PortF6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PF6_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF6_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF6_UT3TXDA    = GPIO_FR_1,  /*!< PEFR1: UT3TXDA        */
  GPIO_PF6_UT3RXD     = GPIO_FR_2,  /*!< PEFR2: UT3RXD         */
#if defined(TMPM4KNA)
  GPIO_PF6_NBDDATA0   = GPIO_FR_3,  /*!< PEFR3: NBDDATA0       */
#endif
  GPIO_PF6_T32A01INB0 = GPIO_FR_4,  /*!< PEFR4: T32A01INB0     */
#if defined(TMPM4KNA)
  GPIO_PF6_TRACEDATA0 = GPIO_FR_7,  /*!< PEFR7: TRACEDATA0     */
#endif
}gpio_pf6_func_t;
#endif

 /** 
 *  @enum gpio_pf7_func_t
 *  @brief    PortF7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PF7_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PF7_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PF7_UT3RXD     = GPIO_FR_1,  /*!< PEFR1: UT3RXD         */
  GPIO_PF7_UT3TXDA    = GPIO_FR_2,  /*!< PEFR2: UT3TXDA        */
#if defined(TMPM4KNA)
  GPIO_PF7_NBDDATA1   = GPIO_FR_3,  /*!< PEFR3: NBDDATA1       */
#endif
  GPIO_PF7_T32A01INB1 = GPIO_FR_4,  /*!< PEFR4: T32A01INB1     */
#if defined(TMPM4KNA)
  GPIO_PF7_TRACEDATA1 = GPIO_FR_7,  /*!< PEFR7: TRACEDATA1     */
#endif
}gpio_pf7_func_t;
#endif

/** 
 *  @enum gpio_pg0_func_t
 *  @brief    PortG0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PG0_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PG0_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PG0_T32A04INA0 = GPIO_FR_4,  /*!< PEFR4: T32A04INA0     */
  GPIO_PG0_T32A04INC0 = GPIO_FR_5,  /*!< PEFR5: T32A04INC0     */
}gpio_pg0_func_t;
#endif

/** 
 *  @enum gpio_pg1_func_t
 *  @brief    PortG1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PG1_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PG1_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PG1_TSPI1CS1   = GPIO_FR_1,  /*!< PEFR1: TSPI1CS1       */
  GPIO_PG1_T32A04INA1 = GPIO_FR_4,  /*!< PEFR4: T32A04INA1     */
  GPIO_PG1_T32A04INC1 = GPIO_FR_5,  /*!< PEFR5: T32A04INC1     */
}gpio_pg1_func_t;
#endif

/** 
 *  @enum gpio_pg2_func_t
 *  @brief    PortG2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PG2_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PG2_OUTPUT     = 0,         /*!<     0: Output Port    */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PG2_TSPI1CS0   = GPIO_FR_1, /*!< PGFR1: TSPI1CS0       */
#endif
  GPIO_PG2_T32A04OUTA = GPIO_FR_4, /*!< PGFR4: T32A04OUTA     */
  GPIO_PG2_T32A04OUTC = GPIO_FR_5, /*!< PGFR5: T32A04OUTC     */
}gpio_pg2_func_t;

/** 
 *  @enum gpio_pg3_func_t
 *  @brief    PortG3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PG3_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PG3_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PG3_INT21      = 0,         /*!<     0: INT21          */
  GPIO_PG3_TSPI1CSIN  = GPIO_FR_1, /*!< PGFR1: TSPI1CSIN      */
  GPIO_PG3_T32A04OUTB = GPIO_FR_4, /*!< PGFR4: T32A04OUTB     */
}gpio_pg3_func_t;
#endif

/** 
 *  @enum gpio_pg4_func_t
 *  @brief    PortG4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PG4_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PG4_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PG4_TSPI1RXD   = GPIO_FR_1, /*!< PGFR1: TSPI1RXD       */
  GPIO_PG4_T32A04INB0 = GPIO_FR_4, /*!< PGFR4: T32A04INB0     */
}gpio_pg4_func_t;
#endif

/** 
 *  @enum gpio_pg5_func_t
 *  @brief    PortG5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PG5_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PG5_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PG5_TSPI1TXD   = GPIO_FR_1, /*!< PGFR1: TSPI1TXD       */
  GPIO_PG5_T32A04INB1 = GPIO_FR_4, /*!< PGFR4: T32A04INB1     */
}gpio_pg5_func_t;
#endif

/** 
 *  @enum gpio_pg6_func_t
 *  @brief    PortG6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PG6_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PG6_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PG6_TSPI1SCK   = GPIO_FR_1, /*!< PGFR1: TSPI1SCK       */
}gpio_pg6_func_t;
#endif

/** 
 *  @enum gpio_ph0_func_t
 *  @brief    PortH0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH0_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PH0_X1         = GPIO_FR_NA, /*!< N/A: X1              */
  GPIO_PH0_EHCLKIN    = GPIO_FR_NA, /*!< N/A: EHCLKIN         */
}gpio_ph0_func_t;
 
/** 
 *  @enum gpio_ph1_func_t
 *  @brief    PortH1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PH1_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PH1_X2         = GPIO_FR_NA, /*!< N/A: X2              */
}gpio_ph1_func_t;
 

/** 
 *  @enum gpio_pj0_func_t
 *  @brief    PortJ0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PJ0_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ0_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ0_AINC00     = GPIO_FR_NA, /*!< N/A: AINC00          */
}gpio_pj0_func_t;
#endif

/** 
 *  @enum gpio_pj1_func_t
 *  @brief    PortJ1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PJ1_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ1_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ1_AINC01     = GPIO_FR_NA, /*!< N/A: AINC00          */
}gpio_pj1_func_t;
#endif

/** 
 *  @enum gpio_pj2_func_t
 *  @brief    PortJ2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PJ2_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ2_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ2_AINC02     = GPIO_FR_NA, /*!< N/A: AINC02          */
}gpio_pj2_func_t;
#endif

 /** 
 *  @enum gpio_pj3_func_t
 *  @brief    PortJ3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PJ3_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ3_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ3_AINC03     = GPIO_FR_NA, /*!< N/A: AINC03          */
}gpio_pj3_func_t;
#endif

 /** 
 *  @enum gpio_pj4_func_t
 *  @brief    PortJ4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PJ4_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ4_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ4_AINC04     = GPIO_FR_NA, /*!< N/A: AINC04          */
}gpio_pj4_func_t;
#endif

/** 
 *  @enum gpio_pj5_func_t
 *  @brief    PortJ5 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PJ5_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PJ5_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PJ5_AINC05     = GPIO_FR_NA, /*!< N/A: AINC05          */
}gpio_pj5_func_t;
#endif

/** 
 *  @enum gpio_pk0_func_t
 *  @brief    PortK0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK0_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PK0_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PK0_AINB00     = GPIO_FR_NA, /*!< N/A: AINB00          */
}gpio_pk0_func_t;
 
/** 
 *  @enum gpio_pk1_func_t
 *  @brief    PortK1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PK1_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PK1_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PK1_AINB01     = GPIO_FR_NA, /*!< N/A: AINB01          */
}gpio_pk1_func_t;
 
/** 
 *  @enum gpio_pk2_func_t
 *  @brief    PortK2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PK2_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PK2_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PK2_AINB02     = GPIO_FR_NA, /*!< N/A: AINB02          */
}gpio_pk2_func_t;
#endif

 /** 
 *  @enum gpio_pk3_func_t
 *  @brief    PortK3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{ 
  GPIO_PK3_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PK3_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PK3_AINB03     = GPIO_FR_NA, /*!< N/A: AINB03          */
}gpio_pk3_func_t;
#endif

 /** 
 *  @enum gpio_pk4_func_t
 *  @brief    PortK4 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PK4_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PK4_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PK4_AINB04     = GPIO_FR_NA, /*!< N/A: AINB04          */
}gpio_pk4_func_t;
#endif

/** 
 *  @enum gpio_pl0_func_t
 *  @brief    PortL0 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL0_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL0_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL0_AINA16     = GPIO_FR_NA, /*!< N/A: AINA16          */
}gpio_pl0_func_t;
 
/** 
 *  @enum gpio_pl1_func_t
 *  @brief    PortL1 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL1_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL1_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL1_AINA15     = GPIO_FR_NA, /*!< N/A: AINA15          */
}gpio_pl1_func_t;
 
/** 
 *  @enum gpio_pl2_func_t
 *  @brief    PortL2 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL2_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL2_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL2_AINA17     = GPIO_FR_NA, /*!< N/A: AINA17          */
}gpio_pl2_func_t;
 
 /** 
 *  @enum gpio_pl3_func_t
 *  @brief    PortL3 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL3_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL3_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL3_AINA14     = GPIO_FR_NA, /*!< N/A: AINA14          */
}gpio_pl3_func_t;

 /** 
 *  @enum gpio_pl4_func_t
 *  @brief    PortL4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL4_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL4_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL4_AINA18     = GPIO_FR_NA, /*!< N/A: AINA18          */
}gpio_pl4_func_t;
 
/** 
 *  @enum gpio_pl5_func_t
 *  @brief    PortL5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PL5_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL5_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL5_AINA13     = GPIO_FR_NA, /*!< N/A: AINA13          */
}gpio_pl5_func_t;
 
/** 
 *  @enum gpio_pl6_func_t
 *  @brief    PortL6 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PL6_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL6_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL6_AINA09     = GPIO_FR_NA, /*!< N/A: AINA09          */
}gpio_pl6_func_t;
#endif

/** 
 *  @enum gpio_pl7_func_t
 *  @brief    PortL7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PL7_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PL7_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PL7_AINA08     = GPIO_FR_NA, /*!< N/A: AINA08          */
}gpio_pl7_func_t;
#endif

/** 
 *  @enum gpio_pm0_func_t
 *  @brief    PortM0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PM0_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PM0_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PM0_AINA07     = GPIO_FR_NA, /*!< N/A: AINA07          */
}gpio_pm0_func_t;
#endif

/** 
 *  @enum gpio_pm1_func_t
 *  @brief    PortM1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PM1_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PM1_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PM1_AINA06     = GPIO_FR_NA, /*!< N/A: AINA06          */
}gpio_pm1_func_t;
#endif

/** 
 *  @enum gpio_pm2_func_t
 *  @brief    PortM2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PM2_INPUT      = GPIO_FR_NA, /*!< N/A: Input Port      */
  GPIO_PM2_OUTPUT     = GPIO_FR_NA, /*!< N/A: Output Port     */
  GPIO_PM2_AINA05     = GPIO_FR_NA, /*!< N/A: AINA05          */
}gpio_pm2_func_t;
#endif


/** 
 *  @enum gpio_pn0_func_t
 *  @brief    PortN0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PN0_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PN0_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PN0_UT0TXDA    = GPIO_FR_1, /*!< PNFR1: UT0TXDA        */
  GPIO_PN0_UT0RXD     = GPIO_FR_2, /*!< PNFR2: UT0RXD         */
#if defined(TMPM4KNA)
  GPIO_PN0_NBDDATA2   = GPIO_FR_3, /*!< PNFR3: NBDDATA2       */
#endif
  GPIO_PN0_T32A05INA0 = GPIO_FR_4, /*!< PNFR4: T32A05INA0     */
  GPIO_PN0_T32A05INC0 = GPIO_FR_5, /*!< PNFR5: T32A05INC0     */
  GPIO_PN0_ENC0A      = GPIO_FR_6, /*!< PNFR6: ENC0A          */
#if defined(TMPM4KNA)
  GPIO_PN0_TRACEDATA2 = GPIO_FR_7, /*!< PNFR7: TRACEDATA2     */
#endif
}gpio_pn0_func_t;
#endif

/** 
 *  @enum gpio_pn1_func_t
 *  @brief    PortN1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PN1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PN1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PN1_INT16A     = 0,         /*!<     0: INT16a         */
  GPIO_PN1_UT0RXD     = GPIO_FR_1, /*!< PNFR1: UT0RXD         */
  GPIO_PN1_UT0TXDA    = GPIO_FR_2, /*!< PNFR2: UT0TXDA        */
#if defined(TMPM4KNA)
  GPIO_PN1_NBDDATA3   = GPIO_FR_3, /*!< PNFR3: NBDDATA3       */
#endif
  GPIO_PN1_T32A05OUTA = GPIO_FR_4, /*!< PNFR4: T32A05OUTA     */
  GPIO_PN1_T32A05OUTC = GPIO_FR_5, /*!< PNFR5: T32A05OUTC     */
  GPIO_PN1_ENC0B      = GPIO_FR_6, /*!< PNFR6: ENC0B          */
#if defined(TMPM4KNA)
  GPIO_PN1_TRACEDATA3 = GPIO_FR_7, /*!< PNFR7: TRACEDATA3     */
#endif
}gpio_pn1_func_t;
#endif

/** 
 *  @enum gpio_pn2_func_t
 *  @brief    PortN2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA)
typedef enum
{
  GPIO_PN2_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PN2_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PN2_INT16B     = 0,         /*!<     0: INT16b         */
  GPIO_PN2_UT0CTS_N   = GPIO_FR_1, /*!< PNFR1: UT0RXD         */
  GPIO_PN2_T32A05INA1 = GPIO_FR_4, /*!< PNFR4: T32A05INA1     */
  GPIO_PN2_T32A05INC1 = GPIO_FR_5, /*!< PNFR5: T32A05INC1     */
  GPIO_PN2_ENC0Z      = GPIO_FR_6, /*!< PNFR6: ENC0Z          */
}gpio_pn2_func_t;
#endif

/** 
 *  @enum gpio_pu0_func_t
 *  @brief    PortU0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PU0_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU0_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU0_INT12      = 0,         /*!<     0: INT12          */
  GPIO_PU0_UT2TXDA    = GPIO_FR_1, /*!< PUFR1: UT2TXDA        */
  GPIO_PU0_UT2RXD     = GPIO_FR_2, /*!< PUFR2: UT2RXD         */
  GPIO_PU0_I2C1SDA    = GPIO_FR_3, /*!< PUFR3: I2C1SDA        */
  GPIO_PU0_T32A02INB1 = GPIO_FR_4, /*!< PUFR4: T32A02INB1     */
  GPIO_PU0_UO2        = GPIO_FR_6, /*!< PUFR6: UO2            */
  GPIO_PU0_EI2C1SDA   = GPIO_FR_7, /*!< PUFR7: EI2C1SDA       */
}gpio_pu0_func_t;
#endif

/** 
 *  @enum gpio_pu1_func_t
 *  @brief    PortU1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PU1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU1_INT07A     = 0,         /*!<     0: INT07a         */
  GPIO_PU1_UT2RXD     = GPIO_FR_1, /*!< PUFR1: UT2TXDA        */
  GPIO_PU1_UT2TXDA    = GPIO_FR_2, /*!< PUFR2: UT2RXD         */
  GPIO_PU1_I2C1SCL    = GPIO_FR_3, /*!< PUFR3: I2C1SCL        */
  GPIO_PU1_T32A02INA0 = GPIO_FR_4, /*!< PUFR4: T32A02INA0     */
  GPIO_PU1_T32A02INC0 = GPIO_FR_5, /*!< PUFR5: T32A02INC0     */
  GPIO_PU1_XO2        = GPIO_FR_6, /*!< PUFR6: XO2            */
  GPIO_PU1_EI2C1SCL   = GPIO_FR_7, /*!< PUFR7: EI2C1SCL       */
}gpio_pu1_func_t;
#endif

/** 
 *  @enum gpio_pu2_func_t
 *  @brief    PortU2 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PU2_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU2_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU2_INT07B     = 0,         /*!<     0: INT07b         */
  GPIO_PU2_T32A02OUTA = GPIO_FR_4, /*!< PUFR4: T32A02OUTA     */
  GPIO_PU2_T32A02OUTC = GPIO_FR_5, /*!< PUFR5: T32A02OUTC     */
  GPIO_PU2_VO2        = GPIO_FR_6, /*!< PUFR6: VO2            */
}gpio_pu2_func_t;
#endif

/** 
 *  @enum gpio_pu3_func_t
 *  @brief    PortU3 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
typedef enum
{
  GPIO_PU3_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU3_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU3_INT08A     = 0,         /*!<     0: INT08a         */
  GPIO_PU3_UT1RTS_N   = GPIO_FR_1, /*!< PUFR1: UT1RTS_N       */
  GPIO_PU3_T32A02INB0 = GPIO_FR_4, /*!< PUFR4: T32A02INB0     */
  GPIO_PU3_ENC2A      = GPIO_FR_5, /*!< PUFR5: ENC2A          */
  GPIO_PU3_YO2        = GPIO_FR_6, /*!< PUFR6: YO2            */
}gpio_pu3_func_t;
#endif

/** 
 *  @enum gpio_pu4_func_t
 *  @brief    PortU4 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PU4_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU4_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU4_INT08B     = 0,         /*!<     0: INT08b         */
  GPIO_PU4_UT1CTS_N   = GPIO_FR_1, /*!< PUFR1: UT1CTS_N       */
  GPIO_PU4_T32A02OUTB = GPIO_FR_4, /*!< PUFR4: T32A02OUTB     */
  GPIO_PU4_T32A02INC1 = GPIO_FR_5, /*!< PUFR5: T32A02INC1     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PU4_WO2        = GPIO_FR_6, /*!< PUFR6: WO2            */
#endif
}gpio_pu4_func_t;

/** 
 *  @enum gpio_pu5_func_t
 *  @brief    PortU5 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PU5_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU5_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU5_INT13      = 0,         /*!<     0: INT13          */
  GPIO_PU5_UT1TXDA    = GPIO_FR_1, /*!< PUFR1: UT1TXDA        */
  GPIO_PU5_UT1RXD     = GPIO_FR_2, /*!< PUFR2: UT1RXD         */
  GPIO_PU5_T32A02INA1 = GPIO_FR_4, /*!< PUFR4: T32A02INA1     */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PU5_ENC2B      = GPIO_FR_5, /*!< PUFR5: ENC2B          */
  GPIO_PU5_ZO2        = GPIO_FR_6, /*!< PUFR6: ZO2            */
#endif
}gpio_pu5_func_t;

/** 
 *  @enum gpio_pu6_func_t
 *  @brief    PortU6 Function Enumerated Type Definition.
 */
typedef enum
{
  GPIO_PU6_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU6_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU6_INT09      = 0,         /*!<     0: INT09          */
  GPIO_PU6_UX1RXD     = GPIO_FR_1, /*!< PUFR1: UX1RXD         */
  GPIO_PU6_UT1TXDA    = GPIO_FR_2, /*!< PUFR2: UT1TXDA        */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
  GPIO_PU6_ENC2Z      = GPIO_FR_5, /*!< PUFR5: ENC2Z          */
  GPIO_PU6_EMG2       = GPIO_FR_6, /*!< PUFR6: EMG2           */
#endif
}gpio_pu6_func_t;

/** 
 *  @enum gpio_pu7_func_t
 *  @brief    PortU7 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PU7_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PU7_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PU7_OVV2       = GPIO_FR_6, /*!< PUFR6: OVV2           */
  GPIO_PU7_PMD2DBG    = GPIO_FR_7, /*!< PUFR7: PMD2DBG        */
}gpio_pu7_func_t;
#endif

/** 
 *  @enum gpio_pv0_func_t
 *  @brief    PortV0 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PV0_INPUT      = 0,          /*!<     0: Input Port     */
  GPIO_PV0_OUTPUT     = 0,          /*!<     0: Output Port    */
  GPIO_PV0_TSPI1CSIN  = GPIO_FR_2,  /*!< PVFR2: TSPI1CSIN      */
  GPIO_PV0_T32A01OUTB = GPIO_FR_4,  /*!< PVFR4: T32A01OUTB     */
}gpio_pv0_func_t;
#endif

/** 
 *  @enum gpio_pv1_func_t
 *  @brief    PortV1 Function Enumerated Type Definition.
 */
#if defined(TMPM4KNA)
typedef enum
{
  GPIO_PV1_INPUT      = 0,         /*!<     0: Input Port     */
  GPIO_PV1_OUTPUT     = 0,         /*!<     0: Output Port    */
  GPIO_PV1_UT0RTS_N   = GPIO_FR_1, /*!< PVFR1: UT0RTS_N       */
  GPIO_PV1_TSPI1RXD   = GPIO_FR_2, /*!< PVFR2: TSPI1RXD       */
}gpio_pv1_func_t;
#endif

/**
 *  @} 
 */ /* End of group PORT_Exported_Typedef */
 
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup PORT_Exported_Typedef PORT Exported Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  PORT handle structure definenition.
*/
/*----------------------------------*/
typedef struct gpio_pa_handle
{  
    TSB_PA_TypeDef       *p_pa_instance;     /*!< Registers base address.           */
    TSB_PB_TypeDef       *p_pb_instance;     /*!< Registers base address.           */
    TSB_PC_TypeDef       *p_pc_instance;     /*!< Registers base address.           */
#if defined(TMPM4KNA)
    TSB_PD_TypeDef       *p_pd_instance;     /*!< Registers base address.           */
#endif
    TSB_PE_TypeDef       *p_pe_instance;     /*!< Registers base address.           */
    TSB_PF_TypeDef       *p_pf_instance;     /*!< Registers base address.           */
    TSB_PG_TypeDef       *p_pg_instance;     /*!< Registers base address.           */
    TSB_PH_TypeDef       *p_ph_instance;     /*!< Registers base address.           */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
    TSB_PJ_TypeDef       *p_pj_instance;     /*!< Registers base address.           */
#endif
    TSB_PK_TypeDef       *p_pk_instance;     /*!< Registers base address.           */
    TSB_PL_TypeDef       *p_pl_instance;     /*!< Registers base address.           */
#if defined(TMPM4KNA)
    TSB_PM_TypeDef       *p_pm_instance;     /*!< Registers base address.           */
#endif
#if defined(TMPM4KNA) || defined(TMPM4KMA)
    TSB_PN_TypeDef       *p_pn_instance;     /*!< Registers base address.           */
#endif
    TSB_PU_TypeDef       *p_pu_instance;     /*!< Registers base address.           */
#if defined(TMPM4KNA)
    TSB_PV_TypeDef       *p_pv_instance;     /*!< Registers base address.           */
#endif
} gpio_t;

/**
 *  @}
 */ /* End of group PORT_Exported_Typedef */

/*------------------------------------------------------------------------------*/
/*  Table Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_typedef PORT Private Typedef
 *  @{
 */
/*!
  * @brief  Pin Exist Table
  * @details Bit0 :GPIO_Mode_DATA
  * @details Bit1 :GPIO_Mode_CR
  * @details Bit2 :GPIO_Mode_FR1
  * @details Bit3 :GPIO_Mode_FR2
  * @details Bit4 :GPIO_Mode_FR3
  * @details Bit5 :GPIO_Mode_FR4
  * @details Bit6 :GPIO_Mode_FR5
  * @details Bit7 :GPIO_Mode_FR6
  * @details Bit8 :GPIO_Mode_FR7
  * @details Bit9 :-
  * @details Bit10 :GPIO_Mode_OD
  * @details Bit11 :GPIO_Mode_PUP
  * @details Bit12 :GPIO_Mode_PDN
  * @details Bit13 :-
  * @details Bit14 :GPIO_Mode_IE
  * @details Bit15 :-
  */

#if defined(TMPM4KNA)
static uint16_t PinExistTbl[GPIO_GROUP_Max][GPIO_PORT_Max] =
{
   /* Port-0  Port-1  Port-2  Port-3  Port-4  Port-5  Port-6  Port-7 */
    { 0x5C27, 0x5C27, 0x5DE7, 0x5D67, 0x5D27, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_A */
    { 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C63 }, /** GPIO_PORT_B */
    { 0x5CFF, 0x5CFF, 0x5DD3, 0x5D53, 0x5C1F, 0x5C1F, 0x5CD3, 0x5C53 }, /** GPIO_PORT_C */
    { 0x5C23, 0x5C23, 0x5C67, 0x5CFF, 0x5CBB, 0x5CA3, 0x0000, 0x0000 }, /** GPIO_PORT_D */
    { 0x5C83, 0x5CE3, 0x5CE3, 0x5CE3, 0x5CA3, 0x5CA3, 0x5CA3, 0x5D83 }, /** GPIO_PORT_E */
    { 0x5D6F, 0x5D6F, 0x5D63, 0x5DEF, 0x5DFF, 0x5DF3, 0x5D3F, 0x5D3F }, /** GPIO_PORT_F */
    { 0x5C63, 0x5C67, 0x5C67, 0x5C27, 0x5C27, 0x5C27, 0x5C07, 0x0000 }, /** GPIO_PORT_G */
    { 0x5001, 0x5001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_H */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000 }, /** GPIO_PORT_J */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_K */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03 }, /** GPIO_PORT_L */
    { 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_M */
    { 0x5DFF, 0x5DFF, 0x5CE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_N */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_P */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_R */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_T */
    { 0x5DBF, 0x5DFF, 0x5CE3, 0x5CE7, 0x5CE7, 0x5CEF, 0x5CCF, 0x5D83 }, /** GPIO_PORT_U */
    { 0x5C2B, 0x5C0F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_V */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_W */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_Y */
};
#elif defined(TMPM4KMA)
static uint16_t PinExistTbl[GPIO_GROUP_Max][GPIO_PORT_Max] =
{
   /* Port-0  Port-1  Port-2  Port-3  Port-4  Port-5  Port-6  Port-7 */
    { 0x5C27, 0x5C27, 0x5DE7, 0x5D67, 0x5D27, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_A */
    { 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x0000 }, /** GPIO_PORT_B */
    { 0x5CFF, 0x5CFF, 0x5DD3, 0x5D53, 0x5C1F, 0x5C1F, 0x0000, 0x0000 }, /** GPIO_PORT_C */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_D */
    { 0x5C83, 0x5CE3, 0x5CE3, 0x5CE3, 0x5CA3, 0x5CA3, 0x5CA3, 0x0000 }, /** GPIO_PORT_E */
    { 0x5D6F, 0x5D6F, 0x0000, 0x5DEF, 0x5DFF, 0x0000, 0x5D3F, 0x5D3F }, /** GPIO_PORT_F */
    { 0x5C63, 0x5C67, 0x5C67, 0x5C27, 0x5C27, 0x5C27, 0x5C07, 0x0000 }, /** GPIO_PORT_G */
    { 0x5001, 0x5001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_H */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_J */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_K */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03 }, /** GPIO_PORT_L */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_M */
    { 0x5DFF, 0x5DFF, 0x5CE7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_N */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_P */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_R */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_T */
    { 0x5DBF, 0x5DFF, 0x5CE3, 0x5CE7, 0x5CE7, 0x5CEF, 0x5CCF, 0x0000 }, /** GPIO_PORT_U */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_V */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_W */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_Y */
};
#elif defined(TMPM4KLA)
static uint16_t PinExistTbl[GPIO_GROUP_Max][GPIO_PORT_Max] =
{
   /* Port-0  Port-1  Port-2  Port-3  Port-4  Port-5  Port-6  Port-7 */
    { 0x0000, 0x0000, 0x5DE7, 0x5D67, 0x5D27, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_A */
    { 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x0000 }, /** GPIO_PORT_B */
    { 0x5CFF, 0x5CFF, 0x5DD3, 0x5D53, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_C */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_D */
    { 0x5C83, 0x5CE3, 0x5CE3, 0x5CE3, 0x5CA3, 0x5CA3, 0x5CA3, 0x0000 }, /** GPIO_PORT_E */
    { 0x5D6F, 0x5D6F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_F */
    { 0x0000, 0x0000, 0x5C67, 0x5C27, 0x5C27, 0x5C27, 0x5C07, 0x0000 }, /** GPIO_PORT_G */
    { 0x5001, 0x5001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_H */
    { 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_J */
    { 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_K */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03 }, /** GPIO_PORT_L */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_M */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_N */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_P */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_R */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_T */
    { 0x5DBF, 0x5DFF, 0x5CE3, 0x5CE7, 0x5CE7, 0x5CEF, 0x5CCF, 0x0000 }, /** GPIO_PORT_U */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_V */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_W */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_Y */
};
#elif defined(TMPM4KHA)
static uint16_t PinExistTbl[GPIO_GROUP_Max][GPIO_PORT_Max] =
{
   /* Port-0  Port-1  Port-2  Port-3  Port-4  Port-5  Port-6  Port-7 */
    { 0x0000, 0x0000, 0x5D67, 0x5D67, 0x5D27, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_A */
    { 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x5C23, 0x0000, 0x0000 }, /** GPIO_PORT_B */
    { 0x5CFF, 0x5CFF, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_C */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_D */
    { 0x5C03, 0x5C63, 0x5C63, 0x5C63, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_E */
    { 0x5D6F, 0x5D6F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_F */
    { 0x0000, 0x0000, 0x5C63, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_G */
    { 0x5001, 0x5001, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_H */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_J */
    { 0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_K */
    { 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x5C03, 0x0000, 0x0000 }, /** GPIO_PORT_L */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_M */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_N */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_P */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_R */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_T */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x5C67, 0x5C2F, 0x5C0F, 0x0000 }, /** GPIO_PORT_U */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_V */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_W */
    { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 }, /** GPIO_PORT_Y */
};
#endif

/*!
  * @brief  Register Exist Table
  * @details Bit0 :GPIO_Mode_DATA
  * @details Bit1 :GPIO_Mode_CR
  * @details Bit2 :GPIO_Mode_FR1
  * @details Bit3 :GPIO_Mode_FR2

  * @details Bit4 :GPIO_Mode_FR3
  * @details Bit5 :GPIO_Mode_FR4
  * @details Bit6 :GPIO_Mode_FR5
  * @details Bit7 :GPIO_Mode_FR6

  * @details Bit8 :GPIO_Mode_FR7
  * @details Bit9 :-
  * @details Bit10 :GPIO_Mode_OD
  * @details Bit11 :GPIO_Mode_PUP

  * @details Bit12 :GPIO_Mode_PDN
  * @details Bit13 :-
  * @details Bit14 :GPIO_Mode_IE
  * @details Bit15 :-
  */

#if defined(TMPM4KNA)
static uint16_t RegExistTbl[GPIO_GROUP_Max]=
   /* Port-A  Port-B  Port-C  Port-D  Port-E  Port-F  Port-G  Port-H  Port-J */
    { 0x5DE7, 0x5C63, 0x5DFF, 0x5CFF, 0x5DE3, 0x5DFF, 0x5C67, 0x5001, 0x5C03,
   /* Port-K  Port-L  Port-M  Port-N  Port-P  Port-R  Port-T  Port-U  Port-V  Port-W  Port-Y */
      0x5C03, 0x5C03, 0x5C03, 0x5DFF, 0x0000, 0x0000, 0x0000, 0x5DFF, 0x5C2F, 0x0000, 0x0000 };
#elif defined(TMPM4KMA)
static uint16_t RegExistTbl[GPIO_GROUP_Max]=
   /* Port-A  Port-B  Port-C  Port-D  Port-E  Port-F  Port-G  Port-H  Port-J */
    { 0x5DE7, 0x5C23, 0x5DFF, 0x0000, 0x5CE3, 0x5DEF, 0x5C67, 0x5001, 0x5C03,
   /* Port-K  Port-L  Port-M  Port-N  Port-P  Port-R  Port-T  Port-U  Port-V  Port-W  Port-Y */
      0x5C03, 0x5C03, 0x0000, 0x5CEF, 0x0000, 0x0000, 0x0000, 0x5DFF, 0x0000, 0x0000, 0x0000 };
#elif defined(TMPM4KLA)
static uint16_t RegExistTbl[GPIO_GROUP_Max]=
   /* Port-A  Port-B  Port-C  Port-D  Port-E  Port-F  Port-G  Port-H  Port-J */
    { 0x5DE7, 0x5C23, 0x5DFF, 0x0000, 0x5CE3, 0x5D6F, 0x5C67, 0x5001, 0x5C03,
   /* Port-K  Port-L  Port-M  Port-N  Port-P  Port-R  Port-T  Port-U  Port-V  Port-W  Port-Y */
      0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x5DFF, 0x0000, 0x0000, 0x0000 };
#elif defined(TMPM4KHA)
static uint16_t RegExistTbl[GPIO_GROUP_Max]=
   /* Port-A  Port-B  Port-C  Port-D  Port-E  Port-F  Port-G  Port-H  Port-J */
    { 0x5DE7, 0x5C23, 0x5CFF, 0x0000, 0x5C63, 0x5D6F, 0x5C63, 0x5001, 0x0000,
   /* Port-K  Port-L  Port-M  Port-N  Port-P  Port-R  Port-T  Port-U  Port-V  Port-W  Port-Y */
      0x5C03, 0x5C03, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x5CEF, 0x0000, 0x0000, 0x0000 };
#endif
/*!
  * @brief  Pin Reset Table
  */

static uint16_t PinResetTbl[GPIO_GROUP_Max][GPIO_Mode_Max] =
{
   /* DATA  CR    FR1   FR2   FR3   FR4   FR5   FR6   FR7   FR8   OD    PUP   PDN   RSV0  IE    RSV1 */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_A */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_B */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_C */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_D */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_E */
    { 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 0x19, 0x02, 0x00, 0x1B, 0x00 }, /** GPIO_PORT_F */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_G */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_H */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_J */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_K */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_L */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_M */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_N */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_P */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_R */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_T */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_U */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_V */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }, /** GPIO_PORT_W */
    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }  /** GPIO_PORT_Y */
};

/*!
  * @brief  Reg CG Enable Table
  */
#define REG_NONE        (0x00000000)
#define REG_CG_FSYSMENA ((uint32_t *)&TSB_CG->FSYSMENA)
typedef struct reg_cg_enable
{
    uint32_t *addr;
    uint32_t bit;
} reg_cg_enable_t;
static reg_cg_enable_t RegCgEnableTbl[GPIO_GROUP_Max] =
{
    { REG_CG_FSYSMENA, 0  }, /** GPIO_PORT_A */
    { REG_CG_FSYSMENA, 1  }, /** GPIO_PORT_B */
    { REG_CG_FSYSMENA, 2  }, /** GPIO_PORT_C */
#if defined(TMPM4KNA)
    { REG_CG_FSYSMENA, 3  }, /** GPIO_PORT_D */
#else
    { REG_NONE,        0  }, /** GPIO_PORT_D */
#endif
    { REG_CG_FSYSMENA, 4  }, /** GPIO_PORT_E */
    { REG_CG_FSYSMENA, 5  }, /** GPIO_PORT_F */
    { REG_CG_FSYSMENA, 6  }, /** GPIO_PORT_G */
    { REG_CG_FSYSMENA, 7  }, /** GPIO_PORT_H */
#if defined(TMPM4KNA) || defined(TMPM4KMA) || defined(TMPM4KLA)
    { REG_CG_FSYSMENA, 8  }, /** GPIO_PORT_J */
#else
    { REG_NONE,        0  }, /** GPIO_PORT_J */
#endif
    { REG_CG_FSYSMENA, 9  }, /** GPIO_PORT_K */
    { REG_CG_FSYSMENA, 10 }, /** GPIO_PORT_L */
#if defined(TMPM4KNA)
    { REG_CG_FSYSMENA, 11 }, /** GPIO_PORT_M */
#else
    { REG_NONE,        0  }, /** GPIO_PORT_M */
#endif
#if defined(TMPM4KNA) || defined(TMPM4KMA)
    { REG_CG_FSYSMENA, 12 }, /** GPIO_PORT_N */
#else
    { REG_NONE,        0  }, /** GPIO_PORT_N */
#endif
    { REG_NONE,        0  }, /** GPIO_PORT_P */
    { REG_NONE,        0  }, /** GPIO_PORT_R */
    { REG_NONE,        0  }, /** GPIO_PORT_T */
    { REG_CG_FSYSMENA, 16 }, /** GPIO_PORT_U */
#if defined(TMPM4KNA)
    { REG_CG_FSYSMENA, 17 }, /** GPIO_PORT_V */
#else
    { REG_NONE,        0  }, /** GPIO_PORT_V */
#endif
    { REG_NONE,        0  }, /** GPIO_PORT_W */
    { REG_NONE,        0  }  /** GPIO_PORT_Y */
};

/**
 *  @}
 */ /* End of group PORT_Private_typedef */
 

 
/**
 *  @}
 */ /* End of group PORT */
 
/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __PORT_M4K2_H */
