/** 
 *******************************************************************************
 * @file    siwdt_m4k2.h
 * @brief   This file provides all the functions prototypes for SIWDT Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __SIWDT_M4K2_H
#define __SIWDT_M4K2_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup SIWDT
 *  @brief      SIWDT Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_define
 *  @{
 */

#define AVAILABLE           (1)     /*!< SIWDT function equipped */
#define UNAVAILABLE         (0)     /*!< SIWDT function not equipped */

#define OSCCR_REGISTER_EQUIPPED     AVAILABLE

/* set AVAILABLE either USE_FSYS4_AS_SIWDT_CLOCK or USE_FSYSM_AS_SIWDT_CLOCK */
#define USE_FSYS4_AS_SIWDT_CLOCK    UNAVAILABLE
#define USE_FSYSM_AS_SIWDT_CLOCK    AVAILABLE

/* SIWDxMOD_WDCLS bit value: different name, same value */
#if (USE_FSYS4_AS_SIWDT_CLOCK == AVAILABLE)
#define SIWDxMOD_WDCLS_FSYS_VALUE       ((uint32_t)0x00000000)
#define SIWDxMOD_WDCLS_FSYS4            (SIWDxMOD_WDCLS_FSYS_VALUE) /*!< WDCLS  :[R/W] :fsys/4 */
#endif
#if (USE_FSYSM_AS_SIWDT_CLOCK == AVAILABLE)
#define SIWDxMOD_WDCLS_FSYS_VALUE       ((uint32_t)0x00000000)
#define SIWDxMOD_WDCLS_FSYSM            (SIWDxMOD_WDCLS_FSYS_VALUE) /*!< WDCLS  :[R/W] :fsysm  */
#endif

/** 
 *  @}
 */ /* End of group SIWDT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_define
 *  @{
 */
#define SIWDxPRO_PROTECT_NONE           ((uint32_t)0x0000001E)    /*!< PROTECT  :[R/W] :Protect none    */
#define SIWDxPRO_PROTECT_A              ((uint32_t)0x000000A9)    /*!< PROTECT  :[R/W] :Protect A       */
#define SIWDxPRO_PROTECT_B              ((uint32_t)0x00000074)    /*!< PROTECT  :[R/W] :Protect B       */

#define SIWDxEN_WDTE_RW_DISABLE         ((uint32_t)0x00000000)    /*!< WDTE     :[R/W] :Disable         */
#define SIWDxEN_WDTE_RW_ENABLE          ((uint32_t)0x00000001)    /*!< WDTE     :[R/W] :Enable          */

#define SIWDxEN_WDTF_START              ((uint32_t)0x00000002)    /*!< WDTF     :[R] :Start             */
#define SIWDxEN_WDTF_STOP               ((uint32_t)0x00000000)    /*!< WDTF     :[R] :Stop              */

#define SIWDxCR_WDCR_W_DISABLE          ((uint32_t)0x000000B1)    /*!< WDCR     :[W] :Disable           */
#define SIWDxCR_WDCR_W_CLEAR            ((uint32_t)0x0000004E)    /*!< WDCR     :[W] :Clear             */

#define SIWDxMOD_WDCLS_FSYS             (SIWDxMOD_WDCLS_FSYS_VALUE) /*!< WDCLS  :[R/W] :fsys/4 or fsysm */
#define SIWDxMOD_WDCLS_FIHOSC1          ((uint32_t)0x00001000)    /*!< WDCLS    :[R/W] :fIHOSC1         */
#define SIWDxMOD_WDCLS_FIHOSC2          ((uint32_t)0x00002000)    /*!< WDCLS    :[R/W] :fIHOSC2         */

#define SIWDxMOD_WDTP_2_15              ((uint32_t)0x00000000)    /*!< WDTP     :[R/W] :2^15/fWD        */
#define SIWDxMOD_WDTP_2_17              ((uint32_t)0x00000100)    /*!< WDTP     :[R/W] :2^17/fWD        */
#define SIWDxMOD_WDTP_2_19              ((uint32_t)0x00000200)    /*!< WDTP     :[R/W] :2^19/fWD        */
#define SIWDxMOD_WDTP_2_21              ((uint32_t)0x00000300)    /*!< WDTP     :[R/W] :2^21/fWD        */
#define SIWDxMOD_WDTP_2_23              ((uint32_t)0x00000400)    /*!< WDTP     :[R/W] :2^23/fWD        */
#define SIWDxMOD_WDTP_2_25              ((uint32_t)0x00000500)    /*!< WDTP     :[R/W] :2^25/fWD        */
#define SIWDxMOD_WDTP_2_27              ((uint32_t)0x00000600)    /*!< WDTP     :[R/W] :2^27/fWD        */
#define SIWDxMOD_WDTP_2_29              ((uint32_t)0x00000700)    /*!< WDTP     :[R/W] :2^29/fWD        */

#define SIWDxMOD_WDCWD_NONE             ((uint32_t)0x00000000)    /*!< WDCMD    :[R/W] :none            */
#define SIWDxMOD_WDCWD_2                ((uint32_t)0x00000010)    /*!< WDCMD    :[R/W] :1/2             */
#define SIWDxMOD_WDCWD_4                ((uint32_t)0x00000020)    /*!< WDCMD    :[R/W] :1/4             */

#define SIWDxMOD_INTF_OCCUR             ((uint32_t)0x00000002)    /*!< INTF     :[R] :occur             */
#define SIWDxMOD_INTF_NOTOCCUR          ((uint32_t)0x00000000)    /*!< INTF     :[R] :not occur         */

#define SIWDxMOD_RESCR_INT              ((uint32_t)0x00000000)    /*!< RESCR    :[R/W] :INTWDT          */
#define SIWDxMOD_RESCR_RESET            ((uint32_t)0x00000001)    /*!< RESCR    :[R/W] :reset           */

#if (OSCCR_REGISTER_EQUIPPED == AVAILABLE)
#define SIWDxOSCCR_OSCPRO_ENABLE        ((uint32_t)0x00000001)    /*!< OSCPRO   :[R/W] :Protect Enable  */
#define SIWDxOSCCR_OSCPRO_DISABLE       ((uint32_t)0x00000000)    /*!< OSCPRO   :[R/W] :Protect Disable */
#endif
/** 
 *  @}
 */ /* End of group SIWDT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SIWDT_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group SIWDT_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_functions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group SIWDT_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup SIWDT_Private_functions
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group SIWDT_Private_functions */

/**
 *  @}
 */ /* End of group SIWDT */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __SIWDT_M4K2_H */


