/** 
 *******************************************************************************
 * @file    flash.h
 * @brief   This file is wrapper file for FLASH driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"
#include "flash_config.h"

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Flash_Exported_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    fc_sr0_t status;                                /*!< status setting.                   */
    uint32_t* src_address;                          /*!< src address setting.              */
    uint32_t* dst_address;                          /*!< dst address setting.              */
    uint32_t size;                                  /*!< size setting.                     */
    fc_code_flash_page_number_t first_page_code;    /*!< code flash first page setting.    */
    uint8_t num_of_pages_code;                      /*!< code flash page number setting.   */
    fc_code_flash_page_number_t last_page_code;     /*!< code flash last page setting.     */
    fc_data_flash_page_number_t first_page_data;    /*!< data flash first page setting.    */
    uint8_t num_of_pages_data;                      /*!< data flash page number setting.   */
    fc_data_flash_page_number_t last_page_data;     /*!< data flash last page setting.     */
    fc_data_flash_block_number_t first_block;       /*!< first block setting.              */
    uint8_t num_of_block;                           /*!< block number setting.             */
    fc_data_flash_block_number_t last_block;        /*!< last block setting.               */
    fc_data_flash_area_number_t area;               /*!< area setting.                     */
} flash_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  flash structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_FC_TypeDef       *p_instance;     /*!< Registers base address.           */
    flash_initial_setting_t init;         /*!< Initial setting.                  */
} flash_t;

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DNF_Exported_functions
 *  @{
 */
/** @weakgroup FLASH_Exported_FunctionPrototypes
  * @{
  */
TXZ_WorkState fc_get_status(flash_t *p_obj);
TXZ_Result fc_write_code_flash(flash_t *p_obj);
TXZ_Result fc_erase_page_code_flash(flash_t *p_obj);
TXZ_Result fc_erase_page_data_flash(flash_t *p_obj);
TXZ_Result fc_blank_check_page_code_flash(flash_t *p_obj);
TXZ_Result fc_erase_block_data_flash(flash_t *p_obj);
TXZ_Result fc_blank_check_page_data_flash(flash_t *p_obj);
TXZ_Result fc_blank_check_block_data_flash(flash_t *p_obj);
TXZ_Result fc_erase_area_data_flash(flash_t *p_obj);
TXZ_Result fc_blank_check_area_data_flash(flash_t *p_obj);
TXZ_Result fc_write_data_flash(flash_t *p_obj);
TXZ_Result fc_verify_check(uint32_t* src_address, uint32_t* dst_address, uint32_t size);
TXZ_Result fc_write_user_information_area(flash_t *p_obj);
TXZ_Result fc_erase_user_information_area(flash_t *p_obj);
void fc_read_user_information_area(flash_t *p_obj);
void fc_read_clock_set(flash_t *p_obj);
void fc_fixed_clock_set(flash_t *p_obj, uint32_t sysclock);
TXZ_Result fc_protect_page_code_flash_set(flash_t *p_obj, fc_code_flash_page_number_t page);
TXZ_Result fc_protect_block_code_flash_set(flash_t *p_obj, fc_code_flash_block_number_t block);
TXZ_Result fc_protect_block_data_flash_set(flash_t *p_obj, fc_data_flash_block_number_t block);
TXZ_Result fc_protect_page_code_flash_clear(flash_t *p_obj, fc_code_flash_page_number_t page);
TXZ_Result fc_protect_block_code_flash_clear(flash_t *p_obj, fc_code_flash_block_number_t block);
TXZ_Result fc_protect_block_data_flash_clear(flash_t *p_obj, fc_data_flash_block_number_t block);
TXZ_WorkState fc_protect_page_code_flash_status(flash_t *p_obj, fc_code_flash_page_number_t page);
TXZ_WorkState fc_protect_block_code_flash_status(flash_t *p_obj, fc_code_flash_block_number_t block);
TXZ_WorkState fc_protect_block_data_flash_status(flash_t *p_obj, fc_data_flash_block_number_t block);
TXZ_Result fc_security_set(flash_t *p_obj);
TXZ_Result fc_security_clear(flash_t *p_obj);
TXZ_WorkState fc_security_status(flash_t *p_obj);
void fc_read_buf_set(flash_t *p_obj, TXZ_FunctionalState state);
/** @} */
/* End of group FLASH_Exported_FunctionPrototypes */

#ifdef __cplusplus
}
#endif /* __cplusplus */
