/** 
 *******************************************************************************
 * @file    i2c.h
 * @brief   This file provides all the functions prototypes for I2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __I2C_H
#define __I2C_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"
#include "i2c_config.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup I2C
 *  @brief      I2C Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Clock setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t               sck;            /*!< Select internal SCL output clock frequency.                 */
    uint32_t               prsck;          /*!< Prescaler clock frequency for generating the Serial clock.  */
} I2C_clock_setting_t;

/*----------------------------------*/
/** 
 * @brief  Clock setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t               selpincd;            /*!< SELPINCD Setting
                                                : Use @ref I2CxIE_SELPINCD_xxx */
    uint32_t               dmari2ctx;           /*!< DMARI2CTX Setting
                                                : Use @ref I2CxIE_DMARI2CTX_xxx */
    uint32_t               dmari2crx;           /*!< DMARI2CRX Setting
                                                : Use @ref I2CxIE_DMARI2CRX_xxx */
    uint32_t               intnack;             /*!< INTNACK Setting
                                                : Use @ref I2CxIE_INTNACK_xxx */
    uint32_t               inti2cbf;            /*!< INTI2CBF Setting
                                                : Use @ref I2CxIE_INTI2CBF_xxx */
    uint32_t               inti2cal;            /*!< INTI2CAL Setting
                                                : Use @ref I2CxIE_INTI2CAL_xxx */
    uint32_t               inti2c;              /*!< INTI2C Setting
                                                : Use @ref I2CxIE_INTI2C_xxx */
} I2C_interrupt_t;

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    I2C_clock_setting_t  clock;             /*!< Serial clock setting.            */
    I2C_interrupt_t interrupt;              /*!< interrupt setting.               */
} I2C_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  I2C handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_I2C_TypeDef       *p_instance;     /*!< Registers base address.           */
    I2C_initial_setting_t init;            /*!< Initial setting.                  */
} I2C_t;

#if (I2CS_EQUIPPED == AVAILABLE)
/*----------------------------------*/
/** 
 * @brief  Wakeup Control setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t               sgcdi;          /*!< Select general call detect ON/OFF.
                                                : Use @ref I2CxWUPCR1_SGCDI_xxx */
    uint8_t               ack;            /*!< Select ACK output.
                                                : Use @ref I2CxWUPCR1_ACK_xxx */
    uint8_t               i2res;          /*!< I2C BUS Rest.
                                                : Use @ref I2CxWUPCR1_I2RES_xxx */
    uint8_t               intend;         /*!< Interrupt release.
                                                : Use @ref I2CxWUPCR1_INTEND_xxx */
} I2CS_wup_setting_t;

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    I2CS_wup_setting_t    wup;              /*!< Wakeup Control setting.          */
} I2CS_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  I2CS handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_I2CS_TypeDef       *p_instance;     /*!< Registers base address.           */
    I2CS_initial_setting_t init;            /*!< Initial setting.                  */
} I2CS_t;
#endif
/**
 *  @}
 */ /* End of group I2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_functions
 *  @{
 */
__STATIC_INLINE void I2C_reset(I2C_t *p_obj);
__STATIC_INLINE int32_t I2C_port_high(I2C_t *p_obj);
__STATIC_INLINE void I2C_stop_condition(I2C_t *p_obj);
__STATIC_INLINE uint32_t I2C_read_data(I2C_t *p_obj);
__STATIC_INLINE void I2C_write_data(I2C_t *p_obj, uint32_t data);
__STATIC_INLINE int32_t I2C_restart(I2C_t *p_obj);
__STATIC_INLINE void I2C_set_ack(I2C_t *p_obj, int32_t nack);
__STATIC_INLINE int32_t I2C_get_ack(I2C_t *p_obj);
__STATIC_INLINE int32_t I2C_status_busy(I2C_t *p_obj);
__STATIC_INLINE int32_t I2C_master(I2C_t *p_obj);
__STATIC_INLINE int32_t I2C_transmitter(I2C_t *p_obj);
__STATIC_INLINE int32_t I2C_int_status(I2C_t *p_obj);
__STATIC_INLINE void I2C_clear_int_status(I2C_t *p_obj);
__STATIC_INLINE void I2C_enable_interrupt(I2C_t *p_obj);
__STATIC_INLINE void I2C_enable_interrupt_dma(I2C_t *p_obj, int32_t tx);
__STATIC_INLINE void I2C_disable_interrupt(I2C_t *p_obj);
__STATIC_INLINE void I2C_set_address(I2C_t *p_obj, uint32_t addr);
__STATIC_INLINE int32_t I2C_slave_detected(I2C_t *p_obj);
 
/*--------------------------------------------------*/
/**
  * @brief     I2C software reset.
  * @param     p_obj       :I2C object.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_reset(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->CR2 = I2CxCR2_SWRES_10;
        p_obj->p_instance->CR2 = I2CxCR2_SWRES_01;
    }
#else
    p_obj->p_instance->CR2 = I2CxCR2_SWRES_10;
    p_obj->p_instance->CR2 = I2CxCR2_SWRES_01;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     I2C bus port high
  * @param     p_obj       :I2C object.
  * @retval    true        :SDA and SCL Port High.
  * @retval    false       :Bus Error.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_port_high(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return (((p_obj->p_instance->PM & I2CxPM_SDA_SCL) == I2CxPM_SDA_SCL));
    }
    return (0);
#else
    return (((p_obj->p_instance->PM & I2CxPM_SDA_SCL) == I2CxPM_SDA_SCL));
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Generate stop condition.
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_stop_condition(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->CR2 = I2CxCR2_STOP_CONDITION;
    }
#else
    p_obj->p_instance->CR2 = I2CxCR2_STOP_CONDITION;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Read from Data buffer
  * @param     p_obj       :I2C object.
  * @retval    result      :Read data.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t I2C_read_data(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return (p_obj->p_instance->DBR & I2CxDBR_DB_MASK);
    }
    return (0);
#else
    return (p_obj->p_instance->DBR & I2CxDBR_DB_MASK);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Write to Data buffer.
  * @param     p_obj       :I2C object.
  * @param     data        :Write data.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_write_data(I2C_t *p_obj, uint32_t data)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->DBR = (data & I2CxDBR_DB_MASK);
    }
#else
    p_obj->p_instance->DBR = (data & I2CxDBR_DB_MASK);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return restart condition
  * @param     p_obj    :I2C object.
  * @retval    true     :Restart Detected.
  * @retval    false    :Restart Non-Detected.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_restart(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        __IO uint32_t opreg = p_obj->p_instance->OP;
        p_obj->p_instance->OP &= ~I2CxOP_RSTA;
        return ((opreg & I2CxOP_RSTA) == I2CxOP_RSTA);
    }
    return (0);
#else
    __IO uint32_t opreg = p_obj->p_instance->OP;
    p_obj->p_instance->OP &= ~I2CxOP_RSTA;
    return ((opreg & I2CxOP_RSTA) == I2CxOP_RSTA);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Set Ack condition
  * @param     p_obj       :I2C object.
  * @param     nack        :1 NACK, 0 ACK.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_set_ack(I2C_t *p_obj, int32_t nack)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        if (nack)
        {
            p_obj->p_instance->OP |= I2CxOP_MFACK;
        }
        else
        {
            p_obj->p_instance->OP &= ~I2CxOP_MFACK;
        }
    }
#else
    if (nack)
    {
        p_obj->p_instance->OP |= I2CxOP_MFACK;
    }
    else
    {
        p_obj->p_instance->OP &= ~I2CxOP_MFACK;
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return received Ack condition
  * @param     p_obj    :I2C object.
  * @retval    true     :NACK Received.
  * @retval    false    :ACK Received.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_get_ack(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return ((p_obj->p_instance->SR & I2CxSR_LRB) == I2CxSR_LRB);
    }
    return (0);
#else
    return ((p_obj->p_instance->SR & I2CxSR_LRB) == I2CxSR_LRB);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return Busy condition
  * @param     p_obj       :I2C object.
  * @retval    true        :I2C bus busy.
  * @retval    false       :I2C bus free.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_status_busy(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return ((p_obj->p_instance->SR & I2CxSR_BB) == I2CxSR_BB);
    }
    return (0);
#else
    return ((p_obj->p_instance->SR & I2CxSR_BB) == I2CxSR_BB);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return The Master status
  * @param     p_obj       :I2C object.
  * @retval    true        :Master mode.
  * @retval    false       :Slave mode.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_master(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return ((p_obj->p_instance->SR & I2CxSR_MST) == I2CxSR_MST);
    }
    return (0);
#else
    return ((p_obj->p_instance->SR & I2CxSR_MST) == I2CxSR_MST);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return The Transmitter
  * @param     p_obj       :I2C object.
  * @retval    true        :Transmitter.
  * @retval    false       :Receiver.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_transmitter(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return ((p_obj->p_instance->SR & I2CxSR_TRX) == I2CxSR_TRX);
    }
    return (0);
#else
    return ((p_obj->p_instance->SR & I2CxSR_TRX) == I2CxSR_TRX);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status
  * @param     p_obj       :I2C object.
  * @retval    true        :Interruput Occured.
  * @retval    false       :No Interruput Occured.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_int_status(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return ((p_obj->p_instance->ST & I2CxST_I2C) == I2CxST_I2C);
    }
    return (0);
#else
    return ((p_obj->p_instance->ST & I2CxST_I2C) == I2CxST_I2C);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_clear_int_status(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->ST = I2CxST_CLEAR;
    }
#else
    p_obj->p_instance->ST = I2CxST_CLEAR;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Enable Interrupt setting.
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_enable_interrupt(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->IE = I2CxIE_INTI2C_ENABLE;
    }
#else
    p_obj->p_instance->IE = I2CxIE_INTI2C_ENABLE;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Enable Interrupt setting.
  * @param     p_obj       :I2C object.
  * @param     tx          :Direction of transfer(1=tx 0=rx).
  * @retval    -
  * @note      For DMA transfer.
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_enable_interrupt_dma(I2C_t *p_obj, int32_t tx)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        if (tx)
        {
            p_obj->p_instance->IE = (I2CxIE_SELPINCD_ENABLE | I2CxIE_DMARI2CTX_ENABLE);
        }
        else
        {
            p_obj->p_instance->IE = (I2CxIE_SELPINCD_ENABLE | I2CxIE_DMARI2CRX_ENABLE);
        }
    }
#else
    if (tx)
    {
        p_obj->p_instance->IE = (I2CxIE_SELPINCD_ENABLE | I2CxIE_DMARI2CTX_ENABLE);
    }
    else
    {
        p_obj->p_instance->IE = (I2CxIE_SELPINCD_ENABLE | I2CxIE_DMARI2CRX_ENABLE);
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Disable Interrupt setting.
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_disable_interrupt(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->IE = I2CxIE_CLEAR;
    }
#else
    p_obj->p_instance->IE = I2CxIE_CLEAR;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Set slave address.
  * @param     p_obj       :I2C object.
  * @param     addr        :slave address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void I2C_set_address(I2C_t *p_obj, uint32_t addr)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        p_obj->p_instance->AR  = (addr & ~I2CxAR_ALS);
        p_obj->p_instance->AR2 = I2CxAR2_INIT;
    }
#else
    p_obj->p_instance->AR  = (addr & ~I2CxAR_ALS);
    p_obj->p_instance->AR2 = I2CxAR2_INIT;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Detecting Slave Address
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t I2C_slave_detected(I2C_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != I2C_NULL) && (p_obj->p_instance != I2C_NULL))
    {
        return (((p_obj->p_instance->SR & I2CxSR_AAS) == I2CxSR_AAS) 
            && ((p_obj->p_instance->OP & I2CxOP_SAST) == I2CxOP_SAST));
    }
    return (0);
#else
    return (((p_obj->p_instance->SR & I2CxSR_AAS) == I2CxSR_AAS) 
        && ((p_obj->p_instance->OP & I2CxOP_SAST) == I2CxOP_SAST));
#endif
}

/**
 *  @}
 */ /* End of group I2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_functions
 *  @{
 */
void I2C_init(I2C_t *p_obj);
void I2C_start_condition(I2C_t *p_obj, uint32_t data);
uint32_t I2C_get_clock_setting(uint32_t frequency, uint32_t fsys, I2C_clock_setting_t *p_setting);
void I2C_slave_init(I2C_t *p_obj);
void I2C_interrupt_setting(I2C_t *p_obj);
#if (I2CS_EQUIPPED == AVAILABLE)
void I2CS_init(I2CS_t *p_obj);
void I2CS_get_status(I2CS_t *p_obj, uint8_t *p_status);
void I2CS_Primary_slave_adr_set(I2CS_t *p_obj, uint8_t adr);
void I2CS_Secondary_slave_adr_set(I2CS_t *p_obj, uint8_t adr);
#endif
/** 
 *  @}
 */ /* End of group I2C_Private_functions */

/**
 *  @}
 */ /* End of group I2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __I2C_H */


