/** 
 *******************************************************************************
 * @file    trgsel.h
 * @brief   This file is wrapper file for TRGSEL driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifndef __TRGSEL_H
#define __TRGSEL_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRGSEL_Private_define
 *  @{
 */

typedef enum
{
    TRGSEL_DISABLE = 0,         /* TRGSEL Disable */
    TRGSEL_ENABLE               /* TRGSEL Enable  */
}trgsel_en_t;

typedef enum
{
    TRGSEL_OUTSEL_UNDETECT = 0, /* TRGSEL Output trigger edge undetection */
    TRGSEL_OUTSEL_DETECT        /* TRGSEL Output trigger edge detection */
}trgsel_outsel_t;

typedef enum
{
    TRGSEL_UPDN_RISING = 0,     /* TRGSEL rising edge detection */
    TRGSEL_UPDN_FALLING         /* TRGSEL falling edge detection */
}trgsel_updn_t;

typedef enum
{
    TRGSEL_INSEL_000 = 0,     /* TRGSEL Input trigger select */
    TRGSEL_INSEL_001,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_010,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_011,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_100,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_101,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_110,         /* TRGSEL Input trigger select */
    TRGSEL_INSEL_111,         /* TRGSEL Input trigger select */
}trgsel_insel_t;

/** 
 *  @}
 */ /* End of group TRGSEL_Private_define */

 
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRGSEL_Private_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  TRGSEL handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    trgsel_en_t     en;
    trgsel_outsel_t outsel;
    trgsel_updn_t   updn;
    trgsel_insel_t  insel;
}trgsel_obj_t;
/*----------------------------------*/
/** 
 * @brief  TRGSEL handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_TSEL_TypeDef *p_instance;     /*!< Registers base address.           */
    trgsel_obj_t     init;            /*!< Initial setting.                  */
} trgsel_t;

/**
 *  @}
 */ /* End of group TRGSEL_Private_typedef */

#include "trgsel_config.h"
/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRGSEL_Private_functions
 *  @{
 */

TXZ_Result set_trgsel(trgsel_t *p_obj, uint32_t num);
trgsel_obj_t get_trgsel(trgsel_t *p_obj, uint32_t num);

/** 
 *  @}
 */ /* End of group TRGSEL_Private_functions */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TRGSEL_H */
