/**
 *******************************************************************************
 * @file    adc_i.c
 * @brief   This file provides API functions for ADC driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2023
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "adc_com.h"
#include "adc_ch.h"
#include "adc.h"

#if defined(__ADC_I_H)
/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup ADC
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_define ADC Private Define
 *  @{
 */


/** 
 *  @}
 */ /* End of group ADC_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_macro ADC Private Macro
 *  @{
 */

/**
 *  @}
 */ /* End of group ADC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_Enumeration ADC Private Enumeration
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_Enumeration */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_typedef ADC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Variable Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_variable ADC Private Variable Definition
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group ADC_Private_variable */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Private_fuctions ADC Private Fuctions
 *  @{
 */
#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_sampling_period0(adc_sampling_period0_t param);
__STATIC_INLINE int32_t check_param_sampling_period1(adc_sampling_period1_t param);
__STATIC_INLINE int32_t check_param_sampling_select(uint32_t param);
__STATIC_INLINE int32_t check_param_prescaler_output(adc_sclk_t param);
__STATIC_INLINE int32_t check_param_mod1(uint32_t param);
__STATIC_INLINE int32_t check_param_mod2(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt(uint32_t param);
__STATIC_INLINE int32_t check_param_type(uint32_t param);
__STATIC_INLINE int32_t check_param_ain(uint32_t ain);
__STATIC_INLINE int32_t check_param_adc_reg_num(uint32_t reg);
#endif /* #ifdef __DEBUG__ */
static void clear_ch_instance_info(adc_ch_t *p_ch);

#ifdef __DEBUG__
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_sampling_period0(adc_sampling_period0_t param)
  * @brief  Check the Sampling Period0's parameter
  * @param[in] param :Sampling Period0's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   Macro definition is ADC_SamplingPeriod"ADC_SAMPLING_PERIOD_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sampling_period0(adc_sampling_period0_t param)
{
    int32_t result = ADC_PARAM_NG;
    uint32_t i;

    for (i = 0; i < VALID_SAMPLING_PERIOD0_TABLE_SIZE; i++) {
        if (param == valid_sampling_period0_setting_table[i]) {
            result = ADC_PARAM_OK;
            break;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_sampling_period1(adc_sampling_period1_t param)
  * @brief  Check the Sampling Period1's parameter
  * @param[in] param :Sampling Period1's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   Macro definition is ADC_SamplingPeriod"ADC_SAMPLING_PERIOD_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sampling_period1(adc_sampling_period1_t param)
{
    int32_t result = ADC_PARAM_NG;
    uint32_t i;

    for (i = 0; i < VALID_SAMPLING_PERIOD1_TABLE_SIZE; i++) {
        if (param == valid_sampling_period1_setting_table[i]) {
            result = ADC_PARAM_OK;
            break;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_sampling_select(uint32_t param)
  * @brief  Check the Sampling Select's parameter
  * @param[in] param :Sampling Select's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sampling_select(uint32_t param)
{
     int32_t result = ADC_PARAM_NG;

     if (param < ADC_SAMPLE_SELECT_MAX)
     {
        result = ADC_PARAM_OK;
     }

     return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_prescaler_output(adc_sclk_t param)
  * @brief  Check the AD Prescaler Output's parameter.
  * @param[in] param :AD Prescaler Output's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   Macro definition is ADC_SCLK"ADC_SCLK_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_prescaler_output(adc_sclk_t param)
{
    int32_t result = ADC_PARAM_NG;
    uint32_t i;

    for (i = 0; i < VALID_SCLK_TABLE_SIZE; i++) {
        if (param == valid_sclk_setting_table[i]) {
            result = ADC_PARAM_OK;
            break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_mod1(uint32_t param)
  * @brief  Check the AD MOD1 parameter.
  * @param[in] param :AD MOD1 parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_mod1(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;
    uint32_t i;

    for (i = 0; i < VALID_MOD1_TABLE_SIZE; i++) {
        if (param == valid_mod1_setting_table[i]) {
            result = ADC_PARAM_OK;
            break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_mod2(uint32_t param)
  * @brief  Check the AD MOD2 parameter.
  * @param[in] param :AD MOD2 parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_mod2(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;
    uint32_t i;

    for (i = 0; i < VALID_MOD2_TABLE_SIZE; i++) {
        if (param == valid_mod2_setting_table[i]) {
            result = ADC_PARAM_OK;
            break;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt(uint32_t param)
  * @brief  Check the Interrupt's parameter.
  * @param[in] param :Interrupt's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   Macro definition is ADC_IntEnable"ADC_INT_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;

    switch (param) 
    {
    case ADC_INT_DISABLE:
    case ADC_INT_ENABLE:
        result = ADC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_type(uint32_t param)
  * @brief  Check the Conversion Type's parameter.
  * @param[in] param :Conversion Type's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   Macro definition is ADC_Conversion"ADC_CONVERSION_xxxx".
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_type(uint32_t param)
{
    int32_t result = ADC_PARAM_NG;

    switch (param) 
    {
    case ADC_CONVERSION_DISABLE:
    case ADC_CONVERSION_CNT:
    case ADC_CONVERSION_SGL:
    case ADC_CONVERSION_TRG:
        result = ADC_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_ain(uint32_t ain)
  * @brief  Check the AIN Range's parameter.
  * @param[in] ain :AIN Range's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ain(uint32_t ain)
{
    int32_t result = ADC_PARAM_NG;

    if (ADC_CH_MIN == 0)
    {
        if (ain <= ADC_CH_MAX)
        {
            result = ADC_PARAM_OK;
        }
    }
    else
    {
        /* cast as int32_t to avoid the warning: ain is less than int32_t max */
        if (((int32_t)ADC_CH_MIN <= (int32_t)ain) && (ain <= ADC_CH_MAX))
        {
            result = ADC_PARAM_OK;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_adc_reg_num(uint32_t reg)
  * @brief  Check the reg num's parameter.
  * @param[in] reg :reg num's parameter
  * @retval    ADC_PARAM_OK :Success.
  * @retval    ADC_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_adc_reg_num(uint32_t reg)
{
    int32_t result = ADC_PARAM_NG;

    if (reg < ADC_REG_NUM_MAX)
    {
        result = ADC_PARAM_OK;
    }

    return (result);
}
#endif /* #ifdef __DEBUG__ */

/*--------------------------------------------------*/
/*!
  * @fn     static void clear_ch_instance_info(adc_ch_t *p_ch)
  * @brief  Channel Instance Information Clear.
  * @param[in] p_ch :Channel Instance Address.
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_ch_instance_info(adc_ch_t *p_ch)
{
    p_ch->p_tset     = ADC_NULL;
    p_ch->p_reg      = ADC_NULL;
    p_ch->init.type  = ADC_CONVERSION_DISABLE;
}
/**
 *  @}
 */ /* End of group ADC_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_init(adc_t *p_obj)
  * @brief     Initialize the ADC object.
  * @param[in] p_obj  :ADC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt. After initialization, 3us of stabilization time is needed.
  */
/*--------------------------------------------------*/
TXZ_Result adc_init(adc_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_sampling_period0(p_obj->init.clk.exaz0));
    assert_param(check_param_sampling_period1(p_obj->init.clk.exaz1));
    assert_param(check_param_prescaler_output(p_obj->init.clk.vadcld));
    assert_param(check_param_sampling_select(p_obj->init.clk.sampling_select));
    assert_param(check_param_mod1(p_obj->init.mod1));
    assert_param(check_param_mod2(p_obj->init.mod2));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Init Variable               */
    /*------------------------------*/
    for (i = 0; i < NUMBER_OF_ADC_CHANNEL; i++)
    {
        clear_ch_instance_info(&p_obj->info.ch[i]);
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- ADxMOD0 ---*/
    p_obj->p_instance->MOD0 = (ADxMOD0_RCUT_NORMAL | ADxMOD0_DACON_ON); 
    /* ADMOD0 DACON Wait time 3us */
    wait(3); 
    /*--- ADxCLK  ---*/
    p_obj->p_instance->CLK  = ((uint32_t)p_obj->init.clk.exaz1 | (uint32_t)p_obj->init.clk.exaz0 | (uint32_t)p_obj->init.clk.vadcld);
    /*--- ADxEXAZSEL  ---*/
    p_obj->p_instance->EXAZSEL = (uint32_t)p_obj->init.clk.sampling_select;

    /*--- ADxMOD1 ---*/
    p_obj->p_instance->MOD1 = p_obj->init.mod1;
    /*--- ADxMOD2 ---*/
    p_obj->p_instance->MOD2 = p_obj->init.mod2;

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_deinit(adc_t *p_obj)
  * @brief     Release the ADC object.
  * @param[in] p_obj :ADC object.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_deinit(adc_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;
    adc_ch_t *p_ch;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- ADxCR0   ---*/
    p_obj->p_instance->CR0   = (ADxCR0_ADEN_DISABLE | ADxCR0_CNT_DISABLE);
    /*------------------------------*/
    /*  Wait Stop                   */
    /*------------------------------*/
    /*--- ADxST   ---*/
    /* When all convetion stop, ADxST is set "0". */
    while(p_obj->p_instance->ST != 0)
    {
            /* no processing */
    }
    /*------------------------------*/
    /*  Channel Class Destruct      */
    /*------------------------------*/
    for (i = 0; i < NUMBER_OF_ADC_CHANNEL; i++)
    {
        p_ch = &p_obj->info.ch[i];
        if (p_ch->init.type == ADC_CONVERSION_DISABLE)
        {
            if (adc_ch_deinit(p_ch) == TXZ_SUCCESS)
            {
                clear_ch_instance_info(p_ch);
            }
        }
    }
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    /*--- ADxCMPEN ---*/
    p_obj->p_instance->CMPEN = (ADxCMPEN_CMP1EN_DISABLE | ADxCMPEN_CMP0EN_DISABLE);
    /*--- ADxCR1   ---*/
    p_obj->p_instance->CR1   = (ADxCR1_CNTDMEN_DISABLE | ADxCR1_SGLDMEN_DISABLE | ADxCR1_TRGDMEN_DISABLE | ADxCR1_TRGEN_DISABLE);
    /*--- ADxMOD0 ---*/
    p_obj->p_instance->MOD0  = (ADxMOD0_RCUT_IREF_CUT | ADxMOD0_DACON_OFF);
    /*--- ADxMOD1 ---*/
    p_obj->p_instance->MOD1  = valid_mod1_setting_table[0];
    /*--- ADxMOD2 ---*/
    p_obj->p_instance->MOD2  = valid_mod2_setting_table[0];

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_channel_setting(adc_t *p_obj, uint32_t reg, adc_channel_setting_t *p_setting)
  * @brief     ADC Channel Setting
  * @param[in] p_obj  :ADC object.
  * @param[in] reg         :Channel. Range is (value < ADC_REG_NUM_MAX).
  * @param[in] p_setting   :Channel Setting Source Address.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @pre       Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_channel_setting(adc_t *p_obj, uint32_t reg, adc_channel_setting_t *p_setting)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_adc_reg_num(reg));
    assert_param(IS_POINTER_NOT_NULL(p_setting));
    assert_param(check_param_interrupt(p_setting->interrupt));
    assert_param(check_param_type(p_setting->type));
    assert_param(check_param_ain(p_setting->ain));
#endif /* #ifdef __DEBUG__ */

    {
        adc_ch_t *p_ch = &p_obj->info.ch[p_setting->ain];

        /*------------------------------*/
        /*  Channel Class Construct     */
        /*------------------------------*/
        p_ch->p_tset   = (__IO uint32_t *)(&p_obj->p_instance->TSET0 + reg);
        p_ch->p_reg    = (__I uint32_t *)(&p_obj->p_instance->REG0 + reg);
        p_ch->init.interrupt   = p_setting->interrupt;
        p_ch->init.type        = p_setting->type;
        p_ch->init.ain         = p_setting->ain;
        result = adc_ch_init(p_ch);
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_channel_clear(adc_t *p_obj, uint32_t ch)
  * @brief     ADC Channel Clear
  * @param[in] p_obj  :ADC object.
  * @param[in] ch          :Channel. Range is (value < ADC_CH_MAX).
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @pre       Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_channel_clear(adc_t *p_obj, uint32_t ch)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_ain(ch));
#endif /* #ifdef __DEBUG__ */

    adc_ch_t *p_ch = &p_obj->info.ch[ch];

    /*------------------------------*/
    /*  Channel Class Destruct      */
    /*------------------------------*/
    result = adc_ch_deinit(p_ch);
    /* Init Variable */
    clear_ch_instance_info(p_ch);

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_cmp_init(adc_t *p_obj, adc_cmpx_t *p_cmpx_t)
  * @brief     Initialize the ADC Compare register
  * @param[in] p_obj  :ADC object.
  * @param[in] p_cmpx_t    :Clock information structure.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  * @attention After initialization, 3us of stabilization time is needed.
  */
/*--------------------------------------------------*/
TXZ_Result adc_cmp_init(adc_t *p_obj, adc_cmpx_t *p_cmpx_t)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_cmpx_t));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Init Variable               */
    /*------------------------------*/
    if (p_cmpx_t->CMPEN == ADCMP0EN_DISABLE)
        p_obj->p_instance->CMPEN &= ~ADCMP0EN_ENABLE;
    if (p_cmpx_t->CMPEN == ADCMP1EN_DISABLE)
        p_obj->p_instance->CMPEN &= ~ADCMP1EN_ENABLE;
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    if (p_cmpx_t->CMPEN == ADCMP0EN_ENABLE)
    {
        p_obj->init.CMPxEN[0].CMPEN = p_cmpx_t->CMPEN;
        p_obj->init.CMPxEN[0].CMPCNT = p_cmpx_t->CMPCNT;
        p_obj->init.CMPxEN[0].CMPCond = p_cmpx_t->CMPCond;
        p_obj->init.CMPxEN[0].CMPBigSml = p_cmpx_t->CMPBigSml;
        p_obj->init.CMPxEN[0].StrReg = p_cmpx_t->StrReg;
        p_obj->init.CMPxEN[0].ADComp = p_cmpx_t->ADComp;
        p_obj->p_instance->CMPEN |= p_cmpx_t->CMPEN;
        p_obj->p_instance->CMPCR0 = p_cmpx_t->CMPCNT | p_cmpx_t->CMPCond | p_cmpx_t->CMPBigSml | p_cmpx_t->StrReg;
        p_obj->p_instance->CMP0 = (p_cmpx_t->ADComp & ADC_CMP_MASK);
    }
    else if (p_cmpx_t->CMPEN == ADCMP1EN_ENABLE)
    {
        p_obj->init.CMPxEN[1].CMPEN = p_cmpx_t->CMPEN;
        p_obj->init.CMPxEN[1].CMPCNT = p_cmpx_t->CMPCNT;
        p_obj->init.CMPxEN[1].CMPCond = p_cmpx_t->CMPCond;
        p_obj->init.CMPxEN[1].CMPBigSml = p_cmpx_t->CMPBigSml;
        p_obj->init.CMPxEN[1].StrReg = p_cmpx_t->StrReg;
        p_obj->init.CMPxEN[1].ADComp = p_cmpx_t->ADComp;
        p_obj->p_instance->CMPEN |= p_cmpx_t->CMPEN;
        p_obj->p_instance->CMPCR1 = p_cmpx_t->CMPCNT | p_cmpx_t->CMPCond | p_cmpx_t->CMPBigSml | p_cmpx_t->StrReg;
        p_obj->p_instance->CMP1 = (p_cmpx_t->ADComp & ADC_CMP_MASK);
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_cmp_deinit(adc_t *p_obj, adc_cmpx_t *p_cmpx_t)
  * @brief     Release the ADC Compare register
  * @param[in] p_obj  :ADC object.
  * @param[in] p_cmpx_t    :Clock information structure.
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  * @attention After initialization, 3us of stabilization time is needed.
  */
/*--------------------------------------------------*/
TXZ_Result adc_cmp_deinit(adc_t *p_obj, adc_cmpx_t *p_cmpx_t)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_cmpx_t));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    if (p_cmpx_t->CMPEN == ADCMP0EN_ENABLE)
    {
        p_obj->init.CMPxEN[0].CMPEN = 0;
        p_obj->init.CMPxEN[0].CMPCNT = 0;
        p_obj->init.CMPxEN[0].CMPCond = 0;
        p_obj->init.CMPxEN[0].CMPBigSml = 0;
        p_obj->init.CMPxEN[0].StrReg = 0;
        p_obj->init.CMPxEN[0].ADComp = 0;
        p_obj->p_instance->CMPEN &= ~ADCMP0EN_ENABLE;
        p_obj->p_instance->CMPCR0 = 0;
        p_obj->p_instance->CMP0 = 0;
    }
    else if (p_cmpx_t->CMPEN == ADCMP1EN_ENABLE)
    {
        p_obj->init.CMPxEN[1].CMPEN = 0;
        p_obj->init.CMPxEN[1].CMPCNT = 0;
        p_obj->init.CMPxEN[1].CMPCond = 0;
        p_obj->init.CMPxEN[1].CMPBigSml = 0;
        p_obj->init.CMPxEN[1].StrReg = 0;
        p_obj->init.CMPxEN[1].ADComp = 0;
        p_obj->p_instance->CMPEN &= ~ADCMP1EN_ENABLE;
        p_obj->p_instance->CMPCR1 = 0;
        p_obj->p_instance->CMP1 = 0;
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_channel_get_value(adc_t *p_obj, uint32_t ch, uint32_t *p_value)
  * @brief  Get AD value
  * @param[in] p_obj  :ADC object.
  * @param[in] ch         :Channel. Range is (value < ADC_CH_MAX).
  * @param  p_value     :AD value. Destination address.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result adc_channel_get_value(adc_t *p_obj, uint32_t ch, uint32_t *p_value)
{
    TXZ_Result result = TXZ_ERROR;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_value));
    assert_param(check_param_ain(ch));
#endif /* #ifdef __DEBUG__ */

    adc_ch_t *p_ch = &p_obj->info.ch[ch];

    /*------------------------------*/
    /*  Get Value                   */
    /*------------------------------*/
    result = adc_ch_get_value(p_ch, p_value);

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_start(adc_t *p_obj)
  * @brief  Start Conversion.
  * @param[in] p_obj    :ADC object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  * @pre    Conversion has stoped.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_start(adc_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;
    adc_ch_t *p_ch;
    uint32_t cr0 = (ADxCR0_ADEN_ENABLE | ADxCR0_CNT_DISABLE);
    uint32_t cr1 = (ADxCR1_CNTDMEN_DISABLE | ADxCR1_SGLDMEN_DISABLE | ADxCR1_TRGDMEN_DISABLE | ADxCR1_TRGEN_DISABLE);

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Enable Conversion           */
    /*------------------------------*/
    for (i = 0; i < NUMBER_OF_ADC_CHANNEL; i++)
    {
        p_ch = &p_obj->info.ch[i];
        switch (p_ch->init.type)
        {
        case ADC_CONVERSION_CNT:
            cr0 |= ADxCR0_CNT_ENABLE;
            break;
        case ADC_CONVERSION_SGL:
            cr0 |= ADxCR0_SGL_ENABLE;
            break;
        case ADC_CONVERSION_TRG:
            cr1 |= ADxCR1_TRGEN_ENABLE;
            break;
        default:
            /* no processing */
            break;
        }
    }
    /*--- ADxCR1  ---*/
    p_obj->p_instance->CR1 = cr1;
    /*--- ADxCR0  ---*/
    p_obj->p_instance->CR0 = cr0;

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result adc_stop(adc_t *p_obj)
  * @brief  Stop Conversion.
  * @param[in] p_obj    :ADC object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result adc_stop(adc_t *p_obj)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;
    adc_ch_t *p_ch;
    uint32_t value;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Disable Conversion          */
    /*------------------------------*/
    /*--- ADxCR0   ---*/
    p_obj->p_instance->CR0 = (ADxCR0_ADEN_DISABLE | ADxCR0_CNT_DISABLE);
    /*------------------------------*/
    /*  Wait Stop                   */
    /*------------------------------*/
    /*--- ADxST   ---*/
    /* When all convetion stop, ADxST is set "0". */
    while(p_obj->p_instance->ST != 0)
    {
        /* no processing */
    }
    /*------------------------------*/
    /*  Dummy Read                  */
    /*------------------------------*/
    /* Read is needed before the next convertion. */
    for (i = 0; i < NUMBER_OF_ADC_CHANNEL; i++)
    {
        p_ch = &p_obj->info.ch[i];
        if (p_ch->init.type == ADC_CONVERSION_DISABLE)
        {
            if (adc_ch_get_value(p_ch, &value) != TXZ_SUCCESS)
            {
                /* no processing */
            }
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_WorkState adc_poll_conversion(adc_t *p_obj, uint32_t timeout)
  * @brief  Wait for single conversion to be completed
  * @param[in] p_obj    :ADC object.
  * @param[in] timeout  :Timeout(tick).
  * @retval TXZ_DONE    :Success.
  * @retval TXZ_BUSY    :Failure.
  * @note   When the 1ms interrupt execute interruption, every time is counted.
            ( The 1ms interrupt is using T32A timer. )
            The count value is being used for a judgment of timeout.
  * @attention This function is not available in interrupt.
  * @attention This function is not available for Continuation Conversion and Universal Trigger.
  */
/*--------------------------------------------------*/
TXZ_WorkState adc_poll_conversion(adc_t *p_obj, uint32_t timeout)
{
    TXZ_WorkState result    = TXZ_BUSY;
    TXZ_WorkState loopBreak = TXZ_BUSY;
    uint32_t base    = hal_get_tick();
    uint32_t current = 0;
    uint32_t status;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Check Status                */
    /*------------------------------*/
    while(loopBreak == TXZ_BUSY)
    {
        /*--- Check Status ---*/
        /* Read status.  */
        status = p_obj->p_instance->ST;
        /* Check status. */
        if ((status & ADxST_SNGF_MASK) == ADxST_SNGF_IDLE)
        {
            result    = TXZ_DONE;
            loopBreak = TXZ_DONE;
        }
        else
        {
            /*--- Check Timeout ---*/
            if (timeout == 0)
            {
                loopBreak = TXZ_DONE;
            }
            else
            {
                current = hal_get_tick();
                if (current > base)
                {
                    if ((current - base) >= timeout)
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
                else
                {
                    base = current;
                }
            }
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @fn     TXZ_Result adc_get_status(adc_t *p_obj, uint32_t *p_status)
  * @brief  Get Conversion Status.
  * @details  Status bits.
  * | Bit   | Bit Symbol | Function                                                           |
  * | :---  | :---       | :---                                                               |
  * | 31-8  | -          | -                                                                  |
  * | 7     | ADBF       | AD Running Flag. Use @ref adc_status_t.                            |
  * | 6-4   | -          | -                                                                  |
  * | 3     | CNTF       | Continuity Conversion Running Flag. Use @ref adc_cnt_status_t.     |
  * | 2     | SNGF       | Single Conversion Running Flag. Use @ref adc_sgl_status_t.         |
  * | 1     | TRGF       | Trigger Conversion Running Flag. Use @ref adc_trg_status_t.        |
  * | 0     | -          | -                                                                  |
  * 
  * @param[in] p_obj      :ADC object.
  * @param[out] p_status  :Conversion Status. Destination address.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
TXZ_Result adc_get_status(adc_t *p_obj, uint32_t *p_status)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the ADC_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_status));
#endif /* #ifdef __DEBUG__ */

    /*------------------------------*/
    /*  Read Register               */
    /*------------------------------*/
    *p_status = p_obj->p_instance->ST;

    return (result);
}
/**
 *  @}
 */ /* End of group ADC_Exported_functions */

/**
 *  @} 
 */ /* End of group ADC */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__ADC_I_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

