/**
 *******************************************************************************
 * @file    dnf.c
 * @brief   This file provides API functions for DNF.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "dnf.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup DNF DNF
 *  @brief    DNF Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_define
 *  @{
 */

/** 
 *  @defgroup Null Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
#define REG_DNF_NULL                  ((void *)0)
/**
 *  @}
 */ /* End of group Null */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define REG_DNF_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define REG_DNF_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */


/** 
 *  @}
 */ /* End of group DNF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group DNF_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group DNF_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Private_fuctions
 *  @{
 */
#ifdef __DEBUG__
    __STATIC_INLINE int32_t check_param_DNFxENCR_mask(TSB_DNF_TypeDef *p, uint32_t param);
#endif /* #ifdef __DEBUG__ */
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  DNFxENCR_mask Paramter Check.
  * @param  param  :DNFxENCR mask
  * @return Result.
  * @retval REG_ENC_PARAM_OK :Valid
  * @retval REG_ENC_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_DNFxENCR_mask(TSB_DNF_TypeDef *p, uint32_t param)
{
    int32_t result = REG_DNF_PARAM_NG;
    int32_t i;
    
    int32_t n = sizeof(valid_encr_nfen_setting_table) / sizeof(encr_nfen_t);
    for (i = 0; i < n; i++)
    {
        encr_nfen_t setting = valid_encr_nfen_setting_table[i];
        if ((setting.addr == p) && (setting.bit == param)) {
            result = REG_DNF_PARAM_OK;
            break;
        }
    }

    return (result);
}
#endif /* #ifdef __DEBUG__ */

static uint32_t convert_clock_to_register_value(dnf_nfcks_t clock)
{
    uint32_t value = REG_DNF_NFCKCR_NFCKS_STOP;
    
    switch (clock) 
    {
    case NFCKS_CLOCK_STOP:
        value = REG_DNF_NFCKCR_NFCKS_STOP;
        break;
    case NFCKS_CLOCK_FC_2:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_2;
        break;
    case NFCKS_CLOCK_FC_4:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_4;
        break;
    case NFCKS_CLOCK_FC_8:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_8;
        break;
    case NFCKS_CLOCK_FC_16:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_16;
        break;
    case NFCKS_CLOCK_FC_32:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_32;
        break;
    case NFCKS_CLOCK_FC_64:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_64;
        break;
    case NFCKS_CLOCK_FC_128:
        value = REG_DNF_NFCKCR_NFCKS_CLOCK_128;
        break;
    default:
        /* no process */
        break;
    }
    
    return value;
}
/**
 *  @}
 */ /* End of group DNF_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup DNF_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief  Set Value Noise Filter Clock Select.
  * @param  p_obj   : DNF object.
  * @param  clock   : NFCKCR_NFCKS(dnf_nfcks_t)
  * @return Success or Failure
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note   The used register.
  * | Register  | Bit Symbol |
  * | :---      | :---       |
  * | NFCKCR    | NFCKS      |
  */
/*--------------------------------------------------*/
TXZ_Result select_noise_filter_clock(dnf_t *p_obj, dnf_nfcks_t clock)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t value = 0x00000000UL;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_DNF_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */
    /*--- [NFCKCR]<NFCKS> ---*/
    /*------------------------------*/
    /*  Register Set                */
    /*------------------------------*/
    value = convert_clock_to_register_value(clock);
    p_obj->p_instance->CKCR = value;
    
    return result;
}
/*--------------------------------------------------*/
/**
  * @brief  Get value Noise Filter Clock Select.
  * @param  p_obj : DNF object
  * @param[out] p_value :Store of value
  * @return Success or Failure
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note   The used register.
  * | Register  | Bit Symbol |
  * | :---      | :---       |
  * | NFCKCR    | NFCKS      |
  */
/*--------------------------------------------------*/
TXZ_Result read_DNFxCKCR_register(dnf_t *p_obj, uint32_t *p_value)
{
    TXZ_Result result = TXZ_SUCCESS;
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_DNF_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_value));
#endif /* #ifdef __DEBUG__ */
    /*--- [NFCKCR]<NFCKS> ---*/
    /*------------------------------*/
    /*  Register Set                */
    /*------------------------------*/
    *p_value = p_obj->p_instance->CKCR;
    
    return result;
}
/*--------------------------------------------------*/
/**
  * @brief  Set Interrupt Noise Filter Control.
  * @param  p_obj : DNF object
  * @param  bit     :Interrupt.
  *                 :This parameter can be one of the following values: 
  *                 :REG_DNF_NFENCR_NFEN0_MASK
  *                 :REG_DNF_NFENCR_NFEN1_MASK
  *                 :REG_DNF_NFENCR_NFEN2_MASK
  *                 :REG_DNF_NFENCR_NFEN3_MASK
  *                 :REG_DNF_NFENCR_NFEN4_MASK
  *                 :REG_DNF_NFENCR_NFEN5_MASK
  *                 :REG_DNF_NFENCR_NFEN6_MASK
  *                 :REG_DNF_NFENCR_NFEN7_MASK
  *                 :REG_DNF_NFENCR_NFEN8_MASK
  *                 :REG_DNF_NFENCR_NFEN9_MASK
  *                 :REG_DNF_NFENCR_NFEN10_MASK
  *                 :REG_DNF_NFENCR_NFEN11_MASK
  *                 :REG_DNF_NFENCR_NFEN12_MASK
  *                 :REG_DNF_NFENCR_NFEN13_MASK
  *                 :REG_DNF_NFENCR_NFEN14_MASK
  *                 :REG_DNF_NFENCR_NFEN15_MASK
  * @return Success or Failure
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note   The used register.
  * | Register   | Bit Symbol    |
  * | :---       | :---          |
  * | NFENCR     | NFEN0         |
  * |            | NFEN1         |
  * |            | NFEN2         |
  * |            | NFEN3         |
  * |            | NFEN4         |
  * |            | NFEN5         |
  * |            | NFEN6         |
  * |            | NFEN7         |
  * |            | NFEN8         |
  * |            | NFEN9         |
  * |            | NFEN10        |
  * |            | NFEN11        |
  * |            | NFEN12        |
  * |            | NFEN13        |
  * |            | NFEN14        |
  * |            | NFEN15        |
  */
/*--------------------------------------------------*/
TXZ_Result enable_noise_filter(dnf_t *p_obj, uint32_t bit)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t inten = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_DNF_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_DNFxENCR_mask(p_obj->p_instance, bit));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    inten  = (uint32_t)(p_obj->p_instance->ENCR & ~bit);
    inten |= bit;
    /*------------------------------*/
    /*  Register Set                */
    /*------------------------------*/
    /*--- [DNFxENCR] ---*/
    p_obj->p_instance->ENCR = inten;
    
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief  Set Interrupt Noise Filter Control.
  * @param  p_obj : DNF object
  * @param  bit     :Interrupt.
  *                 :This parameter can be one of the following values: 
  *                 :REG_DNF_NFENCR_NFEN0_MASK
  *                 :REG_DNF_NFENCR_NFEN1_MASK
  *                 :REG_DNF_NFENCR_NFEN2_MASK
  *                 :REG_DNF_NFENCR_NFEN3_MASK
  *                 :REG_DNF_NFENCR_NFEN4_MASK
  *                 :REG_DNF_NFENCR_NFEN5_MASK
  *                 :REG_DNF_NFENCR_NFEN6_MASK
  *                 :REG_DNF_NFENCR_NFEN7_MASK
  *                 :REG_DNF_NFENCR_NFEN8_MASK
  *                 :REG_DNF_NFENCR_NFEN9_MASK
  *                 :REG_DNF_NFENCR_NFEN10_MASK
  *                 :REG_DNF_NFENCR_NFEN11_MASK
  *                 :REG_DNF_NFENCR_NFEN12_MASK
  *                 :REG_DNF_NFENCR_NFEN13_MASK
  *                 :REG_DNF_NFENCR_NFEN14_MASK
  *                 :REG_DNF_NFENCR_NFEN15_MASK
  * @return Success or Failure
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @note   The used register.
  * | Register   | Bit Symbol    |
  * | :---       | :---          |
  * | NFENCR     | NFEN0         |
  * |            | NFEN1         |
  * |            | NFEN2         |
  * |            | NFEN3         |
  * |            | NFEN4         |
  * |            | NFEN5         |
  * |            | NFEN6         |
  * |            | NFEN7         |
  * |            | NFEN8         |
  * |            | NFEN9         |
  * |            | NFEN10        |
  * |            | NFEN11        |
  * |            | NFEN12        |
  * |            | NFEN13        |
  * |            | NFEN14        |
  * |            | NFEN15        |
  */
/*--------------------------------------------------*/
TXZ_Result disable_noise_filter(dnf_t *p_obj, uint32_t bit)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t inten = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_DNF_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_DNFxENCR_mask(p_obj->p_instance, bit));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Register Setting            */
    /*------------------------------*/
    inten  = (uint32_t)(p_obj->p_instance->ENCR & ~bit);
    inten |= REG_DNF_NFENCR_NFEN0_DISABLE;
    /*------------------------------*/
    /*  Register Set                */
    /*------------------------------*/
    /*--- [DNFxENCR] ---*/
    p_obj->p_instance->ENCR = inten;
    
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief  Get the Interrupt Noise Filter Control.
  * @param  p_obj : DNF object
  * @param[out] p_value :Store of value
  * @return Success or Failure
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR :Failure.
  * @retval value   : NFENCR.
  * @note   The used register.
  * | Register   | Bit Symbol    |
  * | :---       | :---          |
  * | NFENCR     | NFEN0         |
  * |            | NFEN1         |
  * |            | NFEN2         |
  * |            | NFEN3         |
  * |            | NFEN4         |
  * |            | NFEN5         |
  * |            | NFEN6         |
  * |            | NFEN7         |
  * |            | NFEN8         |
  * |            | NFEN9         |
  * |            | NFEN10        |
  * |            | NFEN11        |
  * |            | NFEN12        |
  * |            | NFEN13        |
  * |            | NFEN14        |
  * |            | NFEN15        |
  */
/*--------------------------------------------------*/
TXZ_Result read_DNFxENCR_register(dnf_t *p_obj, uint32_t *p_value)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the REG_DNF_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_value));
#endif /* #ifdef __DEBUG__ */
    /*------------------------------*/
    /*  Register Read               */
    /*------------------------------*/
    /*--- [ENxINTF] ---*/
    *p_value = p_obj->p_instance->ENCR;

    return result;
}

/**
 *  @}
 */ /* End of group DNF_Exported_functions */

/**
 *  @}
 */ /* End of group DNF */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
