/**
 *******************************************************************************
 * @file    ei2c.c
 * @brief   This file provides API functions for EI2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "ei2c.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup EI2C
 *  @brief      EI2C Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define 
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define 
 *  @{
 */


#ifdef __DEBUG__
/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define REG_EI2C_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define REG_EI2C_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */
#endif /* #ifdef __DEBUG__ */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group EI2C_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Const Table                                                                 */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_const
 *  @{
 */
/**
 *  @}
 */ /* End of group EI2C_Private_const */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_functions
 *  @{
 */
 
#ifdef __DEBUG__
    __STATIC_INLINE int32_t check_param_EI2CxACR0_nfsel_is_digital(uint32_t param);
    __STATIC_INLINE int32_t check_param_EI2CxACR0_nfsel_is_analog(uint32_t param);
    __STATIC_INLINE int32_t check_param_EI2CxACR0_mask(TSB_EI2C_TypeDef *p, uint32_t param);
#if (EI2CS_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_wup_sgcdi(uint8_t param);
    __STATIC_INLINE int32_t check_param_wup_ack(uint8_t param);
    __STATIC_INLINE int32_t check_param_wup_reset(uint8_t param);
    __STATIC_INLINE int32_t check_param_wup_intend(uint8_t param);
#endif /* (EI2CS_EQUIPPED == AVAILABLE) */
#endif /* #ifdef __DEBUG__ */
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  EI2CxACR0  nfsel Paramter Check.
  * @param  param  :EI2CxACR0 nfsel
  * @return Result.
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_EI2CxACR0_nfsel_is_digital(uint32_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    if (param == EI2CxCR0_NFSEL_DIG) {
        result = REG_EI2C_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  EI2CxACR0  nfsel Paramter Check.
  * @param  param  :EI2CxACR0 nfsel
  * @return Result.
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_EI2CxACR0_nfsel_is_analog(uint32_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    if (param == EI2CxCR0_NFSEL_ANA) {
        result = REG_EI2C_PARAM_OK;
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  EI2CxACR0_mask Paramter Check.
  * @param  param  :EI2CxACR0 mask
  * @return Result.
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_EI2CxACR0_mask(TSB_EI2C_TypeDef *p, uint32_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;
    int32_t i;
    
    int32_t n = sizeof(valid_acr0_nfsel_setting_table) / sizeof(acr0_nfsel_t);
    for (i = 0; i < n; i++)
    {
        acr0_nfsel_t setting = valid_acr0_nfsel_setting_table[i];
        if ((setting.addr == p) && (setting.bit == param)) {
            result = REG_EI2C_PARAM_OK;
            break;
        }
    }

    return (result);
}

#if (EI2CS_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_sgcdi(uint8_t param)
  * @brief  Check the SGCDI's parameter
  * @param[in] param :SGCDI's parameter
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_sgcdi(uint8_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_SGCDI_OFF:
    case I2CxWUPCR1_SGCDI_ON:
        result = REG_EI2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_ack(uint8_t param)
  * @brief  Check the ACK's parameter
  * @param[in] param :ACK's parameter
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_ack(uint8_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_ACK_ACK:
    case I2CxWUPCR1_ACK_NACK:
        result = REG_EI2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_reset(uint8_t param)
  * @brief  Check the I2RES's parameter
  * @param[in] param :I2RES's parameter
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_reset(uint8_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_I2RES_RESET:
    case I2CxWUPCR1_I2RES_RELEASE:
        result = REG_EI2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_intend(uint8_t param)
  * @brief  Check the INTEND's parameter
  * @param[in] param :INTEND's parameter
  * @retval REG_EI2C_PARAM_OK :Valid
  * @retval REG_EI2C_PARAM_NG :Invalid
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_intend(uint8_t param)
{
    int32_t result = REG_EI2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_INTEND_RELEASE:
    case I2CxWUPCR1_INTEND_HOLD:
        result = REG_EI2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
#endif /* (EI2CS_EQUIPPED == AVAILABLE) */

#endif /* #ifdef __DEBUG__ */

/**
 *  @}
 */ /* End of group EI2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief     Initializing EI2C Regester
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2C_init(ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */
    
    if(EI2C_port_high(p_obj))
    {
        p_obj->p_instance->AEN  = EI2CxAEN_I2CM_DISABLE;
    }

    p_obj->p_instance->ASR1 = EI2CxSR1_CLEAR;

    /* PRS&SCL setting*/
    p_obj->p_instance->APRS = (EI2CxPRS_PRCK & p_obj->init.clock.prs);
    p_obj->p_instance->ASCL = ((EI2CxSCL_SCL_MASK & p_obj->init.clock.sclh)
                            | ((EI2CxSCL_SCL_MASK & p_obj->init.clock.scll)<<8));

    
    /*    other Register init    */
    p_obj->p_instance->AIE  = EI2CxIE_DISABLE;
    p_obj->p_instance->AAR1 = EI2CxAR1_INT;
    p_obj->p_instance->AAR2 = EI2CxAR2_INT;    
    p_obj->p_instance->ACR0 = (EI2CxCR0_DNF_MASK & p_obj->init.cr0.dnf)
                            | (EI2CxCR0_NFSEL_MASK & p_obj->init.cr0.nfsel)
                            | (EI2CxCR0_TOPER_MASK & p_obj->init.cr0.toper)
                            | (EI2CxCR0_TOE_MASK & p_obj->init.cr0.toe)
                            | (EI2CxCR0_ESPE_MASK & p_obj->init.cr0.espe)
                            | (EI2CxCR0_ESTE_MASK & p_obj->init.cr0.este)
                            | (EI2CxCR0_NACKE_MASK & p_obj->init.cr0.nacke)
                            | (EI2CxCR0_GCE_MASK & p_obj->init.cr0.gce)
                            | (EI2CxCR0_ALE_MASK & p_obj->init.cr0.ale);

    /* wait for bus free   */
    while((EI2C_port_high(p_obj))==0)
    {
    }

    /* start I2C control */
    p_obj->p_instance->AEN  = EI2CxAEN_I2CM_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Generate restart condition
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2C_restartcondition(ei2c_t *p_obj)
{
    __IO uint32_t opreg;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* set only when mst=1 bb=1   */
    if((EI2C_status_busy(p_obj))
    && (EI2C_master(p_obj)))
    {
        /* restart  condition*/
        opreg = p_obj->p_instance->ACR1;
        opreg &= ~(EI2CxCR1_ST | EI2CxCR1_SP);
        opreg |= EI2CxCR1_RS;
        p_obj->p_instance->ACR1  = opreg;        
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Generate start condition
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2C_startcondition(ei2c_t *p_obj)
{
    __IO uint32_t opreg;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* wait for bus free   */
    while((p_obj->p_instance->ASR0&EI2CxSR0_BB)==EI2CxSR0_BB)
    {        
    }
    /* start  condition*/
    opreg = p_obj->p_instance->ACR1;
    opreg &= ~(EI2CxCR1_RS | EI2CxCR1_SP);
    opreg |= EI2CxCR1_ST;
    p_obj->p_instance->ACR1  = opreg;
}

/*--------------------------------------------------*/
/**
  * @brief     Slave mode setting.
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2C_slave_init(ei2c_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(EI2C_port_high(p_obj))
    {
        p_obj->p_instance->AEN  = EI2CxAEN_I2CM_DISABLE;
    }
    p_obj->p_instance->ASR1 = EI2CxSR1_CLEAR;

    p_obj->p_instance->APRS = (EI2CxPRS_PRCK & p_obj->init.clock.prs);
    p_obj->p_instance->ASCL = ((EI2CxSCL_SCL_MASK & p_obj->init.clock.sclh)
                                         | ((EI2CxSCL_SCL_MASK & p_obj->init.clock.scll)<<8));

    /*    other Register init    */
    p_obj->p_instance->AIE  = EI2CxIE_DISABLE;

    EI2C_set_address1(p_obj, p_obj->init.aar1.safs, p_obj->init.aar1.sa, p_obj->init.aar1.sae);
    EI2C_set_address2(p_obj, p_obj->init.aar2.safs, p_obj->init.aar2.sa, p_obj->init.aar2.sae);

    p_obj->p_instance->ACR0 = (EI2CxCR0_DNF_MASK & p_obj->init.cr0.dnf)
                            | (EI2CxCR0_NFSEL_MASK & p_obj->init.cr0.nfsel)
                            | (EI2CxCR0_TOPER_MASK & p_obj->init.cr0.toper)
                            | (EI2CxCR0_TOE_MASK & p_obj->init.cr0.toe)
                            | (EI2CxCR0_ESPE_MASK & p_obj->init.cr0.espe)
                            | (EI2CxCR0_ESTE_MASK & p_obj->init.cr0.este)
                            | (EI2CxCR0_NACKE_MASK & p_obj->init.cr0.nacke)
                            | (EI2CxCR0_GCE_MASK & p_obj->init.cr0.gce)
                            | (EI2CxCR0_ALE_MASK & p_obj->init.cr0.ale);

    /* wait for bus free   */
    while(EI2C_port_high(p_obj)==0)
    {
    }

    /* start I2C control */
    p_obj->p_instance->AEN  = EI2CxAEN_I2CM_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting(DNF)
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result EI2C_set_prs_scl_dig(ei2c_t *p_obj, uint32_t fsys, uint8_t dnf)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_EI2CxACR0_nfsel_is_digital(p_obj->init.cr0.nfsel));
    assert_param(check_param_EI2CxACR0_mask(p_obj->p_instance, p_obj->init.cr0.nfsel));
#endif /* #ifdef __DEBUG__ */

    uint32_t frequency,prs,prs_tmp;
    uint32_t prsck;
    uint32_t scl_tf,scl_tr;
    uint32_t sclh,sclh_tmp,scll;
    int32_t scll_tmp;
    uint32_t t_total,t_high,t_low;
    uint32_t low_width_min;
    uint32_t high_width_min;

    frequency = p_obj->init.clock.sck;

    /* unuse value init */
    prs  = EI2C_UNUSE_VALUE;
    sclh = EI2C_UNUSE_VALUE;
    scll = EI2C_UNUSE_VALUE;

    if(frequency <= EI2CSCL_FRQ_FMP)
    {
        /* STD */
        if(frequency <= EI2CSCL_FRQ_STD)
        {
            high_width_min = EI2CSCL_SCLH_STDMIN;
            low_width_min  = EI2CSCL_SCLL_STDMIN;
            scl_tf = EI2CSCL_TF_STD;
            scl_tr = EI2CSCL_TR_STD;
        }
        /* FM */
        else if(frequency <= EI2CSCL_FRQ_FM)
        {
            high_width_min = EI2CSCL_SCLH_FMMIN;
            low_width_min  = EI2CSCL_SCLL_FMMIN;
            scl_tf = EI2CSCL_TF_FM;
            scl_tr = EI2CSCL_TR_FM;
        }
        /* FMP */
        else
        {
            high_width_min = EI2CSCL_SCLH_FMPMIN;
            low_width_min  = EI2CSCL_SCLL_FMPMIN;
            scl_tf = EI2CSCL_TF_FMP;
            scl_tr = EI2CSCL_TR_FMP;
        }

        /*---------------------------------------------------------*/
        /* freq(hz)=(1/(tHIGH(ns)+tLOW(ns)+tr(ns)+tf(ns)))*(10^9)  */
        /* tHIGH(ns)+tLOW(ns)=(10^9/freq(hz))-tr(ns)-tf(ns)        */
        /*---------------------------------------------------------*/
        t_total = (EI2C_UNIT_1US*1000/frequency) - scl_tf - scl_tr;

        /* if small than min return */
        if(t_total < (high_width_min + low_width_min))
        {
            result = TXZ_ERROR;
            return result;
        }

        for (prs_tmp = 0; prs_tmp < EI2CxPRS_MAX; prs_tmp++)
        {
            /*-----------------------------------------------*/
            /* clock width (s) = 1 / PCLK                    */
            /* clock width (s) = 1 / (fsys / PRS)           */
            /* clock width(ns) = (10^9) / (fsys / PRS)      */
            /*-----------------------------------------------*/
            /* NFSEL=0: (50ns/(<DNF>+1))<=clock width(ns)    */
            /*            <=(tHIGH(Min)/(<DNF>+2)            */
            /* ((10^9)*(<DNF>+2))/(tHIGH(Min) <= (fsys/PRS) */
            /*            <=((10^9)*(<DNF>+1))/(50ns)        */
            /*-----------------------------------------------*/
            /* PRS: 1~64   DNF:1~6                           */
            /* (tHIGH(Min): Use @ref EI2CSCL_MAX             */
            /*-----------------------------------------------*/
            prsck = ((uint32_t)fsys /((prs_tmp+1)*1000));

            if((prsck < ((EI2C_UNIT_1US/high_width_min)*(dnf+2)))
            || (prsck > ((EI2C_UNIT_1US/50)*(dnf+1))))
            {
                continue;
            }

            if(t_total == (high_width_min + low_width_min))
            {
                t_high = high_width_min;
                t_low = low_width_min;
                sclh_tmp = ((t_high*prsck)/EI2C_UNIT_1US) - 3 - dnf;
                scll_tmp = ((t_low*prsck)/EI2C_UNIT_1US) - 3 - dnf;

                /* paramater check */
                if(( sclh_tmp <= EI2CxSCL_MAX )
                && ( scll_tmp <= EI2CxSCL_MAX ))
                {
                    prs  = prs_tmp;
                    sclh = sclh_tmp;
                    scll = scll_tmp;
                    break;
                }
            }
            else
            {
                /*-----------------------------------------------*/
                /* t_high(ns)=((SCLH+3+DNF)/prsck(Hz))*(10^9)    */
                /* t_low(ns) =((SCLL+3+DNF)/prsck(Hz))*(10^9)    */
                /* t_high(ns) + t_low(ns) = t_total              */
                /* t_high(ns)>= high_width_min                   */
                /* t_low(ns) >= low_width_min                    */
                /*-----------------------------------------------*/
                for(sclh_tmp = 0; sclh_tmp <= EI2CxSCL_MAX; sclh_tmp++)
                {
                    t_high = ((sclh_tmp + dnf + 3)*EI2C_UNIT_1US)/prsck;
                    t_low  = t_total - t_high;

                    /*  paramater check */
                    if((t_high < high_width_min)
                    || (t_low  < low_width_min))
                    {
                        continue;
                    }

                    scll_tmp = ((t_low*prsck)/EI2C_UNIT_1US) - 3 - dnf;

                    /* paramater check */
                    if(( scll_tmp >= 0 )
                    && ( scll_tmp <= EI2CxSCL_MAX ))
                    {
                        prs  = prs_tmp;
                        sclh = sclh_tmp;
                        scll = scll_tmp;
                        break;
                    }
                }
            }

            /*  when set over break the loop  */
            if((prs != EI2C_UNUSE_VALUE)
            &&(sclh != EI2C_UNUSE_VALUE)
            &&(scll != EI2C_UNUSE_VALUE))
            {
                break;
            }
        }

        /* set the paramater value */
        p_obj->init.clock.prs = prs;
        p_obj->init.clock.sclh = sclh;
        p_obj->init.clock.scll = scll;
    }
    else
    {
        result = TXZ_ERROR;
    }

return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Return the EI2c clock setting(ANF)
  * @param     p_obj       :EI2C object.
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
TXZ_Result EI2C_set_prs_scl_ana(ei2c_t *p_obj, uint32_t fsys)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_EI2CxACR0_nfsel_is_analog(p_obj->init.cr0.nfsel));
    assert_param(check_param_EI2CxACR0_mask(p_obj->p_instance, p_obj->init.cr0.nfsel));
#endif /* #ifdef __DEBUG__ */

    uint32_t frequency,prs,prs_tmp;
    uint32_t prsck;
    uint32_t scl_tf,scl_tr;
    uint32_t sclh,sclh_tmp,scll;
    int32_t scll_tmp;
    uint32_t t_total,t_high,t_low;
    uint32_t low_width_min;
    uint32_t high_width_min;

    frequency = p_obj->init.clock.sck;

    /* unuse value init */
    prs  = EI2C_UNUSE_VALUE;
    sclh = EI2C_UNUSE_VALUE;
    scll = EI2C_UNUSE_VALUE;

    if(frequency <= EI2CSCL_FRQ_FMP)
    {
        /* STD */
        if(frequency <= EI2CSCL_FRQ_STD)
        {
            high_width_min = EI2CSCL_SCLH_STDMIN;
            low_width_min  = EI2CSCL_SCLL_STDMIN;
            scl_tf = EI2CSCL_TF_STD;
            scl_tr = EI2CSCL_TR_STD;
        }
        /* FM */
        else if(frequency <= EI2CSCL_FRQ_FM)
        {
            high_width_min = EI2CSCL_SCLH_FMMIN;
            low_width_min  = EI2CSCL_SCLL_FMMIN;
            scl_tf = EI2CSCL_TF_FM;
            scl_tr = EI2CSCL_TR_FM;
        }
        /* FMP */
        else
        {
            high_width_min = EI2CSCL_SCLH_FMPMIN;
            low_width_min  = EI2CSCL_SCLL_FMPMIN;
            scl_tf = EI2CSCL_TF_FMP;
            scl_tr = EI2CSCL_TR_FMP;
        }

        /*---------------------------------------------------------*/
        /* freq(hz)=(1/(tHIGH(ns)+tLOW(ns)+tr(ns)+tf(ns)))*(10^9)  */
        /* tHIGH(ns)+tLOW(ns)=(10^9/freq(hz))-tr(ns)-tf(ns)        */
        /*---------------------------------------------------------*/
        t_total = (EI2C_UNIT_1US*1000/frequency) - scl_tf - scl_tr;
        
        /* if small than min return */
        if(t_total < (high_width_min + low_width_min))
        {
            result = TXZ_ERROR;
            return result;
        }
        
        for (prs_tmp = 0; prs_tmp < EI2CxPRS_MAX; prs_tmp++)
        {
            /*-----------------------------------------------*/
            /* clock width (s) = 1 / PRSCK                   */
            /* clock width (s) = 1 / (fsys /(PRS+1))        */
            /* clock width(ns) = (10^9) / (fsys/(PRS+1))    */
            /*-----------------------------------------------*/
            /* NFSEL=1: clock width(ns)<= (tHIGH(Min)/2)     */
            /* ((10^9)*2)/(tHIGH(Min)) <= (fsys/(PRS+1))    */
            /*-----------------------------------------------*/
            /* PRS: 0~63                                     */
            /* (tHIGH(Min): Use @ref EI2CSCL_MAX             */
            /*-----------------------------------------------*/
            prsck = ((uint32_t)fsys /((prs_tmp+1)*1000));

            if(prsck < ((EI2C_UNIT_1US*2)/high_width_min))
            {
                continue;
            }

            /*-----------------------------------------------*/
            /* t_high(ns)=((SCLH+ 2)/prsck(Hz))*(10^9)       */
            /* t_low(ns) =((SCLL+ 2)/prsck(Hz))*(10^9)       */
            /* t_high(ns) + t_low(ns) = t_total              */
            /* t_high(ns)>= high_width_min                   */
            /* t_low(ns) >= low_width_min                    */
            /*-----------------------------------------------*/
            for(sclh_tmp = 0; sclh_tmp <= EI2CxSCL_MAX; sclh_tmp++)
            {
                t_high = ((sclh_tmp + 2)*EI2C_UNIT_1US)/prsck;
                t_low  = t_total - t_high;

                /*  paramater chec */
                if((t_high < high_width_min)
                || (t_low  < low_width_min))
                {
                    continue;
                }
               
                scll_tmp = (t_low*prsck)/EI2C_UNIT_1US - 2;

                /*  paramater chec */
                if(( scll_tmp >= 0 )
                && ( scll_tmp <= EI2CxSCL_MAX ))
                {
                    prs  = prs_tmp;
                    sclh = sclh_tmp;
                    scll = scll_tmp;
                    break;
                }
            }

            /* when set over break the loop */
            if((prs != EI2C_UNUSE_VALUE)
            &&(sclh != EI2C_UNUSE_VALUE)
            &&(scll != EI2C_UNUSE_VALUE))
            {
                break;
            }
        }

        /* set the paramater value */
        p_obj->init.clock.prs = prs;
        p_obj->init.clock.sclh = sclh;
        p_obj->init.clock.scll = scll;
    }
    else
    {
        result = TXZ_ERROR;
    }

    return (result);
}


#if (EI2CS_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     EI2C Wakeup Control setting.
  * @param     p_obj       :EI2CS object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_init(EI2CS_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_wup_sgcdi(p_obj->init.wup.sgcdi));
    assert_param(check_param_wup_ack(p_obj->init.wup.ack));
    assert_param(check_param_wup_reset(p_obj->init.wup.i2res));
    assert_param(check_param_wup_intend(p_obj->init.wup.intend));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR1 = (p_obj->init.wup.sgcdi | p_obj->init.wup.ack | p_obj->init.wup.i2res | p_obj->init.wup.intend);
}

/*--------------------------------------------------*/
/**
  * @brief     Get I2C Wakeup Control status.
  * @param     p_obj       :I2CS object.
  * @param     p_status    :I2CS status. Destination address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_get_status(EI2CS_t *p_obj, uint8_t *p_status)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_status));
#endif /* #ifdef __DEBUG__ */

    *p_status = p_obj->p_instance->WUPSL;
}

/*--------------------------------------------------*/
/**
  * @brief     Primary Slave 7Bit Address setting.
  * @param     p_obj       :EI2CS object.
  * @param     addr        :Primary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_Primary_slave_adr_set(EI2CS_t *p_obj, uint8_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR2 = (0xFE & adr);
    p_obj->p_instance->WUPCR4 = I2CSWUPCR4_WUPSAFS1_7BIT;
}

/*--------------------------------------------------*/
/**
  * @brief     Primary Slave 10Bit Address setting.
  * @param     p_obj       :I2CS object.
  * @param     addr        :Primary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_Primary_slave_10b_adr_set(EI2CS_t *p_obj, uint32_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR2 = (0xFE & adr);
    p_obj->p_instance->WUPCR4 = (0x07 & (adr >> 8));
    p_obj->p_instance->WUPCR4 |= I2CSWUPCR4_WUPSAFS1_10BIT;
}

/*--------------------------------------------------*/
/**
  * @brief     Secondary Slave 7Bit Address setting.
  * @param     p_obj       :EI2CS object.
  * @param     addr        :Secondary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_Secondary_slave_adr_set(EI2CS_t *p_obj, uint8_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR3 = (0xFE & adr);
    p_obj->p_instance->WUPCR5 = I2CSWUPCR5_WUPSAFS2_7BIT;
    p_obj->p_instance->WUPCR3 |= 0x01;        /* WUPSA2EN: Secondary Slave Address Use Setting */
}

/*--------------------------------------------------*/
/**
  * @brief     Secondary Slave 10Bit Address setting.
  * @param     p_obj       :EI2CS object.
  * @param     addr        :Secondary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void EI2CS_Secondary_slave_10b_adr_set(EI2CS_t *p_obj, uint32_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR3 = (0xFE & adr);
    p_obj->p_instance->WUPCR5 = (0x07 & (adr >> 8));
    p_obj->p_instance->WUPCR5 |= I2CSWUPCR5_WUPSAFS2_10BIT;
    p_obj->p_instance->WUPCR3 |= 0x01;        /* WUPSA2EN: Secondary Slave Address Use Setting */
}
#endif /* (EI2CS_EQUIPPED == AVAILABLE) */
/**
 *  @}
 */ /* End of group EI2C_Exported_functions */

/**
 *  @} 
 */ /* End of group EI2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */

