/**
 *******************************************************************************
 * @file    except.c
 * @brief   This file provides API functions for EXCEPT driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "cg.h"
#include "except.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup EXCEPT
 *  @brief      EXCEPT Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Private_define EXCEPT Private Define
 *  @{
 */
/* no define */
/** 
 *  @}
 */ /* End of group EXCEPT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Private_define EXCEPT Private Define
 *  @{
 */
#define EXCEPT_INTEN_ENABLE             ((uint8_t)0x01)             /*!< INTEN enable */
#define EXCEPT_INTEN_DISABLE            ((uint8_t)0x00)             /*!< INTEN disable */

#define EXCEPT_INTPCLR                  ((uint8_t)0x40)             /*!< INTP clear */
#define EXCEPT_INTNCLR                  ((uint8_t)0x80)             /*!< INTN clear */

#define RESET_CLEAR_LVD                 ((uint8_t)0xBF)             /*!< RESET clear LVD */
#define RESET_CLEAR_PINR                ((uint8_t)0xF7)             /*!< RESET clear PINR */
#define RESET_CLEAR_POR                 ((uint8_t)0xFE)             /*!< RESET clear POR */
#define RESET_CLEAR_OFD                 ((uint8_t)0xF7)             /*!< RESET clear OFD */
#define RESET_CLEAR_WDT                 ((uint8_t)0xFB)             /*!< RESET clear WDT */
#define RESET_CLEAR_LOCK                ((uint8_t)0xFD)             /*!< RESET clear LOCK */
#define RESET_CLEAR_SYS                 ((uint8_t)0xFE)             /*!< RESET clear SYS */

/** 
 *  @}
 */ /* End of group EXCEPT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Private_define EXCEPT Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EXCEPT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Private_typedef EXCEPT Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group EXCEPT_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Private_fuctions EXCEPT Private Fuctions
 *  @{
 */

#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_target(int32_t param);
__STATIC_INLINE int32_t check_param_nicimc(int32_t param);
__STATIC_INLINE int32_t check_param_iaimc(int32_t param);
__STATIC_INLINE int32_t check_param_ibimc(int32_t param);
__STATIC_INLINE int32_t check_param_imnflag(uint32_t param);

/*--------------------------------------------------*/
/** 
  * @brief  Check the target's parameter.
  * @param  param    :target's parameter
  * @retval EXCEPT_PARAM_OK :Valid
  * @retval EXCEPT_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_target(int32_t param)
{
    int32_t result = EXCEPT_PARAM_NG;

    switch(param){
        case INT_REGISTER_A:
        case INT_REGISTER_B:
            result = EXCEPT_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the nicimc's parameter.
  * @param  param    :nicimc's parameter
  * @retval EXCEPT_PARAM_OK :Valid
  * @retval EXCEPT_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_nicimc(int32_t param)
{
    int32_t result = EXCEPT_PARAM_NG;

    switch(param){
        case INT_NIC:
        case INT_IMC:
            result = EXCEPT_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :iaimc's parameter
  * @retval EXCEPT_PARAM_OK :Valid
  * @retval EXCEPT_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_iaimc(int32_t param)
{
    int32_t result = EXCEPT_PARAM_NG;

    if(param <= IA_IMC_MAX){
        result = EXCEPT_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :ibimc's parameter
  * @retval EXCEPT_PARAM_OK :Valid
  * @retval EXCEPT_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ibimc(int32_t param)
{
    int32_t result = EXCEPT_PARAM_NG;

    if(param <= IB_IMC_MAX){
        result = EXCEPT_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :imnflag's parameter
  * @retval EXCEPT_PARAM_OK :Valid
  * @retval EXCEPT_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_imnflag(uint32_t param)
{
    int32_t result = EXCEPT_PARAM_NG;

    if(param <= IMN_FLAG_MAX){
        result = EXCEPT_PARAM_OK;
    }

    return (result);
}

#endif /* __DEBUG__ */

__STATIC_INLINE uint8_t *get_address(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc)
{
    uint8_t *address = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_target(target));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
    assert_param(check_param_nicimc(nicimc));
    if((target == INT_REGISTER_A) || (nicimc == INT_IMC)){
        assert_param(check_param_iaimc((int32_t)imc));
    }else if((target == INT_REGISTER_B) || (nicimc == INT_IMC)){
        assert_param(check_param_ibimc((int32_t)imc));
    }
#endif /* #ifdef __DEBUG__ */

    if(target == INT_REGISTER_A){
        if(nicimc == INT_NIC){
            address = (uint8_t *)((uint32_t)p_obj->p_instance1 | IA_NIC_START);
        }else{
            address = (uint8_t *)((uint32_t)p_obj->p_instance1 | (IA_IMC_START + imc));
        }

    }else if(target == INT_REGISTER_B){
        if(nicimc == INT_NIC){
            address = (uint8_t *)((uint32_t)p_obj->p_instance2 | IB_NIC_START);
        }else{
            address = (uint8_t *)((uint32_t)p_obj->p_instance2 | (IB_IMC_START + imc));
        }
    }else{
        /* do nothing */
    }
    return address;
}


/**
 *  @}
 */ /* End of group EXCEPT_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup EXCEPT_Exported_functions EXCEPT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Set intteruput mode.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     mode   : Select interrupt mode.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result except_set_mode(except_t *p_obj, except_register_t target, except_mode_t mode, uint32_t imc, except_int_t nicimc)
{
    uint8_t regval = 0;
    TXZ_Result result = TXZ_SUCCESS;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */
    
    if(target == INT_REGISTER_A){
        if(nicimc == INT_NIC){
            if(IA_NIC_MODE == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ia_mode_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else if(target == INT_REGISTER_B){
        if(nicimc == INT_NIC){
            if(IB_NIC_MODE == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ib_mode_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else{
        result = TXZ_ERROR;
    }

    if(result == TXZ_SUCCESS){
        address = get_address(p_obj, target, imc, nicimc);
        regval = *address & ~INT_MODE_MASK;
        regval |= mode << 1;
        *address = regval;
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     clear falling edge intteruput flag.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     mode   : Select interrupt mode.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result except_clear_intn(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */

    if(target == INT_REGISTER_A){
        if(nicimc == INT_NIC){
            if(IA_NIC_INTN == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ia_imc_intn_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else if(target == INT_REGISTER_B){
        if(nicimc == INT_NIC){
            if(IB_NIC_INTN == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ib_imc_intn_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else{
        result = TXZ_ERROR;
    }

    if(result == TXZ_SUCCESS){
        address = get_address(p_obj, target, imc, nicimc);
        *address |= INT_INTN_CLEAR_BIT;
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     clear rising edge intteruput flag.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     mode   : Select interrupt mode.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result except_clear_intp(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */

    if(target == INT_REGISTER_A){
        if(nicimc == INT_NIC){
            if(IA_NIC_INTP == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ia_imc_intp_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else if(target == INT_REGISTER_B){
        if(nicimc == INT_NIC){
            if(IB_NIC_INTP == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ib_imc_intp_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else{
        result = TXZ_ERROR;
    }

    if(result == TXZ_SUCCESS){
        address = get_address(p_obj, target, imc, nicimc);
        *address |= INT_INTP_CLEAR_BIT;
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     get falling edge intteruput flag.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     mode   : Select interrupt mode.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
except_state except_get_intn(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc)
{
    except_state result = INTERRUPT_NOT_DETECT;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */

    address = get_address(p_obj, target, imc, nicimc);

    if(*address & INT_INTN_FLAG){
        result = INTERRUPT_DETECT;
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     get rising edge intteruput flag.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     mode   : Select interrupt mode.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
except_state except_get_intp(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc)
{
    except_state result = INTERRUPT_NOT_DETECT;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */

    address = get_address(p_obj, target, imc, nicimc);

    if(*address & INT_INTP_FLAG){
        result = INTERRUPT_DETECT;
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     set inten bit.
  * @param     p_obj  : EXCEPT object.
  * @param     target : Select interrupt register.
  * @param     imc    : Select interrupt num.
  * @param     nicimc : Select nic or imc.
  * @param     status : select inten status.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result except_set_inten(except_t *p_obj, except_register_t target, uint32_t imc, except_int_t nicimc, TXZ_FunctionalState status)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint8_t *address;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    if(target == INT_REGISTER_A){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance1));
    }else if(target == INT_REGISTER_B){
        assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance2));
    }
#endif /* #ifdef __DEBUG__ */

    if(target == INT_REGISTER_A){
        if(nicimc == INT_NIC){
            if(IA_NIC_INTEN == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ia_inten_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else if(target == INT_REGISTER_B){
        if(nicimc == INT_NIC){
            if(IB_NIC_INTEN == UNAVAILABLE){
                result = TXZ_ERROR;
            }
        }else{
            if(!(ib_inten_table[imc/32] & (1 << (imc % 32)))){
                result = TXZ_ERROR;
            }
        }
    }else{
        result = TXZ_ERROR;
    }

    if(result == TXZ_SUCCESS){
        address = get_address(p_obj, target, imc, nicimc);
        if(status == TXZ_ENABLE){
            *address |= INT_INTEN_BIT;
        }else{
            *address &= ~INT_INTEN_BIT;
        }
    }else{
        /* do nothing */
    }
    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     get rlm reset flag0.
  * @param     p_obj  : EXCEPT object.
  * @param     target : select reset flag.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
except_state rlm_get_flag0(rlm_t *p_obj, rlm_rstflg0_t target)
{
    except_state result = INTERRUPT_NOT_DETECT;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(p_obj->p_instance->RSTFLG0 & (1 << target)){
        result = INTERRUPT_DETECT;
    }else{
        /* do nothing */
    }

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     get rlm reset flag1.
  * @param     p_obj  : EXCEPT object.
  * @param     target : select reset flag.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
except_state rlm_get_flag1(rlm_t *p_obj, rlm_rstflg1_t target)
{
    except_state result = INTERRUPT_NOT_DETECT;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(p_obj->p_instance->RSTFLG1 & (1 << target)){
        result = INTERRUPT_DETECT;
    }else{
        /* do nothing */
    }

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     clear rlm reset flag0.
  * @param     p_obj  : EXCEPT object.
  * @param     target : select reset flag.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result rlm_clear_flag0(rlm_t *p_obj, rlm_rstflg0_t target)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->RSTFLG0 &= ~(1 << target);

    if(p_obj->p_instance->RSTFLG0 & (1 << target)){
        result = TXZ_ERROR;
    }else{
        /* do nothing */
    }

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     clear rlm reset flag1.
  * @param     p_obj  : EXCEPT object.
  * @param     target : select reset flag.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result rlm_clear_flag1(rlm_t *p_obj, rlm_rstflg1_t target)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->RSTFLG1 &= ~(1 << target);

    if(p_obj->p_instance->RSTFLG1 & (1 << target)){
        result = TXZ_ERROR;
    }else{
        /* do nothing */
    }

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief     get imn flag.
  * @param     p_obj  : EXCEPT object.
  * @param     target : select reset flag.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
except_state imn_get_flag(except_t *p_obj, uint32_t target)
{
    except_state result = INTERRUPT_NOT_DETECT;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the EXCEPT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance3));
    assert_param(check_param_imnflag(target));
#endif /* #ifdef __DEBUG__ */

    uint32_t *address = (uint32_t *)((uint32_t)p_obj->p_instance3 | ((target / 32) * 4));

    if(*address & (1 << (target % 32))){
        result = INTERRUPT_DETECT;
    }else{
        /* do nothing */
    }

    return result;
}



/**
 *  @} 
 */ /* End of group EXCEPT_Exported_functions */

/**
 *  @} 
 */ /* End of group EXCEPT */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */

