/**
 *******************************************************************************
 * @file    lvd_d.c
 * @brief   This file provides API functions for LVD.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "driver_com.h"
#include "lvd.h"
#include "sys_timer.h"

#if defined(__LVD_D_H)

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup LVD
 *  @brief      LVD Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */
/* no define */
/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_define LVD Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group LVD_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_typedef LVD Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group LVD_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Private_fuctions LVD Private Fuctions
 *  @{
 */
#ifdef __DEBUG__
   __STATIC_INLINE int32_t check_param_en(uint8_t param);
   __STATIC_INLINE int32_t check_param_outen(uint8_t param);
   __STATIC_INLINE int32_t check_param_sel(uint8_t param);
   __STATIC_INLINE int32_t check_param_lvl(uint8_t param);

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_en(uint32_t param)
  * @brief  Check the Operation control's parameter
  * @param[in] param :Operation control's parameter
  * @retval    LVD_PARAM_OK :Success.
  * @retval    LVD_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_en(uint8_t param)
{
     int32_t result = LVD_PARAM_NG;

    switch (param) 
    {
    case LVD_ENABLE:
    case LVD_DISABLE:
        result = LVD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_outen(uint32_t param)
  * @brief  Check the Output setting's parameter
  * @param[in] param :Output setting's parameter
  * @retval    LVD_PARAM_OK :Success.
  * @retval    LVD_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_outen(uint8_t param)
{
     int32_t result = LVD_PARAM_NG;

    switch (param) 
    {
    case LVD_OUT_ENABLE:
    case LVD_OUT_DISABLE:
        result = LVD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_sel(uint32_t param)
  * @brief  Check the INT select's parameter
  * @param[in] param :INT select's parameter
  * @retval    LVD_PARAM_OK :Success.
  * @retval    LVD_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sel(uint8_t param)
{
     int32_t result = LVD_PARAM_NG;

    switch (param) 
    {
    case LVD_SEL_INT:
    case LVD_SEL_RESET:
        result = LVD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_lvl(uint32_t param)
  * @brief  Check the Voltage setting's parameter
  * @param[in] param :Voltage setting's parameter
  * @retval    LVD_PARAM_OK :Success.
  * @retval    LVD_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_lvl(uint8_t param)
{
     int32_t result = LVD_PARAM_NG;

    switch (param) 
    {
    case LVD_VOLTAGE_265:
    case LVD_VOLTAGE_27:
    case LVD_VOLTAGE_28:
    case LVD_VOLTAGE_29:
    case LVD_VOLTAGE_40:
    case LVD_VOLTAGE_42:
    case LVD_VOLTAGE_44:
    case LVD_VOLTAGE_46:
        result = LVD_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
#endif /* #ifdef __DEBUG__ */

/**
 *  @}
 */ /* End of group LVD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     Initialize the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void init_LVD(lvd_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the LVD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_en(p_obj->init.en));
    assert_param(check_param_outen(p_obj->init.outen));
    assert_param(check_param_sel(p_obj->init.sel));
    assert_param(check_param_lvl(p_obj->init.lvl));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->CR &= LVD_OUT_DISABLE;
    p_obj->p_instance->CR &= LVD_DISABLE;
    p_obj->p_instance->CR &= LVD_VOLTAGE_265;

    if(p_obj->init.sel == LVD_SEL_INT){
        p_obj->p_instance->CR |= p_obj->init.sel;
    }
    else{
        p_obj->p_instance->CR &= p_obj->init.sel;
    }

    p_obj->p_instance->CR |= p_obj->init.lvl;
    wait(100);

    if(p_obj->init.en == LVD_ENABLE){
        p_obj->p_instance->CR |= p_obj->init.en;
        
    }
    else{
        p_obj->p_instance->CR &= p_obj->init.en;
    }
    wait(1000);

    if(p_obj->init.outen == LVD_OUT_ENABLE){
        p_obj->p_instance->CR |= p_obj->init.outen;
    }
    else{
        p_obj->p_instance->CR &= p_obj->init.outen;
    }

}

/*--------------------------------------------------*/
/**
  * @brief     Start the LVD object.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void start_LVD(lvd_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the LVD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(p_obj->init.outen == LVD_OUT_ENABLE){
        p_obj->p_instance->CR &= (uint8_t)LVD_OUT_DISABLE;
    }
    p_obj->p_instance->CR |= (uint8_t)LVD_ENABLE;                  /* LVD Enable */
    wait(1000);

    if(p_obj->init.outen == LVD_OUT_ENABLE){
        p_obj->p_instance->CR |= p_obj->init.outen;
    }
    else{
        p_obj->p_instance->CR &= p_obj->init.outen;
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Get the LVD register.
  * @param     -
  * @retval    LVD register value
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint8_t GetCR_LVD(lvd_t *p_obj)
{
    uint8_t retval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the LVD_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    retval = p_obj->p_instance->CR;
    return retval;
}

/**
 *  @} 
 */ /* End of group LVD_Exported_functions */
 
/**
 *  @} 
 */ /* End of group LVD */
 
/**
 *  @} 
 */ /* End of group Periph_Driver */
 
#endif /* defined(__LVD_D_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */

