/** 
 *******************************************************************************
 * @file    lvd_d.h
 * @brief   This file provides all the functions prototypes for LVD.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __LVD_D_H
#define __LVD_D_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @defgroup LVD LVD
 *  @brief    LVD Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_define LVD Exported Define
 *  @{
 */
/** 
 *  @defgroup NULL_Pointer NULL Pointer
 *  @brief    NULL Pointer.
 *  @{
 */
#define LVD_NULL        ((void *)0)
/**
 *  @}
 */ /* End of group NULL_Pointer */

/** 
 *  @defgroup Parameter_Result Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define LVD_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define LVD_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group Parameter_Result */


#define LVD_UPPER           ((uint8_t)0x00)          /*!< LVD UPPER                */
#define LVD_LOWER           ((uint8_t)0x01)          /*!< LVD LOWER                */

#define LVD_VOLTAGE_265     ((uint8_t)0x00)          /*!< LVD voltage 2.65+-0.1V   */
#define LVD_VOLTAGE_27      ((uint8_t)0x10)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint8_t)0x20)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint8_t)0x30)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_40      ((uint8_t)0x40)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint8_t)0x50)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint8_t)0x60)          /*!< LVD voltage 4.4+-0.1V    */
#define LVD_VOLTAGE_46      ((uint8_t)0x70)          /*!< LVD voltage 4.6+-0.1V    */

#define LVD_SEL_INT         ((uint8_t)0x04)          /*!< LVD INTLVD               */
#define LVD_SEL_RESET       ((uint8_t)0xFB)          /*!< LVD reset                */
#define LVD_OUT_ENABLE      ((uint8_t)0x02)          /*!< LVD output enable        */
#define LVD_OUT_DISABLE     ((uint8_t)0xFD)          /*!< LVD output disable       */
#define LVD_ENABLE          ((uint8_t)0x01)          /*!< LVD enable               */
#define LVD_DISABLE         ((uint8_t)0xFE)          /*!< LVD disable              */
/**
 *  @}
 */ /* End of group LVD_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_define LVD Exported Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group LVD_Exported_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Exported_typedef LVD Exported Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t lvl;                         /*!< Voltage setting.                  */
    uint8_t sel;                         /*!< INT select.                       */
    uint8_t outen;                       /*!< Output setting.                   */
    uint8_t en;                          /*!< Operation control.                */
} lvd_initial_setting_t;

typedef struct
{
    TSB_LVD_TypeDef       *p_instance;     /*!< Registers base address.           */
    lvd_initial_setting_t init;            /*!< Initial setting.                  */
} lvd_t;

/**
 *  @}
 */ /* End of group LVD_Exported_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup LVD_Exported_functions LVD Exported Functions
 *  @{
 */
void init_LVD(lvd_t *p_obj);
void start_LVD(lvd_t *p_obj);
uint8_t GetCR_LVD(lvd_t *p_obj);

/**
 *  @}
 */ /* End of group LVD_Exported_functions */

/**
 *  @}
 */ /* End of group LVD */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __LVD_D_H */


