/**
 *******************************************************************************
 * @file    port.c
 * @brief   This file provides API functions for PORT driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "port.h"
     
#if defined(__PORT_H)
/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @defgroup PORT PORT
 *  @brief    PORT Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_define PORT Private Define
 *  @{
 */
/** 
 *  @name  Parameter Result
 *  @brief Whether the parameter is specified or not.
 *  @{
 */
#define PARAM_OK    ((int32_t)1)    /*!< Parameter is valid(specified).         */
#define PARAM_NG    ((int32_t)0)    /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter Result */
 
/** 
 *  @name  Shift Count Max
 *  @brief The shift count max value indicating whether the func/mode is valid or not.
 *  @{
 */
#define SHIFT_COUNT_MAX ((uint8_t)16)
/**
 *  @}
 */ /* End of Shift Count Max */
 
/** 
 *  @name  Bit Operation Macro
 *  @brief Whether the parameter is specified or not.
 *  @{
 */
#define BITBAND_PORT_OFFSET (0x0000100UL) /*!< Port Register Offset Value */
#define BITBAND_PORT_BASE(gr) (PORT_BASE + (uint32_t)((BITBAND_PORT_OFFSET) * (gr)) ) /*!<  Operational target Port Adress */
#define BITBAND_PORT_MODE_BASE(base, pinmode) ((uint32_t)(base) + (uint32_t)(pinmode) )  /*!< Operational target Control Register Adress */
#define BITBAND_PORT_SET(base, bitnum) (*((__IO uint32_t *)base) |= (uint32_t)(0x0000001UL<< bitnum)) /*!< Target Pin Bit set */
#define BITBAND_PORT_CLR(base, bitnum) (*((__IO uint32_t *)base) &= ~((uint32_t)(0x0000001UL<< bitnum))) /*!< Target Pin Bit clear */
#define BITBAND_PORT_READ(val, base, bitnum) val = ((*((__IO uint32_t *)base) & (uint32_t)(0x0000001UL<< bitnum)) >> bitnum) /*!< Target Pin Bit read */
/**
 *  @}
 */ /* End of Bit Operation Macro */
/**
 *  @}
 */ /* End of group PORT_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_define PORT Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group PORT_Private_define */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_typedef PORT Private Typedef
 *  @{
 */
/**
 *  @}
 */ /* End of group PORT_Private_typedef */
 
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup PORT_Private_fuctions PORT Private Fuctions
 *  @{
 */
TXZ_Result gpio_set(gpio_gr_t group, uint32_t enable);
static uint32_t change_num_to_mode( uint8_t num );
#ifdef __DEBUG__
    __STATIC_INLINE uint8_t change_func_to_num( uint32_t mode );
    __STATIC_INLINE uint8_t change_mode_to_num( uint32_t mode );
    __STATIC_INLINE int32_t check_param_pin_exist(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode);
    __STATIC_INLINE int32_t check_param_func_pin_exist(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode);
#endif /* #ifdef __DEBUG__ */



static uint32_t change_num_to_mode( uint8_t num )
{
    uint32_t retVal = 0;
    if (num == 0) {retVal = GPIO_Mode_DATA;}
    else if (num == 1) {retVal = GPIO_Mode_CR;}
    else if (num == 2) {retVal = GPIO_Mode_FR1;}
    else if (num == 3) {retVal = GPIO_Mode_FR2;}
    else if (num == 4) {retVal = GPIO_Mode_FR3;}
    else if (num == 5) {retVal = GPIO_Mode_FR4;}
    else if (num == 6) {retVal = GPIO_Mode_FR5;}
    else if (num == 7) {retVal = GPIO_Mode_FR6;}
    else if (num == 8) {retVal = GPIO_Mode_FR7;}
    else if (num == 9) {retVal = GPIO_Mode_FR8;}
    else if (num == 10) {retVal = GPIO_Mode_OD;}
    else if (num == 11) {retVal = GPIO_Mode_PUP;}
    else if (num == 12) {retVal = GPIO_Mode_PDN;}
    else if (num == 13) {retVal = GPIO_Mode_RSV0;}
    else if (num == 14) {retVal = GPIO_Mode_IE;}
    else if (num == 15) {retVal = GPIO_Mode_RSV1;}

    return retVal;
}

static uint8_t change_mode_to_num( uint32_t mode )
{
    uint8_t retVal = 0;

    if (mode == GPIO_Mode_DATA) {retVal = 0;}
    else if (mode == GPIO_Mode_CR) {retVal = 1;}
    else if (mode == GPIO_Mode_FR1) {retVal = 2;}
    else if (mode == GPIO_Mode_FR2) {retVal = 3;}
    else if (mode == GPIO_Mode_FR3) {retVal = 4;}
    else if (mode == GPIO_Mode_FR4) {retVal = 5;}
    else if (mode == GPIO_Mode_FR5) {retVal = 6;}
    else if (mode == GPIO_Mode_FR6) {retVal = 7;}
    else if (mode == GPIO_Mode_FR7) {retVal = 8;}
    else if (mode == GPIO_Mode_FR8) {retVal = 9;}
    else if (mode == GPIO_Mode_OD) {retVal = 10;}
    else if (mode == GPIO_Mode_PUP) {retVal = 11;}
    else if (mode == GPIO_Mode_PDN) {retVal = 12;}
    else if (mode == GPIO_Mode_RSV0) {retVal = 13;}
    else if (mode == GPIO_Mode_IE) {retVal = 14;}
    else if (mode == GPIO_Mode_RSV1) {retVal = 15;}
    else {retVal = SHIFT_COUNT_MAX;}

    return retVal;
}

static int32_t register_exists(gpio_gr_t group, gpio_mode_t mode)
{
    uint32_t result = PARAM_NG;
    uint8_t count = 0;

    count = change_mode_to_num((uint32_t)mode);
    if (count < SHIFT_COUNT_MAX)
    {
        if ((RegExistTbl[(uint32_t)group] >> count) & 0x01) result = PARAM_OK;
    }
    
    return result;
}

static void clear_port(uint32_t  port_base, gpio_gr_t group, gpio_num_t num, gpio_mode_t mode)
{
    uint32_t mode_base;
    if (register_exists(group, mode))
    {
        mode_base = BITBAND_PORT_MODE_BASE(port_base, mode);
        BITBAND_PORT_CLR(mode_base, num);
    }
}
static void set_port(uint32_t  port_base, gpio_gr_t group, gpio_num_t num, gpio_mode_t mode)
{
    uint32_t mode_base;
    if (register_exists(group, mode))
    {
        mode_base = BITBAND_PORT_MODE_BASE(port_base, mode);
        BITBAND_PORT_SET(mode_base, num);
    }
}

#ifdef __DEBUG__
__STATIC_INLINE uint8_t change_func_to_num( uint32_t mode )
{
    uint8_t retVal = 0;

    /* convert gpio_fr_t(GPIO_FR_1 ~ GPIO_FR_8) to shift count of PinExistTbl(register bit-data) */
    /*   ex: GPIO_FR_6 -> data >> 7 = data >> ((6 - 1) % 8) + 2 */
    /*   note: param check skip if func is INPUT or OUTPUT        */
    if (((uint32_t)GPIO_FR_1 <= mode ) && (mode <= (uint32_t)GPIO_FR_Max)) {
        /* + 2 = lower GPIO_Mode_CR + GPIO_Mode_DATA bit */
        retVal = (mode - (uint32_t)GPIO_FR_1) % ((uint32_t)GPIO_FR_Max) + 2;
    } else if ((mode == 0) || (mode == GPIO_FR_NA)) {
        retVal = 1; /* 1 means skip */
    } else {
        retVal = SHIFT_COUNT_MAX;
    }

    return retVal;
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_pin_exist(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode)
  * @brief  Check the Pin Exist.
  * @param[in] p_obj :GPIO object.
  * @param[in] group :PORT Port Group.  : Use @ref gpio_gr_t
  * @param[in] num   :GPIO Port Number. : Use @ref gpio_num_t
  * @param[in] mode  :GPIO Port Mode. : Use @ref gpio_mode_t
  * @retval PARAM_OK :Valid
  * @retval PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_pin_exist(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode)
{
    int32_t result = PARAM_NG;
    uint8_t chgmode;
    uint16_t tmp;

    chgmode = change_mode_to_num(mode);
    if ((chgmode < SHIFT_COUNT_MAX) && (group < GPIO_GROUP_Max) && (num < GPIO_PORT_Max))
    {
        tmp = (PinExistTbl[group][num] >> chgmode) & 0x01;
        result = PARAM_OK;
        if (tmp == 0) {result = PARAM_NG;}
    }
    else
        {result = PARAM_NG;}

    return (result);
}

__STATIC_INLINE int32_t check_param_func_pin_exist(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode)
{
    int32_t result = PARAM_NG;
    uint8_t chgfunc;
    uint16_t tmp;

    chgfunc = change_func_to_num(mode);
    /* param check skip if func is INPUT or OUTPUT */
    if(chgfunc == 1)
    {
        return (PARAM_OK);
    }       
    if ((chgfunc < SHIFT_COUNT_MAX) && (group < GPIO_GROUP_Max) && (num < GPIO_PORT_Max))
    {
        tmp = (PinExistTbl[group][num] >> chgfunc) & 0x01;
        result = PARAM_OK;
        if (tmp == 0) {result = PARAM_NG;}
    }
    else
        {result = PARAM_NG;}

    return (result);
}
#endif /* #ifdef __DEBUG__ */
/**
 *  @}
 */ /* End of group PORT_Private_functions */



/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup PORT_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_init(gpio_gr_t group)
  * @brief     Initialize the PORT object.
  * @param[in] group :PORT Port Group.  : Use @ref gpio_gr_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note      An initial value of default is set at the
               reset status value. If needed, please
               rewrite and use an initial value.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/

TXZ_Result gpio_init(gpio_gr_t group)
{
    TXZ_Result result = TXZ_SUCCESS;

    gpio_set( group, 1U) ;

    return (result);
}


/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_deinit(gpio_gr_t group)
  * @brief     Release the PORT object.
  * @param[in] group :PORT Port Group.  : Use @ref gpio_gr_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note      An initial value of default is set at the
               reset status value. If needed, please
               rewrite and use an initial value.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/

TXZ_Result gpio_deinit(gpio_gr_t group)
{
    TXZ_Result result = TXZ_SUCCESS;

    gpio_set( group, 0U) ;

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_set(gpio_gr_t group, uint32_t enable)
  * @brief     Set the PORT object.
  * @param[in] group :PORT Port Group.  : Use @ref gpio_gr_t
  * @param[in] enable :GPIO Port Enable.  : enable=1, release=0
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note      An initial value of default is set at the
               reset status value. If needed, please
               rewrite and use an initial value.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_set(gpio_gr_t group, uint32_t enable)
{
    TXZ_Result result = TXZ_SUCCESS;
    reg_cg_enable_t reg;
    uint32_t i = 0;

    for(i = 0; i < GPIO_Mode_Max; i++){
        if(RegExistTbl[group] & (0x1 << i)){
            *(uint32_t *)(TSB_PA_BASE + (group << 8) + change_num_to_mode(i)) = PinResetTbl[group][i];
        }
    }

    reg = RegCgEnableTbl[group];
    if (reg.addr != REG_NONE)
    {
        *((__IO uint32_t *)BITBAND_PERI(reg.addr, reg.bit)) = enable;
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_write_mode(gpio_t *p_obj, uint32_t group, uint32_t mode, uint32_t val)
  * @brief  Port Mode Write
  * @param[in] p_obj :GPIO object.
  * @param[in] group :PORT Port Group. : Use @ref gpio_gr_t
  * @param[in] mode  :GPIO Port Mode. : Use @ref gpio_num_t
  * @param[in] val   :value
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_write_mode(gpio_t *p_obj, gpio_gr_t group, gpio_mode_t mode, uint32_t val)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    int32_t i;
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, mode));
    for (i = GPIO_PORT_0; i < GPIO_PORT_Max; i++ )
    {
        if (val & 0x1U << i){
            assert_param(check_param_pin_exist(p_obj, group, i, mode));
        }
    }
#endif /* #ifdef __DEBUG__ */

    *(uint32_t *)(TSB_PA_BASE + (group << 8) + mode) = val;

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_read_mode(uint32_t group, uint32_t mode, uint32_t *val)
  * @brief  Port Mode Read
  * @param[in]  group :PORT Port Group.  : Use @ref gpio_gr_t
  * @param[in]  mode  :GPIO Port Mode. : Use @ref gpio_num_t
  * @param[out] val   :Store of value
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_read_mode(gpio_gr_t group, gpio_mode_t mode, uint32_t *val)
{
    TXZ_Result result = TXZ_SUCCESS;

#ifdef __DEBUG__
    assert_param(register_exists(group, mode));
    assert_param(IS_POINTER_NOT_NULL(val));
#endif /* #ifdef __DEBUG__ */

    *val = *(uint32_t *)(TSB_PA_BASE + (group << 8) + mode);

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief  Port Function switching
  * @param  p_obj  :GPIO object.
  * @param  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param  func   :GPIO Portxx Func. : Use @ref gpio_pa0_func_t - @ref gpio_pl4_func_t
  * @param  inout  :GPIO Input/Output.: Use @ref gpio_pininout_t
  * @retval GPIO_RESULT_SUCCESS :Success.
  * @retval GPIO_RESULT_FAILURE :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_func(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, uint32_t func, gpio_pininout_t inout)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t port_base;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_func_pin_exist(p_obj, group, num, func));
#endif /* #ifdef __DEBUG__ */

    port_base = (uint32_t)TSB_PA_BASE + (group << 8);

    /* Initialization PxFR OFF */    
    clear_port(port_base, group, num, GPIO_Mode_FR1);
    clear_port(port_base, group, num, GPIO_Mode_FR2);
    clear_port(port_base, group, num, GPIO_Mode_FR3);
    clear_port(port_base, group, num, GPIO_Mode_FR4);
    clear_port(port_base, group, num, GPIO_Mode_FR5);
    clear_port(port_base, group, num, GPIO_Mode_FR6);
    clear_port(port_base, group, num, GPIO_Mode_FR7);
    clear_port(port_base, group, num, GPIO_Mode_FR8);

    /* Initialize Input/Output */
    clear_port(port_base, group, num, GPIO_Mode_CR);
    clear_port(port_base, group, num, GPIO_Mode_IE);

    if(inout & 0x1U){
        set_port(port_base, group, num, GPIO_Mode_IE);
    }
    switch(func){
        case GPIO_FR_1:
            set_port(port_base, group, num, GPIO_Mode_FR1);
            break;
        case GPIO_FR_2:
            set_port(port_base, group, num, GPIO_Mode_FR2);
            break;
        case GPIO_FR_3:
            set_port(port_base, group, num, GPIO_Mode_FR3);
            break;
        case GPIO_FR_4:
            set_port(port_base, group, num, GPIO_Mode_FR4);
            break;
        case GPIO_FR_5:
            set_port(port_base, group, num, GPIO_Mode_FR5);
            break;
        case GPIO_FR_6:
            set_port(port_base, group, num, GPIO_Mode_FR6);
            break;
        case GPIO_FR_7:
            set_port(port_base, group, num, GPIO_Mode_FR7);
            break;
        case GPIO_FR_8:
            set_port(port_base, group, num, GPIO_Mode_FR8);
            break;
        default:
            break;
    }

    if(inout & 0x2U){
        set_port(port_base, group, num, GPIO_Mode_CR);
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Set Pull up mode
  * @param  p_obj  :GPIO object.
  * @param  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param  val    :GPIO Pin Reset/Set. : Use @ref gpio_pinstate_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_SetPullUp(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, gpio_pinstate_t val)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t port_base;
    uint32_t mode_base;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, GPIO_Mode_PUP));
    assert_param(check_param_pin_exist(p_obj, group, num, GPIO_Mode_PUP));
#endif /* #ifdef __DEBUG__ */

    port_base = BITBAND_PORT_BASE(group);

    mode_base = BITBAND_PORT_MODE_BASE(port_base, GPIO_Mode_PUP);
    if (val == GPIO_PIN_SET)
       {BITBAND_PORT_SET(mode_base, num);}
    else if (val == GPIO_PIN_RESET)
       {BITBAND_PORT_CLR(mode_base, num);}
    else {result = TXZ_ERROR;}

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief  Set Pull down mode
  * @param  p_obj  :GPIO object.
  * @param  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param  val    :GPIO Pin Reset/Set. : Use @ref gpio_pinstate_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_SetPullDown(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, gpio_pinstate_t val)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t port_base;
    uint32_t mode_base;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, GPIO_Mode_PDN));
    assert_param(check_param_pin_exist(p_obj, group, num, GPIO_Mode_PDN));
#endif /* #ifdef __DEBUG__ */

    port_base = BITBAND_PORT_BASE(group);

    mode_base = BITBAND_PORT_MODE_BASE(port_base, GPIO_Mode_PDN);
    if (val == GPIO_PIN_SET)
        {BITBAND_PORT_SET(mode_base, num);}
    else if (val == GPIO_PIN_RESET)
        {BITBAND_PORT_CLR(mode_base, num);}
    else {result = TXZ_ERROR;}

    return result;
}

/*--------------------------------------------------*/
/**
  * @brief  Set Open drain mode
  * @param  p_obj  :GPIO object.
  * @param  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param  val    :GPIO Pin Reset/Set. : Use @ref gpio_pinstate_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_SetOpenDrain(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, gpio_pinstate_t val)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t port_base;
    uint32_t mode_base;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, GPIO_Mode_OD));
    assert_param(check_param_pin_exist(p_obj, group, num, GPIO_Mode_OD));
#endif /* #ifdef __DEBUG__ */

    port_base = BITBAND_PORT_BASE(group);

    mode_base = BITBAND_PORT_MODE_BASE(port_base, GPIO_Mode_OD);
    if (val == GPIO_PIN_SET)
        {BITBAND_PORT_SET(mode_base, num);}
    else if (val == GPIO_PIN_RESET) 
        {BITBAND_PORT_CLR(mode_base, num);}
    else {result = TXZ_ERROR;}

    return result;
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_write_bit(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode, uint32_t val)
  * @brief  Port Bit Write
  * @param  p_obj  :GPIO object.
  * @param  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param  mode   :GPIO Port Mode. : Use @ref gpio_mode_t
  * @param  val    :GPIO Pin Reset/Set.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.
  */
/*--------------------------------------------------*/
TXZ_Result gpio_write_bit(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, gpio_mode_t mode, gpio_pinstate_t val)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t base;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, mode));
    assert_param(check_param_pin_exist(p_obj, group, num, mode));
#endif /* #ifdef __DEBUG__ */

    base = BITBAND_PORT_BASE(group);
    base = BITBAND_PORT_MODE_BASE(base, mode);
    if (val == GPIO_PIN_SET) {BITBAND_PORT_SET(base, num);}
    else if (val == GPIO_PIN_RESET) {BITBAND_PORT_CLR(base, num);}
    else {result = TXZ_ERROR;}

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     TXZ_Result gpio_read_bit(gpio_t *p_obj, uint32_t group, uint32_t num, uint32_t mode, gpio_pinstate_t *pinstate)
  * @brief  Port Bit Read
  * @param[in]  p_obj  :GPIO object.
  * @param[in]  group  :PORT Port Group.  : Use @ref gpio_gr_t
  * @param[in]  num    :GPIO Port Number. : Use @ref gpio_num_t
  * @param[in]  mode   :GPIO Port Mode. : Use @ref gpio_mode_t
  * @param[out] *pinstate  : store  Value of GPIO BitPin. : Use @ref gpio_pinstate_t
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   When p_obj is NULL, no processing.: Use @ref gpio_pinstate_t
  */
/*--------------------------------------------------*/
TXZ_Result gpio_read_bit(gpio_t *p_obj, gpio_gr_t group, gpio_num_t num, gpio_mode_t mode, gpio_pinstate_t *pinstate)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t base;
    uint32_t val;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the PORT_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(register_exists(group, mode));
    assert_param(check_param_pin_exist(p_obj, group, num, mode));
    assert_param(IS_POINTER_NOT_NULL(pinstate));
#endif /* #ifdef __DEBUG__ */

    base = BITBAND_PORT_BASE(group);
    base = BITBAND_PORT_MODE_BASE(base, mode);
    BITBAND_PORT_READ(val, base, num);
    if(val == GPIO_PIN_RESET) {*pinstate = GPIO_PIN_RESET;}
    else if (val == GPIO_PIN_SET) {*pinstate = GPIO_PIN_SET;}
    else {result = TXZ_ERROR;}

    return result;
}

/**
 *  @}
 */ /* End of group PORT_Exported_functions */

/**
 *  @} 
 */ /* End of group PORT */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__PORT_H)  */

#ifdef __cplusplus
}
#endif /* __cplusplus */


