/** 
 *******************************************************************************
 * @file    trm_b.h
 * @brief   This file provides all the functions prototypes for TRM.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __TRM_B_H
#define __TRM_B_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup TRM
 *  @brief      TRM Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRM_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_define
 *  @{
 */
/** 
 *  @defgroup NULL_Pointer NULL Pointer
 *  @brief    NULL Pointer.
 *  @{
 */
#define TRM_NULL        ((void *)0)
/**
 *  @}
 */ /* End of group NULL_Pointer */

/** 
 *  @defgroup Parameter_Result Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define TRM_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define TRM_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group Parameter_Result */

/** 
 *  @defgroup TRM_ValueMax Value Max
 *  @brief    Max Value of trm.
 *  @{
 */

#define TRM_VALUE_MAX                 ((uint8_t)0xFF)            /*!< Max Value of conversion. */

/**
 *  @}
 */ /* End of group TRM_ValueMax */

/** 
 *  @defgroup TRM_Control  Trimming Control
 *  @brief    Trimming Control.
 *  @{
 */
#define  TRMOSC_CONTROL_ENABLE      (0x00000001UL)      /*!< Trimming Control enable   */
#define  TRMOSC_CONTROL_DISABLE     (0x00000000UL)      /*!< Trimming Control disable  */
/**
 *  @}
 */ /* End of group TRM_Control */

/** 
 *  @}
 */ /* End of group TRM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  TRM handle structure definenition.
*/
/*----------------------------------*/
typedef struct {
    uint32_t TrimmingControl;       /*!< Value to TRIMEN */
    uint32_t TrimmingValue;         /*!< Value to TRIMSETC */
} TRM_ControlTypeDef;

typedef struct
{
    TSB_TRM_TypeDef           *p_instance;     /*!< Registers base address.           */
} trm_t;
/**
 *  @}
 */ /* End of group TRM_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_functions
 *  @{
 */

void SetTrmOsc(trm_t *p_obj, TRM_ControlTypeDef *ControlStruct);
uint32_t GetTrimmingControlValue(trm_t *p_obj);
uint32_t GetInitTrimValue(trm_t *p_obj);
uint32_t GetSetTrimValue(trm_t *p_obj);

/** 
 *  @}
 */ /* End of group TRM_Private_functions */

/**
 *  @}
 */ /* End of group TRM */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TRM_B_H */

